@setlocal
@echo off
rem Version mqe_java/source/resources/Windows/CreateBranchQM.bat, MQe, la200, la200-L030324.1 1.4
rem -------------------------------------------------------------------------
rem MQSeries Everyplace for Multiplatforms                
rem                                                                          
rem Licensed Materials - Property of IBM                                     
rem                                                                          
rem 5765-E63                                                                 
rem                                                                          
rem (C) Copyright IBM Corp. 2001                                             
rem 
rem -------------------------------------------------------------------------
rem 
rem A batch file to create a queue manager on a Branch machine.
rem 
rem Syntax: CreateBranchQM.bat branchNumber portListeningOn
rem 
rem Where the branchNumber is a 3-digit number, padded with leading zeros,
rem indicating which branch the queue manager is being created for.
rem For example, 000, 001, 002...etc.
rem
rem Where portListeningOn is a port on which the device branch queue manager
rem will be listening on for admin requests. 
rem For example, 8081, 8082...etc.
rem Note that the port must not already be in use.
rem Use "netstat -a" to view the list of ports in-use.
rem 
rem Note: The JavaEnv and MQE_BASE_DIR enviroment variables must be set.
rem 
rem -------------------------------------------------------------------------

rem Check that the first parameter is the branch number...
if .%1%.==..                             goto no_branch_number
set BRANCH_NUMBER=%1%

if .%2%.==..                             goto no_port_listening_on
set PORT_LISTENING_ON=%2%

rem Check that the java command is set.
if .%JavaCmd%.==..                       goto no_java_cmd      

rem Check that the MQE_BASE_DIR environment variable is set.
if .%MQE_BASE_DIR%.==..                  goto no_mqe_base_dir

rem Collect the details about the gateway queue manager.      
call GatewayQMDetails.bat

rem Check that the user has filled-in the correct details.
if .%GATEWAY_QMGR_ADDR%.==..             goto no_gateway_qmgr_addr
if .%GATEWAY_QMGR_PORT%.==..             goto no_gateway_qmgr_port

rem Collect the details about the central (MQSeries) queue manager...
call CentralQMDetails.bat

rem Check to make sure the details were filled-in.
if .%CENTRAL_QMGR_NAME%.==..             goto no_central_qmgr_name
if .%CENTRAL_QMGR_LISTENING_PORT%.==..   goto no_central_qmgr_listening_port
if .%CENTRAL_QMGR_HOSTNAME%.==..         goto no_central_qmgr_hostname
if .%TARGET_MQ_Q%.==..                   goto no_target_mq_q

rem Set up the name of the branch queue manager...
set QMGR_NAME=BRANCH%BRANCH_NUMBER%QM

rem Set up the base directory in which the queue manager will sit.
set QMGR_DIR=%MQE_BASE_DIR%\%QMGR_NAME%

rem What is the name of the MQe queue manager .ini file ?
set INI_FILE_NAME=%QMGR_DIR%\%QMGR_NAME%.ini 

rem Symbolics for making calling the command line tools easier to read.               
set BASE_PACKAGE=examples.administration.commandline
set BRIDGE_PAKCAGE=examples.mqbridge.administration.commandline

rem The security attributes we are going to use on all the queues
set COMPRESSOR=null
rem set CRYPTOR=com.ibm.mqe.attributes.MQeDESCryptor
set CRYPTOR=null
set AUTHENTICATOR=null

rem Some symbolics to make it clear when we're not using attributes.
set NO_COMPRESSOR=null
set NO_CRYPTOR=null
set NO_AUTHENTICATOR=null

rem The pin and password required to access the MQe queue manager's registry.                  
set PIN=123456789
set KEYRINGPASSWORD=mySecret

rem The queue manager rule for the store queue manager...
set QMGR_RULE=examples.rules.ExampleQueueManagerRules
                  
rem The name of the storage adapter which is used to store messages, and registry information.                           
rem When/if we have a mirroring disk fields adapter, then it's base-class name,
rem excluding the .class extension, but including the package name prefix,
rem can be specified here instead of "default".
rem Note: The mirroring adapter will only work on the 4690, and not the NT store machine.
set STORAGE_ADAPTER=com.ibm.mqe.adapters.MQeDiskFieldsAdapter
set STORAGE_LOCATION=%STORAGE_ADAPTER%:%QMGR_DIR%
                  
rem Delete the queue manager if it already exists...
if exist %QMGR_DIR% del /S /Q %QMGR_DIR%\*.*
if exist %QMGR_DIR% rmdir /S /Q %QMGR_DIR%                                            

rem Create the directory for the queue manager to live in...             
mkdir %QMGR_DIR%             

rem Create the store device .ini file for use when creating the queue manager...              
%JavaCmd% %BASE_PACKAGE%.IniFileCreator %INI_FILE_NAME% %QMGR_NAME% %QMGR_DIR%\Registry\ ^
Server %PORT_LISTENING_ON% %PIN% %KEYRINGPASSWORD% %STORAGE_ADAPTER% FileRegistry FastNetwork

rem Create the queue manager using the .ini file...
%JavaCmd% examples.install.SimpleCreateQM %INI_FILE_NAME% %QMGR_DIR%\Queues\
              
rem configure the device queue manager such that it has a connection to the gateway queue manager...
%JavaCmd% %BASE_PACKAGE%.ConnectionCreator  %GATEWAY_QMGR_NAME% %QMGR_NAME% ^
%GATEWAY_LISTENING_ADAPTER%:%GATEWAY_QMGR_ADDR%:%GATEWAY_QMGR_PORT% DefaultChannel %INI_FILE_NAME%

rem configure a further connection on the device queue manager such that it has a connection to the MQ queue manager
rem which is routed via the gateway queue manager.
%JavaCmd% %BASE_PACKAGE%.ConnectionCreator %CENTRAL_QMGR_NAME% %QMGR_NAME% %GATEWAY_QMGR_NAME% DefaultChannel %INI_FILE_NAME%

rem confure a remote asynchronous queue on the device queue manager which refers to the sales queue on the MQ system.
rem RemoteQueueCreator <queue_name> <hosting_queue_manager> <queue_manager_name> <ini_file>
rem * <queue_name>                    
rem *       is the name of the remote queue we are creating.
rem * <hosting_queue_manager>
rem *       is the name of the queue manager on which the queue actually exists.
rem * <queue_manager_name>          
rem *       is the name of the queue manager on which the remote queue 
rem *       definition is to be created on.
rem * <ini_file>
rem *       Name of the ini file used to open a client queue manager, to 
rem *       send the admin messages from.
%JavaCmd% %BASE_PACKAGE%.RemoteQueueCreator %TARGET_MQ_Q% %CENTRAL_QMGR_NAME% Asynchronous ^
%COMPRESSOR% %CRYPTOR% %AUTHENTICATOR% %QMGR_NAME% %INI_FILE_NAME% %STORAGE_LOCATION%

rem Create a home server queue to pull-down files from the central site...
rem HomeServerQueueCreator <queue_name> <get_from_qmgr> <poll_period> <changing_qmgr> <ini_file>
rem poll-period is in milliseconds. 30 seconds = 30*1000 
%JavaCmd% %BASE_PACKAGE%.HomeServerQueueCreator %GATEWAY_S_F_Q% %GATEWAY_QMGR_NAME% 30000 %QMGR_NAME% %INI_FILE_NAME%

rem Create a local queue which will store messages which are sent to this store from the
rem central site.
rem java examples.administration.commandline.LocalQueueCreator queue_name \
rem compressor cryptor authenticator maxMsgSize maxQDepth queue_manager_name ini_file
%JavaCmd% %BASE_PACKAGE%.LocalQueueCreator FromCentralQ %NO_COMPRESSOR% %NO_CRYPTOR% %NO_AUTHENTICATOR% ^
nolimit nolimit %QMGR_NAME% %INI_FILE_NAME% %STORAGE_LOCATION%

rem Set the device queue manager rule to a rule class which triggers transmission every 5 minutes.
rem Syntax: java QueueManagerUpdater rule qmgr iniFile
%JavaCmd% %BASE_PACKAGE%.QueueManagerUpdater %QMGR_RULE%  %QMGR_NAME% %INI_FILE_NAME%                                               
              
rem Send a simple test message to the MQ queue manager... to prove that we have connectivity...
%JavaCmd% %BASE_PACKAGE%.SendTestMessage %TARGET_MQ_Q% %CENTRAL_QMGR_NAME% %INI_FILE_NAME%              
                    
rem Kick-off the admin tool just so the device queue manager is kept alive...                                                          
%JavaCmd% examples.mqbridge.awt.AwtMQBridgeServer %INI_FILE_NAME%

rem We've finished the script...
goto ending
 
 

rem -------------------------------------------------------------------------
rem
rem Error cases...                                 
rem 
rem -------------------------------------------------------------------------
                                 
:no_central_qmgr_name
echo The CENTRAL_QMGR_NAME variable not set. 
echo Edit the CentralQMDetails.bat file
goto ending

:no_central_qmgr_listening_port
echo The CENTRAL_QMGR_LISTENING_PORT variable not set. 
echo Edit the CentralQMDetails.bat file
goto ending

:no_central_qmgr_hostname
echo The CENTRAL_QMGR_HOSTNAME variable not set. 
echo Edit the CentralQMDetails.bat file
goto ending

:no_target_mq_q
echo The TARGET_MQ_Q variable not set. 
echo Edit the CentralQMDetails.bat file
goto ending

:no_gateway_qmgr_addr
echo The GATEWAY_QMGR_ADDR variable not set. 
echo Edit the GatewayQMDetails.bat file. 
goto ending

:no_gateway_qmgr_port
echo The GATEWAY_QMGR_PORT variable not set. 
echo Edit the GatewayQMDetails.bat file. 
goto ending

:no_branch_number
echo No branch number specified !
echo This should be the 3-digit number of the branch.
goto ending

:no_port_listening_on
echo No port number specified !
echo This should be a numeric value indicating the port the MQe queue manager
echo will be listening on.
echo For example, 8081, 8082...etc.
echo Note that the port must not already be in use.
echo Use "netstat -a" to view the list of ports in-use
goto ending

:no_java_cmd
echo No JavaCmd environment variable set up.
echo Please update and run the JavaEnv.bat file, then retry.
goto ending

:no_mqe_base_dir
echo No MQE_BASE_DIR enviroment variable set up.
echo Please update and run the JavaEnv.bat file, then retry.
goto ending
 

rem -------------------------------------------------------------------------
rem 
rem we're done, so leave this script.       
rem 
rem -------------------------------------------------------------------------
:ending            
@endlocal
