// Version mqe_java/source/examples/adapters/WESAuthenticationGUIAdapter.java, MQeExamples, la200, la200-L030324.1 1.6

/* -----------------------------------------------------------------------------*/ 

/*                   WebSphere MQ Everyplace for Multiplatforms                 */ 

/*                                                                              */ 

/* Licensed Materials - Property of IBM                                         */ 

/*                                                                              */ 

/* 5765-E63                                                                     */ 

/*                                                                              */ 

/* (C) Copyright IBM Corp. 1999                                                 */ 

/* -----------------------------------------------------------------------------*/

package examples.adapters;


import  com.ibm.mqe.*;
import  com.ibm.mqe.adapters.*;
import  examples.awt.*;


/******************************************************************************** 
 * WESAuthenticationGUIAdapter extends the MQeWESAuthenticationdapter by popping up
 * a dialog box whenever authentication is required. (i.e. on the first write,
 * or if the last write failed due to an authentication failure).
 *
 * The user can then click on 'OK' if he/she wishes to continue the request
 * with the credentials supplied, or 'Cancel' if he/she wishes to, errr..
 * cancel the operation.
 *
 * @see com.ibm.mqe.adapters.MQeWESAuthenticationAdapter
 ********************************************************************************/

public class WESAuthenticationGUIAdapter extends MQeWESAuthenticationAdapter {

    /**
     * Version string
     */
    public static short  version[] = { 2, 0, 0, 5 }; 

    /**
     * Is set to true if we need to get hold of authorization information
     * on the next write to the server.
     */
    private boolean authorizationNeeded = true;

    /**
     * Store the user's entry for the userid field away, so that we can
     * use it as a default value for any subsequent dialog boxes.
     */
    private static String userid = "";

    /**
     * Brings up a dialog box that asks the user for authorization information.
     * @param none 
     * @return void
     * @exception MQeException  Except_Stopped if the cancel button is pressed.
     * @exception Exception  If the AwtDialog couldn't been created.
     */
    private void promptForAuthorization() throws Exception {
        String password = null;
        String promptString;

        // If the user didn't supply the realm part of the user id (that is,
        // the 'realm' part of 'userid@realm') then look to see if we already
        // know what the realm should be. If we do, then append it
        // to the userid string.
        if (userid != null) {
            if (userid.indexOf("@") == -1) {
                String realm = MQeWESAuthenticationAdapter.getRealm();

                if (realm != null) {
                    if (!realm.equals("")) {
                        userid = userid.concat("@" + realm);
                    }
                }
            }
        }

        // Create the dialog box in which the user can enter the login info.
        AwtDialog loginBox = new AwtDialog("WES Authenticator",
                AwtDialog.Show_OK_Cancel,
                new String[][][]
                { { { "L", "Userid"   /* Index 0 */}, { "T", userid     /* Index 1 */}
                    },
                    { { "L", "Password" /* Index 2 */}, { "T*", ""         /* Index 3 */}
                    }
                }
            );

        // If the user clicked on OK, then set the authorization information
        // in the "real" WES Adapter.
        if (loginBox.getActionIndex(loginBox.South) == loginBox.Button_OK) {
            userid = loginBox.getText(AwtDialog.Center, 1);
            password = loginBox.getText(AwtDialog.Center, 3);
            loginBox.dispose();
            setBasicAuthorization(userid, password);
            authorizationNeeded = false;
        } // If the user clicked on Cancel, don't set the authorization
        // information and tell the caller that the user bailed out.
        else {
            loginBox.dispose();
            throw new MQeException(MQeException.Except_Stopped, "Canceled");
        }
    }

    /**
     * Intercept any calls to the write() method, in order to
     * pop up the dialog box if required. If the user clicks on
     * the cancel button in the dialog box, then don't do
     * the write.
     * @param opt Options to supply to the write()
     * @param recordSize the number of bytes to write
     * @param data[] the data to write
     * @return void
     * @exception MQeException - Except_Stopped if user wants to 
     * cancel the operation, or another MQeException if the
     * underlying write fails.
     */
    public synchronized void write(Object opt, int recordSize, byte data[])
        throws Exception {
        if (authorizationNeeded) {
            promptForAuthorization(); // Throws MQeException if cancel clicked
        }
        super.writeData(data);

    }

    /**
     * Intercept any data coming back from WES on a read().
     * Look to see if the read was accepted by the server.
     * If not, check to see if it was because our authentication
     * was invalid. If so, we need to redisplay the dialog box
     * on the next write.
     * @param none
     * @return byte[] The data read from the adapter
     * @exception Exception If the adapter read failed
     */
    public synchronized byte[] read() throws Exception { 

        byte[] response = null;

        try {
            response = super.read();
        } catch (MQeException e) {

            if (e.code() == MQeException.Except_Authenticate) {
                authorizationNeeded = true;
            }

            throw e;
        }

        return (response);
    }

}
