//Version mqe_java/source/examples/administration/commandline/QueueManagerUpdater.java, MQe, la200, la200-L030414.1 1.8

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.administration.commandline;


import com.ibm.mqe.*; 
import com.ibm.mqe.administration.*; 
import examples.queuemanager.*;
 
                
/**
 * To start a queue manager and add a local queue to its configuration.
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.administration.commandline.QueueManagerUpdater 
 * &lt;rule&gt;
 * &lt;queue_manager_name&gt;
 * &lt;ini_file&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * <DT>rule</DT>
 * <DD>Is the name of the queue manager rule. This is a class name, including the
 * .class file extension, or an MQe alias which resolves to a class name.</DD>
 * <DT>queue_manager_name</DT>
 * <DD>is the name of the queue manager on which is being updated</DD>
 * <DT>ini_file</DT>
 * <DD>Name of the ini file used to open a client queue manager, to
 * send the admin messages from.</DD>
 * </DL>
 */
public class QueueManagerUpdater extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public static short[] version = { 2, 0, 0, 5 }; 
    static final String syntax =
        "Syntax :\n"
        + "QueueManagerUpdater <rule> <queue_manager_name> <ini_file>\n"
        + "Parameters:\n"
        + "<rule> is the name of the new queue manager rule"
        + "<queue_manager_name>\n"
        + "     is the name of the queue manager on which the remote queue\n"
        + "     definition is to be created on.\n"
        + "<ini_file>\n"
        + "     Name of the ini file used to open a client queue manager, to\n"
        + "     send the admin messages from.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        QueueManagerUpdater me = new QueueManagerUpdater();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gets command-line parameters into variables, and calls the method which constructs
     * and sends the admin message.
     * 
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        String qMgr = null;     // Name of the queue manager which is to be updated
        String qMgrRule = null; // Name of the new queue manager rule.
        String iniFile = null;      // Name of the ini file used to open a client queue manager, to 

        if (args.length < 3) {
            throw new Exception("Not enough parameters !\n" + syntax);
        }

        qMgrRule = args[0].trim();
        qMgr = args[1].trim();
        iniFile = args[2].trim();

        System.out.println("i: Attempting to update queue manager " + qMgr
            + " using the " + iniFile + " ini file."
            + " new rule=" + qMgrRule
        );
        
        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true // Suppress stdout information
            );

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = MQeClient.queueManager;

        updateQueueManager(qMgr, qMgrRule, myQMgr);

        // Close down the MQeClient
        client.close();
    } 

    /**
     * Creates an admin message which will create a queue, and sends it
     * to the queue manager.
     * 
     * @param qMgr   The queue manager whose details are to be updated.
     * 
     * @param qMgrRule The rule class which will be set into the queue manager configuration.
     * 
     * @param myQMgr A reference to an active MQe queue manager which will be used to send and
     *               receive MQe administration messages and responses.
     * @return void 
     * @exception Exception
     */
    static public void updateQueueManager(String qMgr, String qMgrRule, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to update the queue manager "
            + " on queue manager " + qMgr + " rule=" + qMgrRule
        );

        // Construct the AdminMsg...

        MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg(qMgr);              

        // Create a parameters MQeFields object...
        MQeFields params = new MQeFields();

        // Populate the parameters MQeFields object with any values you want 
        // the new Remote Queue to be created with...
        // ... using putXXXX( fieldName , fieldValue ) calls

        params.putAscii(MQeQueueManagerAdminMsg.QMgr_Rules, qMgrRule);

        // Say this admin message should create the queue...
        msg.update(params);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, qMgr, msg);

        System.out.println("Queue manager updated successfully.");
    } 
} 
