// Version mqe_java/source/examples/administration/console/AdminCommsListener.java, MQeExamples, la200, la200-L030324.1 1.2

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.communications.*;
import examples.awt.*;
import java.awt.*;


/***
 *
 * <p>Specialisation of AdminFrame allowing for administration of a
 * communications listeners.
 *
 **/
public class AdminCommsListener extends AdminFrame {
    public static short[] version = { 2, 0, 0, 5 }; 
    // Additional fields that are not part of the admin message itself.

    /**
     * Default constructor.
     */
    public AdminCommsListener() {
        super("Comms Listener Administration");
    }

    /**
     * Main constructor.
     *
     * @param  msg An administration message to contain the parameters for the action.
     * If it already contains any parameters then these are taken as defaults.
     * @param action  The action to be performed
     * @param adminModel The administration model which will send the message.
     * @exception java.lang.Exception
     */
    public AdminCommsListener(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {
        super(msg, action, adminModel);
    }

    /**
     * AdminQueueManager constructor.
     * @param title java.lang.String
     * @exception java.lang.Exception
     */
    public AdminCommsListener(String title) throws Exception {
        super(title);
    }

    /**
     * Add any aliases related to this resource.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    public void addAlias() throws Exception {
        MQe.alias("MQeCommunicationsListenerAdminMsgGUI",
            "examples.administration.console.AdminCommsListener");

    }

    /**
     * Returns an actions panel.
     * @param eventHandler An AwtEvent
     * @return A Panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        // ..Comms listener actions
        AwtFormat actionF = new AwtFormat(eventHandler, East);

        return  actionF.panel(new Panel(), new String[][][]
                { { { "B", "Inquire"}
                    },    // 0
                    { { "B", "Create"}
                    },    // 1
                    { { "B", "Update"}
                    },    // 2
                    { { "B", "Delete"}
                    },    // 3
                    { { "B", "Start"}
                    },    // 4
                    { { "B", "Stop"}
                    }     // 5
                }
            );

    }

    /**
     * Get the set of characteristics for the managed object.
     * @param none
     * @return MQeFields Object
     * @exception java.lang.Exception
     */
    public MQeFields getCharacteristics() throws Exception {
        MQeFields fields = msg.characteristics();

        // The protocol parameters are specific to each adapter and
        // hence are not part of the listeners generic characteristics.
        // Add into the characteristics those parameters that are appropriate
        // for a tcpip based listener.
        MQeFields prot = fields.getFields(MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL);

        prot.putUnicode(MQeCommunicationsAdapter.COMMS_ADAPTER_CLASS, "");
        prot.putUnicode(MQeCommunicationsAdapter.COMMS_ADAPTER_PORT, "");
        return fields;
    }

    /**
     * Return the name of resource being managed.
     * @param none
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return "Comms Listener";
    }

    /**
     * Process a comms listener action request.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A Boolean
     * @return void
     * @exception java.lang.Exception
     */
    public void resourceAction(Object e, int where, int index, String  choice, boolean state) throws Exception {
        int act = -1;       // Unknown action

        // Decide on which action user selected ?
        switch (index) {
        case 0:
            act = MQeCommunicationsListenerAdminMsg.Action_InquireAll;
            break;

        case 1:
            act = MQeCommunicationsListenerAdminMsg.Action_Create;
            break;

        case 2:
            act = MQeCommunicationsListenerAdminMsg.Action_Update;
            break;

        case 3:
            act = MQeCommunicationsListenerAdminMsg.Action_Delete;
            break;

        case 4:
            act = MQeCommunicationsListenerAdminMsg.Action_Start;
            break;

        case 5:
            act = MQeCommunicationsListenerAdminMsg.Action_Stop;
            break;

        }

        // Display the message in a window
        activate(new MQeCommunicationsListenerAdminMsg(adminModel.getManageQM()),
            act,
            adminModel);

    }

    /**
     * Setup the admin message with the action to be performed and the
     * parameters required by the action.
     * @param parms MQeFields Object
     * @return MQeAdminMsg Object
     * @exception java.lang.Exception
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {
        switch (new Integer(action).intValue()) {
        case MQeCommunicationsListenerAdminMsg.Action_InquireAll:
            // Inquire
            ((MQeCommunicationsListenerAdminMsg) msg).inquireAll(parms);
            break;

        case MQeCommunicationsListenerAdminMsg.Action_Update:
            ((MQeCommunicationsListenerAdminMsg) msg).update(parms);
            break;

        case MQeCommunicationsListenerAdminMsg.Action_Create:
            ((MQeCommunicationsListenerAdminMsg) msg).create(parms);
            break;

        case MQeCommunicationsListenerAdminMsg.Action_Delete:
            ((MQeCommunicationsListenerAdminMsg) msg).delete(parms);
            break;

        case MQeCommunicationsListenerAdminMsg.Action_Start:
            // Start listener
            ((MQeCommunicationsListenerAdminMsg) msg).setName(parms.getAscii(MQeCommunicationsListenerAdminMsg.Admin_Name));
            msg.setAction(MQeCommunicationsListenerAdminMsg.Action_Start);
            break;

        case MQeCommunicationsListenerAdminMsg.Action_Stop:
            // Stop listener
            ((MQeCommunicationsListenerAdminMsg) msg).setName(parms.getAscii(MQeCommunicationsListenerAdminMsg.Admin_Name));
            msg.setAction(MQeCommunicationsListenerAdminMsg.Action_Stop);
            break;
        }
        return (MQeCommunicationsListenerAdminMsg) msg;
    }

    /**
     * Configure the set of actions.
     * @param none
     * @return void
     */
    protected void setupActions() {
        super.setupActions();

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_InquireAll),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"}
            }
        );

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_Create),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_DESCRIPTION, "O"},
                { MQeCommunicationsAdapter.COMMS_ADAPTER_CLASS, "M"},
                { MQeCommunicationsAdapter.COMMS_ADAPTER_PORT, "M"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_MAXIMUM_CHANNELS, "O"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_CHANNEL_TIMEOUT, "O"},
            }
        );

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_Update),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_DESCRIPTION, "O"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_MAXIMUM_CHANNELS, "O"},
                { MQeCommunicationsListenerAdminMsg.LISTENER_CHANNEL_TIMEOUT, "O"},
            }
        );

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_Delete),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"},
            }
        );

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_Start),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"}
            }
        );

        addAction(new String().valueOf(MQeCommunicationsListenerAdminMsg.Action_Stop),
            new String[][] {
                { MQeCommunicationsListenerAdminMsg.Admin_Name, "M"}
            }
        );
    }

    /**
     * Setup the valid set of characteristics.
     * The sequence is important as this is the sequence that the fields
     * will displayed.
     * @param none
     * @return void
     */
    protected void setupFields() {
        super.setupFields();

        addCharacteristic(MQeCommunicationsListenerAdminMsg.Admin_Name, "TF", null);
        addCharacteristic(MQeCommunicationsListenerAdminMsg.LISTENER_DESCRIPTION, "TF", null);
        addCharacteristic(MQeCommunicationsListenerAdminMsg.LISTENER_MAXIMUM_CHANNELS, "TF", null);
        addCharacteristic(MQeCommunicationsListenerAdminMsg.LISTENER_RUNNING, "TF", null);
        addCharacteristic(MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL, "TF", null);
        addCharacteristic(MQeCommunicationsListenerAdminMsg.LISTENER_CHANNEL_TIMEOUT, "TF", null);
        addCharacteristic(MQeCommunicationsAdapter.COMMS_ADAPTER_CLASS, "TF", MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL);
        addCharacteristic(MQeCommunicationsAdapter.COMMS_ADAPTER_PORT, "TF", MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL);
        addCharacteristic(MQeCommunicationsAdapter.COMMS_ADAPTER_LISTEN, "TF", MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL);
    }

    /**
     * Setup NLS names.
     * @param none
     * @return void
     */
    protected void setupNLS() {
        super.setupNLS();

        addNLSName(MQeCommunicationsListenerAdminMsg.Admin_Name, "Listener name");
        addNLSName(MQeCommunicationsListenerAdminMsg.LISTENER_DESCRIPTION, "Description");
        addNLSName(MQeCommunicationsListenerAdminMsg.LISTENER_MAXIMUM_CHANNELS, "Maximim Channels");
        addNLSName(MQeCommunicationsListenerAdminMsg.LISTENER_RUNNING, "Running");
        addNLSName(MQeCommunicationsListenerAdminMsg.LISTENER_PROTOCOL, "Protocol");
        addNLSName(MQeCommunicationsListenerAdminMsg.LISTENER_CHANNEL_TIMEOUT, "Channel Timeout");
        addNLSName(MQeCommunicationsAdapter.COMMS_ADAPTER_CLASS, "Communications adapter");
        addNLSName(MQeCommunicationsAdapter.COMMS_ADAPTER_PORT, "Listen on Port");
        addNLSName(MQeCommunicationsAdapter.COMMS_ADAPTER_LISTEN, "Listen?");

        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_InquireAll), "Inquire");
        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_Update), "Update");
        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_Create), "Add Comms Listener");
        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_Delete), "Remove Comms Listener");
        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_Start), "Start");
        addNLSName((new String()).valueOf(MQeCommunicationsListenerAdminMsg.Action_Stop), "Stop");
        addNLSName("cancel", "Cancel");

    }

    /**
     * Returns a string that provides information about the current frame.
     * This is displayed in the south panel of the main admin gui when a frame is selected.
     * @param none
     * @return String documenting the current frame.
     */
    public String getMessageHelp() {
        return "A communications listener is handles incoming communication requests.";
    }
}
