// Version mqe_java/source/examples/administration/console/AdminConnection.java, MQeExamples, la200, la200-L030324.1 1.9

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.awt.*;
import java.awt.*;


/***
 * 
 * <p>Specialisation of AdminFrame allowing for administration of a           
 * a queue manager.
 *
 **/

public class AdminConnection extends AdminFrame {
    public          static short version[] = { 2, 0, 0, 5 }; 

    // Additional fields that are not part of the admin message itself.

    // Alias name
    public final static String     Con_Alias = "aliasn";

    /**
     * Default constructor.
     */
    public AdminConnection() {
        super("Connection Administration");
    }

    /**
     * Main constructor.
     *
     * @param  msg An administration message to contain the parameters for the action. 
     * If it already contains any parameters then these are taken as defaults.
     * @param action  The action to be performed
     * @param adminModel The administration model which will send the message.
     * @exception java.lang.Exception
     */
    public AdminConnection(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {
        super(msg, action, adminModel);
    }

    /**
     * AdminQueueManager constructor.
     * @param title java.lang.String
     * @exception java.lang.Exception
     */
    public AdminConnection(String title) throws Exception {
        super(title);
    }

    /**
     * Add any aliases related to this resource.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    public void addAlias() throws Exception {
        MQe.alias("MQeConnectionAdminMsgGUI",
            "examples.administration.console.AdminConnection");

    }

    /**
     * Returns an actions panel.
     * @param eventHandler An AwtEvent
     * @return A Panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        // ..Remote queue manager definition actions
        AwtFormat actionF = new AwtFormat(eventHandler, East);

        return  actionF.panel(new Panel(), new String[][][]
                { { { "B", "Inquire"}
                    },    // 0
                    { { "B", "Create"}
                    },    // 1
                    { { "B", "Update"}
                    },    // 2
                    { { "B", "Delete"}
                    },    // 3
                    { { "B", "Add Alias"}
                    },    // 4
                    { { "B", "Remove Alias"}
                    }     // 5
                }
            );

    }

    /**
     * Get the set of characteristics for the managed object.
     * @param none
     * @return MQeFields Object
     * @exception java.lang.Exception
     */
    public MQeFields getCharacteristics() throws Exception {
        MQeFields fields = msg.characteristics();

        fields.putAscii(Con_Alias, null);

        return fields;
    }

    /**
     * Return the name of resource being managed.
     * @param none
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return "Connection";
    }

    /**
     * Process a queue manager action request.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A Boolean
     * @return void
     * @exception java.lang.Exception
     */
    public void resourceAction(Object e, int where, int index, String  choice, boolean state) throws Exception {
        int act = -1;       // Unknown action

        // Decide on which action user selected ?
        switch (index) {
        case 0:
            act = MQeConnectionAdminMsg.Action_InquireAll;
            break;

        case 1:
            act = MQeConnectionAdminMsg.Action_Create;
            break;

        case 2:
            act = MQeConnectionAdminMsg.Action_Update;
            break;

        case 3:
            act = MQeConnectionAdminMsg.Action_Delete;
            break;

        case 4:
            act = MQeConnectionAdminMsg.Action_AddAlias;
            break;

        case 5:
            act = MQeConnectionAdminMsg.Action_RemoveAlias;
            break;

        }

        // Display the message in a window
        activate(new MQeConnectionAdminMsg(adminModel.getManageQM()),
            act,
            adminModel);

    }

    /**
     * Setup the admin message with the action to be performed and the
     * parameters required by the action.
     * @param parms MQeFields Object
     * @return MQeAdminMsg Object
     * @exception java.lang.Exception
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {
        switch (new Integer(action).intValue()) {
        case MQeConnectionAdminMsg.Action_InquireAll:
            // Inquire
            ((MQeConnectionAdminMsg) msg).inquireAll(parms);
            break;

        case MQeConnectionAdminMsg.Action_Update:
        case MQeConnectionAdminMsg.Action_Create:
            // Add or Update a simple connection definition
            // A simple connection only has one adapter. Complex connections
            // allow more than one adapter to be specified.
            // Note: This example only allows one adapter to be specified.
            ((MQeConnectionAdminMsg) msg).setName(parms.getAscii(MQeConnectionAdminMsg.Admin_Name));

            // Get the default values
            MQeFields defaultAttrs = msg.characteristics();
            MQeFields[] defaultAdap = defaultAttrs.getFieldsArray(MQeConnectionAdminMsg.Con_Adapters);

            String desc = defaultAttrs.getUnicode(MQeConnectionAdminMsg.Con_Description);
            String channel = defaultAttrs.getAscii(MQeConnectionAdminMsg.Con_Channel);
            String adapter = defaultAdap[0].getAscii(MQeConnectionAdminMsg.Con_Adapter);
            String parameters = defaultAdap[0].getAscii(MQeConnectionAdminMsg.Con_AdapterAsciiParm);
            String options = defaultAdap[0].getAscii(MQeConnectionAdminMsg.Con_AdapterOptions);

            // Use user provided description if set otherwise use default.
            if (parms.contains(MQeConnectionAdminMsg.Con_Description))
                desc = parms.getUnicode(MQeConnectionAdminMsg.Con_Description);

            // Use user provided channel if set otherwise use default.
            if (parms.contains(MQeConnectionAdminMsg.Con_Channel))
                channel = parms.getAscii(MQeConnectionAdminMsg.Con_Channel);

            // Use user provided adapter if set otherwise use default.
            if (parms.contains(MQeConnectionAdminMsg.Con_Adapters)) {
                MQeFields adapters = parms.getFields(MQeConnectionAdminMsg.Con_Adapters);

                // Default if no adapter specified.
                if (adapters.contains(MQeConnectionAdminMsg.Con_Adapter)) {
                    adapter = adapters.getAscii(MQeConnectionAdminMsg.Con_Adapter);
                    if (adapters.contains(MQeConnectionAdminMsg.Con_AdapterAsciiParm))
                        parameters = adapters.getAscii(MQeConnectionAdminMsg.Con_AdapterAsciiParm);
                    if (adapters.contains(MQeConnectionAdminMsg.Con_AdapterOptions))
                        options = adapters.getAscii(MQeConnectionAdminMsg.Con_AdapterOptions);
                }
            }
            // Rather than directly updating the msg object helper methods
            // will be used to create and update a simple connection definition.
            if (new Integer(action).intValue() == MQeConnectionAdminMsg.Action_Create)
                // Create a connection
                ((MQeConnectionAdminMsg) msg).create(
                    adapter,
                    parameters,
                    options,
                    channel,
                    desc);
            else
                // Update a connection
                ((MQeConnectionAdminMsg) msg).update(
                    adapter,
                    parameters,
                    options,
                    channel,
                    desc);
            break;

        case MQeConnectionAdminMsg.Action_Delete:
            // Remove a remote queue manager definition
            ((MQeConnectionAdminMsg) msg).delete(parms);
            break;

        case MQeConnectionAdminMsg.Action_AddAlias:
            // Add an alias
            ((MQeConnectionAdminMsg) msg).setName(parms.getAscii(MQeConnectionAdminMsg.Admin_Name));
            ((MQeConnectionAdminMsg) msg).addAlias(parms.getAscii(AdminConnection.Con_Alias));
            break;

        case MQeConnectionAdminMsg.Action_RemoveAlias:
            // Remove an alias
            ((MQeConnectionAdminMsg) msg).setName(parms.getAscii(MQeConnectionAdminMsg.Admin_Name));
            ((MQeConnectionAdminMsg) msg).removeAlias(parms.getAscii(AdminConnection.Con_Alias));
            break;
        }
        return (MQeConnectionAdminMsg) msg;
    }

    /**
     * Configure the set of actions.
     * @param none
     * @return void
     */
    protected void setupActions() {
        super.setupActions();

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_InquireAll),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"}
            }
        );

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_Create),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"},
                { MQeConnectionAdminMsg.Con_Description, "O"},
                { MQeConnectionAdminMsg.Con_Channel, "O"},
                { MQeConnectionAdminMsg.Con_Adapter, "O"},
                { MQeConnectionAdminMsg.Con_AdapterAsciiParm, "O"},
                { MQeConnectionAdminMsg.Con_AdapterOptions, "O"},
            }
        );

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_Update),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"},
                { MQeConnectionAdminMsg.Con_Description, "O"},
                { MQeConnectionAdminMsg.Con_Channel, "O"},
                { MQeConnectionAdminMsg.Con_Adapter, "O"},
                { MQeConnectionAdminMsg.Con_AdapterAsciiParm, "O"},
                { MQeConnectionAdminMsg.Con_AdapterOptions, "O"},
            }
        );

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_Delete),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"},
            }
        );

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_AddAlias),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"},
                { AdminConnection.Con_Alias, "M"}
            }
        );

        addAction(new String().valueOf(MQeConnectionAdminMsg.Action_RemoveAlias),
            new String[][] {
                { MQeConnectionAdminMsg.Admin_Name, "M"},
                { AdminConnection.Con_Alias, "M"}
            }
        );
    }

    /**
     * Setup the valid set of characteristics.
     * The sequence is important as this is the sequence that the fields
     * will displayed.
     * @param none
     * @return void
     */
    protected void setupFields() {
        super.setupFields();

        addCharacteristic(MQeConnectionAdminMsg.Admin_Name, "TF", null);
        addCharacteristic(MQeConnectionAdminMsg.Con_Description, "TF", null);
        addCharacteristic(MQeConnectionAdminMsg.Con_Channel, "TF", null);
        addCharacteristic(MQeConnectionAdminMsg.Con_Adapters, "C", null);
        addCharacteristic(MQeConnectionAdminMsg.Con_Adapter, "TF", MQeConnectionAdminMsg.Con_Adapters);
        addCharacteristic(MQeConnectionAdminMsg.Con_AdapterAsciiParm, "TF", MQeConnectionAdminMsg.Con_Adapters);
        addCharacteristic(MQeConnectionAdminMsg.Con_AdapterOptions, "TF", MQeConnectionAdminMsg.Con_Adapters);
        addCharacteristic(MQeConnectionAdminMsg.Con_Aliases, "C", null);
        addCharacteristic(AdminConnection.Con_Alias, "TF", null);

    }

    /**
     * Setup NLS names.
     * @param none
     * @return void
     */
    protected void setupNLS() {
        super.setupNLS();

        addNLSName(MQeConnectionAdminMsg.Admin_Name, "Target QMgr");
        addNLSName(MQeConnectionAdminMsg.Con_Description, "Description");
        addNLSName(MQeConnectionAdminMsg.Con_Channel, "Channel *");
        addNLSName(MQeConnectionAdminMsg.Con_Adapters, "Adapters");
        addNLSName(MQeConnectionAdminMsg.Con_Adapter, "Network adapter");
        addNLSName(MQeConnectionAdminMsg.Con_AdapterAsciiParm, "Network adapter parms *");
        addNLSName(MQeConnectionAdminMsg.Con_AdapterOptions, "Network adapter options *");
        addNLSName(MQeConnectionAdminMsg.Con_Aliases, "Aliases");

        addNLSName(AdminConnection.Con_Alias, "Alias");

        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_InquireAll), "Inquire");
        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_Update), "Update");
        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_Create), "Add connection");
        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_Delete), "Remove connection");
        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_AddAlias), "Add alias");
        addNLSName((new String()).valueOf(MQeConnectionAdminMsg.Action_RemoveAlias), "Remove alias");
        addNLSName("cancel", "Cancel");

    }

    /**
     * Returns a string that provides information about the current frame.
     * This is displayed in the south panel of the main admin gui when a frame is selected.
     * @param none
     * @return String documenting the current frame.
     */
    public String getMessageHelp() {
        return "Connections tell the current Queue Manager how to connect to other Queue Managers.";
    }
}
