// Version mqe_java/source/examples/administration/console/AdminFrame.java, MQeExamples, la200, la200-L030324.1 1.11

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import java.awt.*;
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import java.util.*;
import examples.awt.*;


/**
 * <p>To provide a simple window interface for managing WebSphere MQ Everywhere
 * resources. Any resource to be managed should be implemented as a
 * subclass of this class.
 *
 **/

public abstract class AdminFrame extends AwtFrame {
    public          static short version[] = { 2, 0, 0, 5 }; 

    boolean active = false;                 // First pass ?

    /**
     *  Characteristic names displayed in West panel
     *  Characteristic value displayed in center panel
     *  Characteristic erros displayed in east panel
     *  Actions displayed in south panel
     */
    Panel   labelP = new Panel();
    Panel   valueP = new Panel();
    Panel   errorP = new Panel();
    Panel   actionP = new Panel();

    /**
     *  A subset of the GUI widgets
     */
    Label messageL;

    protected Button cancelB;
    protected Button applyB;
    protected Button actionB;

    /**
     * Information to help display resource
     */
    protected Hashtable nLSFields = new Hashtable();    // NLS fields
    protected Hashtable actionFields = new Hashtable();    // actions
    protected Hashtable displayFields = new Hashtable();    // characteristics
    protected Hashtable displayFieldsGroups = new Hashtable();    // characteristic groups
    protected Vector    displayFieldsSeq = new Vector();       // display chars in seq

    protected Hashtable displayedFields = new Hashtable();    // fields displayed
    protected int       fieldCount = 0;      // Number of fields displayed

    /**
     * All admin requests are stored in admin messages
     */
    protected String        action;         // Action to perform or that was performed
    protected MQeAdminMsg   msg;            // Admin message
    protected MQeAdminMsg   origMsg;        // Copy of the resource admin message
    protected AdminModel    adminModel;     // admin model to process message

    /**
     * Default constructor.
     */
    public AdminFrame() {
        super("Admin Frame");
    }

    /**
     * AdminFrame constructor.
     * @param msg An administration message to contain the parameters for the action.
     * If it already contains any parameters then these are taken as defaults.
     * @param action The action to be performedr
     * @param adminModel The administration model which will send the message.
     * @exception java.lang.Exception
     */
    public AdminFrame(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {
        this("Admin frame");
        activate(msg, action, adminModel);
    }

    /**
     * AdminFrame constructor.
     * @param title java.lang.String
     */
    public AdminFrame(String title) {
        super(title);
    }

    /**
     * Called to process an MQe awt event.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A boolean
     * @return void
     */
    public void action(Object e, int where, int index, String  choice, boolean state) {
        try {
            switch (where) {
            case South:
                // clear last message
                message("");

                // If OK or apply button then process action
                if (e == actionB || e == applyB) {
                    // Check input is valid
                    if (!validateInput())
                        break;

                    // set the admin message up and then send it
                    MQeAdminMsg compMsg = setupAction(getChanges());

                    adminModel.sendRequest(compMsg);

                    // Either close window or allow window to be reused.
                    if (e == applyB)
                        // If apply then need to reset message to original state
                        // as it may have been changed if running locally.
                        msg = origMsg.duplicate();
                }

                if (e == cancelB || e == actionB)
                    // If the OK or cancel button then close window/cleanup
                    close();

                break;
            }
        } catch (Exception ex) {
            message(ex.toString());
        }
    }

    /**
     * Initialisation code.
     * @param msg com.ibm.mqe.MQeAdminMsg
     * @param action  The action to be performed
     * @param adminModel The administration model which will send the message.
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {

        /**
         * Store input parameters locally
         */
        this.action = new String().valueOf(action); // Action to be performed
        this.msg = msg;                          // The message to process
        this.adminModel = adminModel;                   // admin model to process msg

        /**
         * Keep a copy of original message.
         */
        origMsg = msg.duplicate();          // Copy all fields (shallow copy)

        if (active) {

            /**
             * The frame is active and will be reused.
             * first it needs to be tidied up
             */

            // Remove all widgets
            labelP.removeAll();
            errorP.removeAll();
            valueP.removeAll();
            actionP.removeAll();

            // Clear the displayed fields store
            displayedFields.clear();

            // Reset number of fields displayed
            fieldCount = 0;
        } else {

            /**
             * First pass processing
             */

            // Setup message output panel
            format(North, new String[][][]             // 0
                { { { "L", ""}
                    }
                }
            );

            messageL = (java.awt.Label) getObject('N', 0);

            // set Grid layout rows and columns
            GridLayout Grid = new GridLayout();        // want a grid layout

            // Each panel has one coloum
            labelP.setLayout(Grid);
            valueP.setLayout(Grid);
            errorP.setLayout(Grid);

            // Add panels to frame
            setPanel(West, labelP);   // Label panel
            setPanel(Center, valueP);   // Characteristic values panel
            setPanel(East, errorP);   // Characteristic errors panel

            // Setup GUI related items for the resource to be managed
            setupFields();                  // Setup resource characteristics
            setupNLS();                     // Setup NLS names
            setupActions();                 // Setup resrouce actions

            // First pass complete.
            active = true;
        }

        String actionType = "new";

        /**
         * The message could be:
         * - request to setup a new admin command
         * - a completed admin command message
         * - a message containing the results of processing and admin command message
         */
        if (msg.contains(MQe.Msg_Style)) {

            /**
             * its either a completed request or response ...
             *   so display the fields that are contained in the message
             */

            // Iterate through characteristics in the sequence they should be displayed
            Enumeration fieldSeq = displayFieldsSeq.elements();

            while (fieldSeq.hasMoreElements()) {
                String name = (String) fieldSeq.nextElement();

                // If the message contains this field then display it
                if (msg.getOutputFields().contains(name)) {
                    addFieldToDisplay(name, msg.getOutputFields(), msg.getErrorFields());
                }
            }

            // Display the result if it is a response message
            if (msg.contains(MQeAdminMsg.Admin_Reason))
                message(msg.getReason());

            // As it is a completed message then only allow the cancel action
            addActionToDisplay(null);

            switch (msg.getInt(MQe.Msg_Style)) {
            case MQeAdminMsg.Msg_Style_Request:
                actionType = "request";
                break;

            case MQeAdminMsg.Msg_Style_Reply:
                actionType = "response";
                break;

            default:
                actionType = "datagram";
                break;
            }
        } else {

            /**
             * Its a new request ...
             *   so display the fields required for the requested action
             */
            message("Enter value(s) in field(s) to be changed (* advanced options)");

            // Get all fields so that field types can be determined
            MQeFields   allFields = getCharacteristics();

            // display all fields for this action
            String[][]  fieldsForAction = getFieldsForAction(this.action);

            for (int i = 0; i < fieldsForAction.length; i++) {
                MQeFields fld = allFields;

                if (msg.getInputFields().contains(fieldsForAction[i][0])) {
                    // A default has been provided, so use it
                    fld = msg.getInputFields();
                }
                addFieldToDisplay(fieldsForAction[i][0], fld, null);
            }

            // Add the action button to the display
            addActionToDisplay(this.action);
        }

        // Set the frame title
        setTitle(getNLSName(this.action) + " " + msg.type() + " " + actionType);

        // Finally display the frame
        display();
    }

    /**
     * Add details about an action.
     * @param  actionName  Name of the action
     * @param  actionParms  :
     * <pre>
     * 1st dimension : fields required for action
     * 2nd dimension : information related to each field
     *                  "M" = mandatory completion
     *                  "O" = optional input
     * </pre>
     * @return void
     * <p><b>Example:</b>
     * <pre>
     * addAction( new String().valueOf(MQeQueueManagerAdminMsg.Action_Update),
     *                                 new String[][] {
     *                                   { MQeQueueManagerAdminMsg.Name, "M" },
     *                                   { MQeQueueManagerAdminMsg.Desc, "O" }
     *                                 };
     *          );
     * </pre>
     */
    public void addAction(String actionName, String[][] actionParms) {
        actionFields.put(actionName, actionParms);
    }

    /**
     * Add an action to the display (frame).
     * @param action String containing the action to be performed.
     * @return void
     * @exception java.lang.Exception
     */
    public void addActionToDisplay(String action) throws Exception {

        /**
         *  Add buttons to the display related to the action
         *
         *  Assumes that there are only ever 3 buttons per action
         *  - main action, apply and cancel
         */
        if (action == null) {
            // No action - only requure an OK button and cancel button
            format(South, new String[][][]
                {
                    { { "B", "OK"}
                    }                // 0
                }
            );
            cancelB = (Button) getObject('S', 0);
            applyB = null;
            actionB = null;
        } else {
            // Display all 3 buttons
            format(South, new String[][][]
                {
                    { { "B", getNLSName(action)}, // 0
                        { "B", "Apply"},               // 1
                        { "B", "Cancel"}
                    }             // 2
                }
            );

            cancelB = (Button) getObject('S', 2);
            applyB = (Button) getObject('S', 1);
            actionB = (Button) getObject('S', 0);
        }
    }

    /**
     * Add any aliases related to this resource.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    public void addAlias() throws Exception {}

    /**
     * Add GUI properties for a field.
     * @param name name of field
     * @param displayWidget how to display the field
     * @param parent  name of this fields parent (container)
     *
     * <p>Note: the sequence of addCharacteristic calls determines the sequence
     * that fields are displayed.
     * @return void
     * <p><b>Example : </b>
     * <pre>
     *  addCharacteristic( MQeQueueManagerAdminMsg.Name, "TF", null );
     *  addCharacteristic( MQeQueueManagerAdminMsg.Routes, "C", null );
     *  addCharacteristic( MQeQueueManagerAdminMsg.RouteDestination, "TF", MQeQueueManagerAdminMsg.Routes );
     * </pre>
     */
    public void addCharacteristic(String name, String displayWidget, String parent) {
        // Add properties of field to display
        displayFields.put(name, displayWidget);

        // Remember the parent of this field
        if (parent != null)
            displayFieldsGroups.put(name, parent);

        // Add as the next field in sequence to display
        displayFieldsSeq.addElement(name);
    }

    /**
     * Store information about a field that has been displayed.
     * @param name  name of field
     * @param displayedAttrs :
     * <pre>
     *      [0]  -  GUI widget
     *      [1]  -  initial value
     *      [2]  -  field type
     * </pre>
     * @return void
     */
    public void addDisplayedField(String name, Object[] displayedAttrs) {
        displayedFields.put(name, displayedAttrs);
    }

    /**
     * Add a field to the display.
     * @param name Name of the field
     * @param baseFields MQeFields Object
     * @param baseErrors MQeFields Object
     * @return void
     * @exception java.lang.Exception
     */
    public void addFieldToDisplay(String name, MQeFields baseFields, MQeFields baseErrors) throws Exception {
        // Do not handle individual array elements, these are handled when
        // the main array field is reached
        if (name.indexOf(MQeFields.ArraySeparator) < 0) {
            if (baseFields.contains(name) &&
                baseFields.dataType(name) == MQeFields.TypeFields) {
                // Add each field in fields object to display.
                MQeFields fields = baseFields.getFields(name);
                Enumeration fieldEnum = fields.fields();

                while (fieldEnum.hasMoreElements()) {
                    String nextname = (String) fieldEnum.nextElement();

                    addFieldToDisplay(nextname, baseFields, baseErrors);
                }
            } else {
                // Add the label for this field to the label panel
                renderLabel(name);

                // This field may be contained within another fields object,
                // get the embedded fields object that name is contained in.
                MQeFields fields = getSubFields(name, baseFields);
                MQeFields errors = getSubFields(name, baseErrors);

                // Add the value of the field to the value panel
                // (All fields are converted to string representation)
                String value[] = getValueFromFields(name, fields);        // values
                String error[] = getErrorFromFields(name, fields, errors);// errors

                boolean errorsHandled;

                errorsHandled = renderCharacteristic(name, fields, errors, value, error);

                // Add the error for this field to the error panel
                renderError(name, errors, error, errorsHandled);

                // increase nimber of fields displayed
                fieldCount++;

                // Tell the layout manager about new field
                // (the same layout manager is used for each panel)
                ((java.awt.GridLayout) labelP.getLayout()).setRows(fieldCount);
            }
        }

    }

    /**
     * Add an NLS name for a field.
     * @param name Name of the field
     * @param nLSName String containing the NLSName for the field
     * @return void
     */
    public void addNLSName(String name, String nLSName) {
        nLSFields.put(name, nLSName);
    }

    /**
     * Close window.
     * @param none
     * @return void
     */
    public void close() {
        //  adminModel  = null;
        msg = null;
        action = null;

        setVisible(false);           // hide the window
        dispose();                    // finished with the window
    }

    /**
     * Handle window closing event.
     * @param none
     * @return void
     */
    protected void closeFrame() {
        close();
    }

    /**
     * Show the frame.
     * @param none
     * @return void
     */
    public void display() {
        visible(West, true);
        visible(Center, true);
        visible(East, true);
        visible(South, true);
        visible(true);

    }

    /**
     * Return an actions panel.
     * @param eventHandler An AwtEvent
     * @return A Panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        return null;
    }

    /**
     * Get any fields which have changed or are mandatory.
     * @param none
     * @return MQeFields Object
     * @exception java.lang.Exception
     */
    public MQeFields getChanges() throws Exception {
        MQeFields parms = new MQeFields();

        // Iterate through all fields for the action
        String[][] fieldsForAction = getFieldsForAction(this.action);

        for (int i = 0; i < fieldsForAction.length; i++) {
            String name = fieldsForAction[i][0];
            String value = getField(name);

            // Now get fields which have changed plus those which are mandatory
            // !! Need to handle subfields !!
            if (fieldsForAction[i][1].equals("M") || isFieldChanged(name)) {
                //          parms.RestoreFromString( "(#0)#1=#2",
                //                          ((Object[])displayedFields.get( name))[2]+value );
                setField(name, parms, value);
            }
        }

        return parms;
    }

    /**
     * Get the set of characteristics for the managed object.
     * @param none
     * @return MQeFields Object
     * @exception java.lang.Exception
     */
    public MQeFields getCharacteristics() throws Exception {
        return msg.characteristics();
    }

    /**
     *  Get any errors for a named field.
     * @param name Name of the field
     * @param values MQeFields object
     * @param errors MQeFields object
     * @return A String array
     * @exception java.lang.Exception
     */
    public String[] getErrorFromFields(String name, MQeFields values, MQeFields errors) throws Exception {
        String[] value;

        if (errors == null || !errors.contains(name)) {
            if (values.dataType(name) == MQeFields.TypeArrayElements)
                value = new String[ values.getArrayLength(name)];
            else
                value = new String[1];
        } else {
            if (errors.dataType(name) == MQeFields.TypeArrayElements)
                value = errors.getAsciiArray(name);
            else
                value = new String[] {
                            errors.getAscii(name)
                        };
        }

        return value;
    }

    /**
     * Get the value from the display of a named field.
     * @param name Name of the field
     * @return java.lang.String
     * @exception java.lang.Exception
     */
    public String getField(String name) throws Exception {
        // Currently only handles input of data from text fields.

        Object[]    v = null;
        String      value = null;

        if (getFieldDisplayType(name).equals("TF") ||
            getFieldDisplayType(name).equals("T*")) {
            v = (Object[]) displayedFields.get(name);
            value = ((AdminTextField) (v[0])).getText();
        }

        return value;
    }

    /**
     * Return the name of the widget used to display the named field.
     * @param fieldName Name of the field
     * @return java.lang.String
     */
    public String getFieldDisplayType(String fieldName) {
        if (displayFields.get(fieldName) != null)
            return (String) displayFields.get(fieldName);
        else
            return "TF";   // Default to text field
    }

    /**
     * Return the set of fields for the named action.
     * @param action The action to be performed
     * @return An array of java.lang.String array
     */
    public String[][] getFieldsForAction(String action) {
        return (String[][]) actionFields.get(action);
    }

    /**
     * Return the NLS name for the named field.
     */
    public String getNLSName(String fieldName) {
        String nlsName = (String) nLSFields.get(fieldName);

        if (nlsName == null)
            return fieldName;       // Return fieldname as nls name not available
        else
            return nlsName;         // Return nls name
    }

    /**
     * Return the name of resource being managed.
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return null;
    }

    /**
     * Return embedded fields object for named field from the base fields object.
     *
     * Fields objects can contain other fields objects. This method returns the
     * fields object that owns a named field from within the root fields object.
     */
    public MQeFields getSubFields(String name, MQeFields baseFields) throws Exception {
        if (baseFields == null)
            return null;

        // May have been passed an embedded field
        Vector nStack = new Vector();
        String nn = name;

        // Create ownership heirachy stack
        while ((nn = (String) displayFieldsGroups.get(nn)) != null) {
            nStack.addElement(nn);
        }

        MQeFields fields = baseFields;      // If not embedded then return root fields

        // Iterate through stack and return the owning fields object
        for (int i = nStack.size(); i > 0; i--) {
            String n = (String) nStack.elementAt(i - 1);

            if (fields.contains(n)) {
                if (fields.dataType(n) == MQeFields.TypeArrayElements)
                    fields = fields.getFieldsArray(n)[0];
                else if (fields.dataType(n) == MQeFields.TypeFields)
                    fields = fields.getFields(n);
            } else
                return fields;
        }

        return fields;
    }

    /**
     * Get the string value of a named field from within a fields object.
     */
    public String[] getValueFromFields(String name, MQeFields fields) throws Exception {
        String[] value;

        if (fields.dataType(name) == MQeFields.TypeArrayElements) {
            // If field is an array then dump each element of the array
            int elements = fields.getArrayLength(name);

            value = new String[elements];
            String arraySeparator = (new Character(MQeFields.ArraySeparator)).toString();

            for (int i = 0; i < elements; i++) {
                // Need to convert array sep to char array due to bug in sun and
                // ibm jdks which otherwise treat the char as an int!

                value[i] = fields.dumpToString("#2",
                            name +
                            arraySeparator +
                            i);
            }
        } else
            value = new String[] {
                        fields.dumpToString("#2", name)
                    };

        return value;
    }

    /**
     * Has the value of the field changed from its initial setting.
     */
    public boolean isFieldChanged(String name) throws Exception {
        // Currently only handles input of data from text fields.

        Object[]    v = null;
        boolean     value = false;

        if (getFieldDisplayType(name).equals("TF") ||
            getFieldDisplayType(name).equals("T*")) {
            v = (Object[]) displayedFields.get(name);
            value = ((AdminTextField) (v[0])).isChanged();
        }

        return value;
    }

    /**
     * Display a message.
     * @param msg String containing the message to be displayed.
     * @return void
     */
    public void message(String msg) {
        messageL.setText(msg);
    }

    /**
     * Handle rendering of a named field.
     * @param name Name of the field
     * @param fields MQeFields object
     * @param errors MQeFields object
     * @param value A String array
     * @param error A String array
     * @return boolean
     * @exception java.lang.Exception
     */
    public boolean renderCharacteristic(String name, MQeFields fields, MQeFields errors,
        String[] value, String[] error) throws Exception {
        Object  v = null;     // The display widget
        String  d = null;   // The string to display
        int     errCount = 0;        // Number of errors in array

        if (getFieldDisplayType(name).equals("TF")) {

            /**
             * Display in a textfield
             * - if more than one value then only the first is displayed
             */
            v = new AdminTextField(value[0]);
            valueP.add((AdminTextField) v);
        } else if (getFieldDisplayType(name).equals("T*")) {

            /**
             * Display in a echo-suppressed textfield
             * - if more than one value then only the first is displayed
             */
            v = new AdminTextField(value[0]);
            ((AdminTextField) v).tf.setEchoChar('*');     /* set mask character   */
            valueP.add((AdminTextField) v);
        } else if (getFieldDisplayType(name).equals("TA")) {

            /**
             * Display in a textarea
             * - if more than one value each value displayed on a new line
             */
            v = new TextArea();
            for (int i = 0; i < value.length; i++) {
                // If displayed in one line then add in error
                if (error[i] != null) {
                    d = value[i] + "   " + error[i];
                    errCount++;
                } else
                    d = value[i];

                ((TextArea) v).append(d);
            }
            valueP.add((TextArea) v);
        } else if (getFieldDisplayType(name).equals("C")) {

            /**
             * Display in a choice box
             */
            v = new Choice();
            for (int i = 0; i < value.length; i++) {

                if (error[i] != null) {
                    // If displayed in one line then add in error
                    d = value[i] + "   " + error[i];
                    errCount++;
                } else
                    d = value[i];

                ((Choice) v).add(d);
            }
            valueP.add((Choice) v);
        } else {
            // Display widget not known used a textfield
            v = new TextField(value[0]);
            valueP.add((TextField) v);
        }

        // Remember what has been displayed, the initial value and type
        addDisplayedField(name, new Object[] { v, value, fields.dumpToString("(#0)#1=", name)}
        );

        // If errors have occured and been handled then let caller know
        if (errCount > 0)
            return true;
        else
            return false;
    }

    /**
     * Handle rendering of any errors related to a named field.
     * @param name Name of the field
     * @param errors MQeFields object
     * @param error A String array
     * @param alreadyDisplayed A boolean
     * @return void
     * @exception java.lang.Exception
     */
    public void renderError(String name, MQeFields errors, String[] error, boolean alreadyDisplayed) throws Exception {

        /**
         * Error may have already been displayed (i.e. if there was more than
         * one error. In this case just display a generic error message
         */
        String lab = "";

        if (!alreadyDisplayed) {
            if (error.length > 0)
                if (error[0] != null)
                    lab = error[0];
        } else
            lab = "Error(s)";

        Object e = new Label(lab);

        errorP.add((Label) e);
    }

    /**
     * Display the label for a field.
     * @param name Name of the field
     * @return void
     *
     */
    public void renderLabel(String name) {
        // Add the label
        String nlsName = getNLSName(name);
        Label l = new Label(nlsName);

        labelP.add(l, null);
    }

    /**
     * Process a queue manager action request.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A boolean
     * @return void
     * @exception java.lang.Exception
     */
    public void resourceAction(Object e, int where, int index, String  choice, boolean state) throws Exception {}

    /**
     * Sets the AdminModel.
     * @param am AdminModel object
     * @return void
     *
     */
    public void setAdminModel(AdminModel am) {
        adminModel = am;
    }

    /**
     * <p>Return embedded fields object for named field from the base fields object.
     * <p>MQeFields objects can contain other fields objects. This method returns the
     * fields object that owns a named field from within the root fields object.
     * @param name Name of the field
     * @param baseFields MQeFields object
     * @param value java.lang.String
     * @return void
     * @exception java.lang.Exception
     */
    public void setField(String name, MQeFields baseFields, String value) throws Exception {
        // May have been passed an embedded field
        Vector nStack = new Vector();
        String nn = name;

        // Create ownership heirachy stack
        while ((nn = (String) displayFieldsGroups.get(nn)) != null) {
            nStack.addElement(nn);
        }

        MQeFields fields = baseFields;      // If not embedded then use root fields

        // Iterate through stack and return the owning fields object
        for (int i = nStack.size(); i > 0; i--) {
            String n = (String) nStack.elementAt(i - 1);

            if (!fields.contains(n))           // does subfield exist
                fields.putFields(n, new MQeFields());     // no create it

            fields = fields.getFields(n);         // get subfield
        }

        fields.restoreFromString("(#0)#1=#2",
            ((Object[]) displayedFields.get(name))[2] + value);
    }

    /**
     * Setup the administration message with the action to be performed and the
     * parameters required by the action.
     * MUST be implemented in subclass
     * @param parms MQeFields object
     * @return MQeAdminMsg object
     * @exception java.lang.Exception
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {
        return null;
    }

    /**
     * Configure the valid set of actions for a particular type of resource.
     * MUST be implemented in subclass.
     * @param none
     * @return void
     */
    protected void setupActions() {}

    /**
     * Setup the set of characteristics for a particular type of resource.
     * MUST be implemented in subclass.
     * @param none
     * @return void
     */
    protected void setupFields() {}

    /**
     * Setup NLS names for characteristics of a particular type of resource.
     * @param none
     * @return void
     */
    protected void setupNLS() {}

    /**
     * Validate the user's input.
     * @param none
     * @return a boolean value
     */
    public boolean validateInput() throws Exception {

        /*
         * If the field is manadatory, ensure user has entered some data
         */
        String[][] fieldsForAction = getFieldsForAction(this.action);

        for (int i = 0; i < fieldsForAction.length; i++) {
            String name = fieldsForAction[i][0];
            String value = getField(name);

            // If field is manadatory then a value must be entered
            if (fieldsForAction[i][1].equals("M") && (value.equals("") || value.equals("null"))) {
                message("Must enter value for " + getNLSName(name));
                return false;
            }
        }

        return true;
    }

    /**
     * Returns a string that provides information about the current frame.
     * This is siaplyed in the south panel of the main admin gui when a frame is selected.
     * @param none
     * @return String documenting the current frame.
     *
     */
    public String getMessageHelp() {
        return "";
    }

}
