// Version mqe_java/source/examples/administration/console/AdminLogBrowser.java, MQeExamples, la200, la200-L030324.1 1.6

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import java.util.*;


/***
 * Extend the queue browser to provide additional support for the          
 * administration reply to queue/log. 
 *                                                                                     
 *                                                                             
 **/

public class AdminLogBrowser extends    AdminQueueBrowser
    implements MQeMessageListenerInterface,
        java.beans.PropertyChangeListener {
    public          static short version[] = { 2, 0, 0, 5 }; 

    /**
     * automatically display message detail window ?
     */
    protected boolean autoDisplayMsg = false;

    /**
     * AdminLog default constructor.
     */
    public AdminLogBrowser() {
        super("Admin Log Browser");
    }

    /**
     * AdminLog constructor.
     * @param admin AdminModel
     */
    public AdminLogBrowser(AdminModel admin) {
        super("Admin Log Browser");
        activate(admin);
    }

    /**
     * AdminLog constructor.
     * @param title java.lang.String
     */
    public AdminLogBrowser(String title) {
        super(title);
    }

    /**
     * <p>Adds a new message to list of messages.
     * <p>Overriden to display the message in a new window if it is a
     * reply.
     * @param msg MQeFields object
     * @return void
     */
    public synchronized void addMessage(MQeFields msg) {
        // Call super ....
        super.addMessage(msg);

        // If auto display of message turned on and managing the local
        // queue manager then display the message
        if (autoDisplayMsg && !adminModel.getMode()) {
            if (msg instanceof MQeAdminMsg) {
                MQeAdminMsg aMsg = (MQeAdminMsg) msg;

                try {
                    // Only display the message if it is a response
                    if (aMsg.getInt(MQe.Msg_Style) == MQe.Msg_Style_Reply)
                        displayMessage(aMsg);
                } catch (Exception ex) {}
            }
        }
    }

    /**
     * Handle window closing event.
     * @param none
     * @return void
     */
    public void close() {
        adminModel.setLogBrowser(null);
        super.close();
    }

    /**
     * Gets the name of the queue manager that owns the queue to be browsed.
     * @param none
     * @return The admin model's reply to queue manager name
     */
    public String getQMgrName() {
        return adminModel.getReplyToQM();
    }

    /**
     * Gets the name of the queue to be browsed.
     * @param none 
     * @return The admin model's reply to queue name
     */
    public String getQueueName() {
        return adminModel.getReplyToQ();
    }

    /**
     * Process a message arrived event.
     * @param e An MQeMessageEvent object
     * @return void
     */
    public void messageArrived(MQeMessageEvent e) {
        autoDisplayMsg = true;          // Turn on auto message display

        super.messageArrived(e);
    }

    /**
     * Parse a message for display in a single line.
     * @param msg MQeFields object
     * @return java.lang.String
     */
    public String parseMessageForList(MQeFields msg) {
        StringBuffer sb = new StringBuffer(150);

        // Initialise buffer to blanks
        sb.setLength(150);
        for (int i = 0; i < 150; i++) {
            sb.setCharAt(i, ' ');
        }

        // Maybe its not an admin message
        if (msg instanceof MQeAdminMsg) {
            MQeAdminMsg rec = (MQeAdminMsg) msg;

            String destQM = "MsgID?";
            String correlID = "CorrelID?";
            String startQM = "Origin QM?";
            String style = "< Response";
            String type = "unknown type";
            String action = "unknown action";
            String rc = "unknown RC";

            try {
                // destination queue manager
                if (rec.contains(MQe.Msg_MsgID))
                    destQM = (MQe.byteToAscii(rec.getArrayOfByte(MQe.Msg_MsgID)));

                // time message was created
                if (rec.contains(MQe.Msg_CorrelID))
                    correlID = (new Date(MQe.byteToLong(rec.getArrayOfByte(MQe.Msg_CorrelID), 0)).toString());

                // initiating queue manager
                if (rec.contains(MQe.Msg_OriginQMgr))
                    startQM = (rec.getOriginQMgr());

                // Message style
                if (rec.contains(MQe.Msg_Style))
                    if (rec.getInt(MQe.Msg_Style) == MQeAdminMsg.Msg_Style_Request)
                        style = "> Request";
                    else if (rec.getInt(MQe.Msg_Style) == MQeAdminMsg.Msg_Style_Datagram)
                        style = "= Datagram";

                // Type of admin message
                type = MQe.abbreviate(msg.type(), 0);
                int idot = type.lastIndexOf(".");
                int iadm = type.lastIndexOf("AdminMsg");

                if (iadm == -1)
                    iadm = type.length();
                type = type.substring(idot + 1, iadm);

                if (rec.contains(MQeAdminMsg.Admin_Action)) {
                    switch (rec.getAction()) {
                    case MQeAdminMsg.Action_Create:
                        action = "Create";
                        break;

                    case MQeAdminMsg.Action_Delete:
                        action = "Delete";
                        break;

                    case MQeAdminMsg.Action_Update:
                        action = "Update";
                        break;

                    case MQeAdminMsg.Action_Inquire:
                        action = "Inquire";
                        break;

                    case MQeAdminMsg.Action_InquireAll:
                        action = "InquireAll";
                        break;

                    default:
                        // Handle specialised actions
                        if (rec instanceof MQeConnectionAdminMsg) {
                            switch (rec.getAction()) {
                            case MQeConnectionAdminMsg.Action_AddAlias:
                                action = "Add Alias";
                                break;

                            case MQeConnectionAdminMsg.Action_RemoveAlias:
                                action = "Remove Alias";
                                break;
                            }
                        } else
                            action = new Integer(rec.getAction()).toString();
                    }
                }

                // Return code
                if (rec.contains(MQeAdminMsg.Admin_RC)) {
                    if (rec.getInt(MQe.Msg_Style) == MQeAdminMsg.Msg_Style_Request)
                        rc = " ";
                    else {
                        switch (rec.getRC()) {
                        case MQeAdminMsg.RC_Success:
                            rc = "Success";
                            break;

                        case MQeAdminMsg.RC_Fail:
                            rc = "Failed";
                            break;

                        case MQeAdminMsg.RC_Mixed:
                            rc = "Error(s) occured";
                            break;
                        }
                    }
                }

                // Put data in buffer
                sb.insert(0, style);
                sb.insert(11, ' ');
                sb.insert(12, destQM);
                sb.insert(39, ' ');
                sb.insert(40, type);
                sb.insert(59, ' ');
                sb.insert(60, action);
                sb.insert(76, ' ');
                sb.insert(77, rc);
                sb.insert(94, ' ');
                sb.insert(95, correlID);
                sb.insert(125, ' ');
                sb.insert(126, startQM);
            } catch (Exception e) {
                sb.append("Failed to parse message: " + e.toString());
            }

            // Reset buffer to correct length
            sb.setLength(150);
        } else
            sb.append("Non admin message");

        return sb.toString();
    }

    /**
     * Method to handle events for the PropertyChangeListener interface.
     * @param evt java.beans.PropertyChangeEvent
     * @return void
     */

    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    public void propertyChange(java.beans.PropertyChangeEvent evt) {
        super.propertyChange(evt);

        if ((evt.getSource() == adminModel) && (evt.getPropertyName().equals("replyToQM")) ||
            (evt.getSource() == adminModel) && (evt.getPropertyName().equals("replyToQ"))) {
            // Reply to queue manager / queue name has changed
            stopMonitor();                      // Stop monitoring the old queue
            qMgrTF.setText(getQMgrName());    // Display new QMgr name
            qTF.setText(getQueueName());      // Display new Queue name
            startMonitor();                     // Start monitoring new queue
            refresh();                          // Refresh
        }
    }

    /**
     * Refreshes the display and then browses the queue.
     * @param none
     * @return void
     */
    public void refresh() {
        autoDisplayMsg = false;     // Turn off auto message display

        super.refresh();
    }
}
