// Version mqe_java/source/examples/administration/console/AdminModel.java, MQeExamples, la200, la200-L030324.1 1.9

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import java.io.*;
import java.util.*;
import examples.queuemanager.*;


/*------------------------------------------------------------------------------*/

/* Class:                                                                       */

/*                                                                              */

/*      AdminModel                                                              */

/*                                                                              */

/* Extends:                                                                     */

/*                                                                              */

/*      MQe  - base MQSeries Everywhere class                                   */

/*                                                                              */

/* Function:                                                                    */

/*                                                                              */

/*      To provide a set of methods for simplyfying the task of administering   */

/*      MQSeries everywhere resources.                                          */

/*                                                                              */

/* History:                                                                     */

/*                                                                              */

/*      Created         01/May/1998                                             */

/*                                                                              */

/*------------------------------------------------------------------------------*/

public class AdminModel extends MQe {
    public          static short version[] = { 2, 0, 0, 5 }; 

    protected transient java.beans.PropertyChangeSupport propertyChange;

    private String  fieldManageQM = new String(); // Manage QMgr name
    private String  fieldManageViaQM = new String(); // Local QMgr name
    private String  fieldStatus = new String(); // Status/message
    private String  fieldReplyToQ = new String(); // Reply to queue
    private String  fieldReplyToQM = new String(); // Reply to QMgr
    private boolean fieldMode = false;        // local or remote mode
    private boolean fieldConnected = false;        // Connected to QMgr
    protected Vector  msgFrames = new Vector(); // Active msg windows
    protected Vector  qFrames = new Vector(); // Active queue browser windows
    private AdminLogBrowser fieldLog = null;         // Log queue browser
    protected MQeClient       client = null;         // client
    private String  fieldQMgrStartupIniFile = new String();

    public final static String requestQ = MQe.Admin_Queue_Name; // Admin queue

    /**
     * AdminModel default constructor.
     */
    public AdminModel() {
        super();
        activate();
    }

    /**
     * activate - initialisation code.
     */
    public void activate() {}

    /**
     * The addPropertyChangeListener method was generated to support the propertyChange field.
     */
    public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener) {
        getPropertyChange().addPropertyChangeListener(listener);
    }

    /**
     * Ensure connection state is set correctly.
     */
    public void checkConnected() {
        // Does the connected state match reality ?
        boolean con = isQMgrStarted();

        if (con != getConnected()) {
            setConnected(con);
        }
    }

    /**
     * Close any active msg display windows.
     */
    public void closeMsgDisplay() {
        // iterate through active message windows
        while (!msgFrames.isEmpty()) {
            // get last one
            AdminFrame fr = (AdminFrame) msgFrames.lastElement();

            // remove from list
            msgFrames.removeElement(fr);

            // Close it
            if (fr != null)
                fr.close();
        }
    }

    /**
     * Close any active queue browser windows.
     */
    public void closeQueueDisplay() {
        // Iterate through active windows
        while (!qFrames.isEmpty()) {
            // Get last one in list
            AdminQueueBrowser fr = (AdminQueueBrowser) qFrames.lastElement();

            // Remove it from the list
            qFrames.removeElement(fr);

            // Close it
            if (fr != null)
                fr.close();
        }
    }

    /**
     * Connect to access the local queue manager.
     */
    public void connectLocalQM() throws Exception {
        checkConnected();
    }

    /**
     * Disconnect from the local queue manager.
     */
    public void disconnectLocalQM() {
        setConnected(false);
    }

    /**
     * Display the relavent GUI for an admin message.
     */
    public AdminFrame displayMsg(MQeAdminMsg msg, int action, AdminFrame redisplayFrame) throws Exception {
        AdminFrame fr = redisplayFrame;

        if (redisplayFrame == null)                        // Does frame already exist ?
        {                                                   // No, so create new one
            String cl = MQe.abbreviate(msg.type(), 0);         // get long class name

            cl = cl.substring(cl.lastIndexOf('.') + 1) + "GUI";    // now find its full name

            fr = (AdminFrame) loadObject(cl);   // load an empty instance of GUI
            msgFrames.addElement(fr);                     // remeber we opened the window
        }

        fr.activate(msg, action, this);                   // activate it

        return fr;
    }

    /**
     * Display the relavent GUI for a queue.
     */
    public AdminQueueBrowser displayQueue(String qMgrName, String qName, String browserType, AdminQueueBrowser redisplayFrame) throws Exception {

        // Could Add code to keep track of all windows if required !

        if (browserType == null)                    // User default queue
            browserType = "QueueBrowserDefault";    // browser ?

        AdminQueueBrowser fr = redisplayFrame;

        if (redisplayFrame == null)           // Does frame already exist ?
        {                                       // No, so create new one
            fr = (AdminQueueBrowser) loadObject(browserType);   // load an empty instance of GUI
            qFrames.addElement(fr);           // remeber we opened the window
        }

        fr.setQMgrName(qMgrName);             // Set QMgr name
        fr.setQueueName(qName);               // Set queue name
        fr.activate(this);                    // activate it

        return fr;
    }

    /**
     * The firePropertyChange method was generated to support the propertyChange field.
     */
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Gets the state of the connection to the local QMgr.
     * @return The connected property value.
     * @see examples.administration.console.AdminModel#setConnected
     */
    public boolean getConnected() {
        return fieldConnected;
    }

    /**
     * Get the log queue browser object.
     * @return The log property value.
     * @see examples.administration.console.AdminModel#setLogBrowser
     */
    public AdminLogBrowser getLogBrowser() {
        if (fieldLog == null)
            setLogBrowser(new AdminLogBrowser());

        return fieldLog;
    }

    /**
     * Gets the name of the queue manager to be administered.
     * @return The manageQM property value.
     * @see examples.administration.console.AdminModel#setManageQM
     */
    public String getManageQM() {
        if (fieldManageQM == null || fieldManageQM.equals(""))
            fieldManageQM = getManageViaQM();

        return fieldManageQM;
    }

    /**
     * Gets the name of the local queue manager.
     * @return The manageViaQM property value.
     * @see examples.administration.console.AdminModel#setManageViaQM
     */
    public String getManageViaQM() {
        return fieldManageViaQM;
    }

    /**
     * Gets the mode property (boolean) value.
     * <pre>
     * - false: manage a local queue manager
     * - true:  manage a remote queue manager
     * </pre>
     * @return The mode property value.
     * @see examples.administration.console.AdminModel#setMode
     */
    public boolean getMode() {
        return fieldMode;
    }

    /**
     * Accessor for the propertyChange field.
     */
    protected java.beans.PropertyChangeSupport getPropertyChange() {
        if (propertyChange == null) {
            propertyChange = new java.beans.PropertyChangeSupport(this);
        };
        return propertyChange;
    }

    /**
     * Get a reference to the queue manager.
     */
    public MQeQueueManager getQM() {
        MQeQueueManager qm = null;

        try {
            qm = MQeQueueManager.getDefaultQueueManager();
        } catch (Exception e) {}

        return qm;
    }

    /**
     * Gets the qMgrStartupIniFile property (java.lang.String) value.
     * @return The qMgrStartupIniFile property value.
     * @see examples.administration.console.AdminModel#setQMgrStartupIniFile
     */
    public java.lang.String getQMgrStartupIniFile() {
        return fieldQMgrStartupIniFile;
    }

    /**
     * Gets the reply to queue name.
     * @return The replyToQ property value.
     * @see examples.administration.console.AdminModel#setReplyToQ
     */
    public String getReplyToQ() {
        return fieldReplyToQ;
    }

    /**
     * Gets the reply to queue manager name.
     * @return The replyToQM property value.
     * @see examples.administration.console.AdminModel#setReplyToQM
     */
    public String getReplyToQM() {
        return fieldReplyToQM;
    }

    /**
     * Gets the status / last message.
     * @return The status property value.
     * @see examples.administration.console.AdminModel#setStatus
     */
    public String getStatus() {
        return fieldStatus;
    }

    /**
     * Is the local queue manager started?
     */
    public boolean isQMgrStarted() {
        boolean avail = false;

        try {
            // Is the queue manager started ?
            if (MQeQueueManager.getDefaultQueueManager() != null)
                avail = true;
        } catch (Exception ex) {}

        return avail;
    }

    /**
     * Is a reply required ?
     */
    public boolean isReplyRequired() {
        if (getReplyToQ() == null || getReplyToQ().equals(""))
            return false;
        else
            return true;
    }

    /**
     * Put a message to the reply to queue manager / queue.
     */
    public void logMsg(MQeAdminMsg msg) throws Exception {
        if (isReplyRequired()) {
            setStatus("Sending audit message");
            getQM().putMessage(getReplyToQM(), getReplyToQ(), msg, null, 0);
            setStatus("Sent audit message");
        }
    }

    /**
     * The removePropertyChangeListener method was generated to support the propertyChange field.
     */
    public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener) {
        getPropertyChange().removePropertyChangeListener(listener);
    }

    /**
     * Send an admin message to the queue manager to be administered.
     * @param adminRequestMsg com.ibm.mqe.MQeAdminMsg
     */
    public synchronized void sendRequest(MQeAdminMsg requestMsg) throws MQeException {
        try {
            // Check if an action has been set. If not throw exception
            requestMsg.getAction();

            // Store the name of the queue manager that should process
            // the message.
            requestMsg.setTargetQMgr(getManageQM());

            // Set a reply to queue
            if (isReplyRequired()) {
                // Ask for a reply
                requestMsg.putAscii(MQe.Msg_ReplyToQ, getReplyToQ());
                requestMsg.putAscii(MQe.Msg_ReplyToQMgr, getReplyToQM());
                requestMsg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
            } else
                // Make it a datagram
                requestMsg.putInt(MQe.Msg_Style, MQe.Msg_Style_Datagram);

            // Make the request unique (Queue manager to manage + plus timestamp)
            requestMsg.putArrayOfByte(MQe.Msg_MsgID, getManageQM().getBytes());
            requestMsg.putArrayOfByte(MQe.Msg_CorrelID, MQe.longToByte(MQe.uniqueValue()));

            // Write request to audit log (reply to queue)
            // Note - need to log the request first otherwise the reply may
            //  preceed the request in the reply to queue.
            if (isReplyRequired()) {
                logMsg(requestMsg);               // Log the message
                requestMsg.resetMsgUIDFields();     // Need to reset UID so it can be sent again
            }

            try {
                // Send the request
                setStatus("Send message to " + getManageQM());
                getQM().putMessage(getManageQM(), requestQ, requestMsg, null, 0);
                setStatus("Sent message to " + getManageQM());
            } catch (Exception e) {
                // If the request failed then log the failure
                requestMsg.resetMsgUIDFields();     // Need to reset UID so it can be sent again
                // Store the reason the send failed
                requestMsg.setResult(MQeAdminMsg.RC_Fail, e);
                // Mark it as the corresponding reply to the request
                requestMsg.putInt(MQe.Msg_Style, MQe.Msg_Style_Reply);
                logMsg(requestMsg);               // Log the message

                throw e;
            }
        } catch (Exception e) {
            setStatus(e.toString());
        }
    }

    /**
     * Sets the connected property (boolean) value.
     * @param connected The new value for the property.
     * @see examples.administration.console.AdminModel#getConnected
     */
    public void setConnected(boolean connected) {
        boolean oldValue = fieldConnected;

        fieldConnected = connected;
        firePropertyChange("connected", new Boolean(oldValue), new Boolean(connected));
    }

    /**
     * Sets the log property (Example.Admin.AdminLogModel) value.
     * @param log The new value for the property.
     * @see examples.administration.console.AdminModel#getLogBrowser
     */
    public void setLogBrowser(AdminLogBrowser log) {
        AdminLogBrowser oldValue = fieldLog;

        fieldLog = log;
        if (log != null)
            qFrames.addElement(log); // Add to the list of queue browser Windows
        firePropertyChange("log", oldValue, log);
    }

    /**
     * Sets the manageQM property (java.lang.String) value.
     * @param manageQM The new value for the property.
     * @see examples.administration.console.AdminModel#getManageQM
     */
    public void setManageQM(String manageQM) {
        String oldValue = fieldManageQM;

        fieldManageQM = manageQM;
        firePropertyChange("manageQM", oldValue, manageQM);
    }

    /**
     * Sets the manageViaQM property (java.lang.String) value.
     * @param manageViaQM The new value for the property.
     * @see examples.administration.console.AdminModel#getManageViaQM
     */
    public void setManageViaQM(String manageViaQM) {
        String oldValue = fieldManageViaQM;

        fieldManageViaQM = manageViaQM;

        // If in local mode then the managing and managed queue manager
        // are one and the same.
        if (!getMode()) {
            setManageQM(manageViaQM);
            setReplyToQM(manageViaQM);
        }

        firePropertyChange("manageViaQM", oldValue, manageViaQM);

    }

    /**
     * Sets the mode property (boolean) value.
     * @param mode The new value for the property.
     * @see examples.administration.console.AdminModel#getMode
     */
    public void setMode(boolean mode) {
        boolean oldValue = fieldMode;

        fieldMode = mode;
        firePropertyChange("mode", new Boolean(oldValue), new Boolean(mode));
    }

    /**
     * Sets the qMgrStartupIniFile property (java.lang.String) value.
     * @param qMgrStartupIniFile The new value for the property.
     * @see examples.administration.console.AdminModel#getQMgrStartupIniFile
     */
    public void setQMgrStartupIniFile(java.lang.String qMgrStartupIniFile) {
        String oldValue = fieldQMgrStartupIniFile;

        fieldQMgrStartupIniFile = qMgrStartupIniFile;
        firePropertyChange("qMgrStartupIniFile", oldValue, qMgrStartupIniFile);
    }

    /**
     * Sets the replyToQ property (java.lang.String) value.
     * @param replyToQ The new value for the property.
     * @see examples.administration.console.AdminModel#getReplyToQ
     */
    public void setReplyToQ(String replyToQ) {
        String oldValue = fieldReplyToQ;

        fieldReplyToQ = replyToQ;
        firePropertyChange("replyToQ", oldValue, replyToQ);
    }

    /**
     * Sets the replyToQM property (java.lang.String) value.
     * @param replyToQM The new value for the property.
     * @see examples.administration.console.AdminModel#getReplyToQM
     */
    public void setReplyToQM(String replyToQM) {
        String oldValue = fieldReplyToQM;

        fieldReplyToQM = replyToQM;
        firePropertyChange("replyToQM", oldValue, replyToQM);
    }

    /**
     * Sets the status property (java.lang.String) value.
     * @param status The new value for the property.
     * @see examples.administration.console.AdminModel#getStatus
     */
    public void setStatus(String status) {
        String oldValue = fieldStatus;

        fieldStatus = status;
        firePropertyChange("status", oldValue, status);
    }

    /**
     * Start the local queue manager.
     */
    public void startLocalQM() throws Exception {
        // First try and connect to an existing queue manager.
        connectLocalQM();

        // If that fails then start it
        if (getQM() == null) {
            client = new examples.queuemanager.MQeClient(getQMgrStartupIniFile());
            setManageViaQM(getQM().getName());
        }

        checkConnected();
    }

    /**
     * Stop the local queue manager.
     */
    public void stopLocalQM() {
        // If started using example client then stop the client
        try {
            if (client != null)
                client.close();
        } catch (Exception ex) {
            setStatus("Close failed " + ex);
        }
        client = null;

        // If queue manager is still running then stop it
        try {
            if (getQM() != null)
                getQM().close();
        } catch (Exception ex) {
            setStatus("Close failed " + ex);
        }

        // Ensure state is correctly set
        checkConnected();
    }
}
