// Version mqe_java/source/examples/administration/console/AdminQueue.java, MQeExamples, la200, la200-L030324.1 1.7

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import java.util.*;
import examples.awt.*;
import java.awt.*;


/***
 * Specialisation of AdminFrame allowing for administration of a           
 * a queue.                                                                
 *                                                                              
 **/
public class AdminQueue extends AdminFrame {
    public          static short version[] = { 2, 0, 0, 5 }; 

    // Additional actions

    /**
     * The browse action is not an action on a queue admin message as all of
     * the other admin actions are. It is an action which interacts directly
     * with the queue manager and not through the admin queue.
     */
    public final static int        Action_Browse = 99;

    /**
     * The action used to represent the putting of a test message.
     */
    public final static int        Action_PutTestMessage = 100;

    /**
     * The action used to represent the getting of a test message.
     */
    public final static int        Action_GetMessage = 101;

    // Additional fields that are not part of the admin message itself.

    // Alias name
    public final static String     Queue_Alias = "aliasn";

    /**
     * Default constructor.
     */
    public AdminQueue() {
        super("Queue manager administration");
    }

    /**
     * Main constructor.
     *
     * @param msg An administration message to contain the parameters for the action. 
     * If it already contains any parameters then these are taken as defaults.
     * @param action  The action to be performed
     * @param adminModel The administration model which will send the message.
     * @exception java.lang.Exception
     *
     */
    public AdminQueue(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {
        super(msg, action, adminModel);
    }

    /**
     * AdminQueue constructor.
     * @param title java.lang.String
     * @exception java.lang.Exception
     */
    public AdminQueue(String title) throws Exception {
        super(title);
    }

    /*
     * Called to process an MQe awt event.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A boolean
     * @return void
     * @exception java.lang.Exception                                         
     **/
    public void action(Object e, int where, int index, String  choice, boolean state) {
        boolean handled = false;

        try {
            switch (where)                          // based on N, S, E, W, C, M
            {
            case South:
                // If OK or apply button then process action
                if (e == actionB || e == applyB) {
                    int thisAction = new Integer(action).intValue();

                    // Check input is valid
                    if (!validateInput())
                        break;
                        
                    // Get the queue/qmgr name from the user entered data...
                    MQeFields parms = getChanges();
                    String qN = parms.getAscii(MQeQueueAdminMsg.Admin_Name);
                    String qmN = null;

                    if (parms.contains(MQeQueueAdminMsg.Queue_QMgrName))
                        qmN = parms.getAscii(MQeQueueAdminMsg.Queue_QMgrName);

                    // Browse the given queue...
                    if (thisAction == Action_Browse) {
                        adminModel.displayQueue(qmN, qN, null, null);
                        close();
                        handled = true;

                    } else if ((thisAction == Action_PutTestMessage) || 
                        (thisAction == Action_GetMessage)) {

                        // Deal with the put/get test message.
                        // Enclose in a try/catch block- be prepared for the test msg failing.
                        try {
        
                            // Put a test message
                            if (thisAction == Action_PutTestMessage) {
                                sendTestMessage(qN, qmN);
        
                                // Create an "it worked" dialog for user feedback.
                                new AwtDialog("Test message sent ok", AwtDialog.Show_OK, new String[][][] {
                                        { { "L", "Good"}
                                        },
                                    }
                                );
                            } // Get a test message
                            else if (thisAction == Action_GetMessage) {
                                MQeMsgObject obj = getTestMessage(qN, qmN);
        
                                // Display the message contents
                                new AwtDialog("Message Successfully Retrieved", AwtDialog.Show_OK, new String[][][] {
                                        { { "AP", "Returned message contained the following fields:\n\n" + obj.dumpToString("(#0)#1=#2")}
                                        },
                                    }
                                );
        
                            }
        
                        } catch (Exception problem) {
        
                            // Work out the failure string we will report.
                            String problemString = problem.toString();

                            if (problem instanceof MQeException) {
                                problemString = "(Failure Code="
                                        + ((MQeException) problem).code()
                                        + ") "
                                        + problemString;
                            }
        
                            new AwtDialog("Failed!", AwtDialog.Show_OK, new String[][][] {
                                    { { "AP", problemString}
                                    },
                                }
                            );
        
                            problem.printStackTrace();
                        } finally {
                            close();
                            handled = true;
                        }
                    } // end if put or get test message
                } // end if action == ok or apply

            } // end switch

            if (!handled)
                super.action(e, where, index, choice, state);

        } catch (Exception ex) {
            message(e.toString());
            ex.printStackTrace();
        }
    }

    /**
     * Add any aliases related to this resource.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    public void addAlias() throws Exception {
        MQe.alias("MQeQueueAdminMsgGUI",
            "examples.administration.console.AdminQueue");

    }

    /**
     * Return an actions panel.
     * @param eventHandler An AwtEvent
     * @return A Panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        String[][][] qActions = new String[][][]
            { { { "B", "Inquire"         }
                },        // 0
                { { "B", "Create"          }
                },        // 1
                { { "B", "Update"          }
                },        // 2
                { { "B", "Delete"          }
                },        // 3
                { { "B", "Add Alias"       }
                },        // 4
                { { "B", "Remove Alias"    }
                },        // 5
                { { "B", "Browse"          }
                },        // 6
                { { "B", "Put Test Message"}
                },        // 7
                { { "B", "Get Message"     }
                },        // 8
            };

        // ..Queue actions
        AwtFormat actionF = new AwtFormat(eventHandler, East);

        return  actionF.panel(new Panel(), qActions);
    }

    /**
     * Configure the valid set of actions.
     * @param none
     * @return A String Array.
     */
    protected String[][] getAddCharacteristics() {
        return  new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
                { MQeQueueAdminMsg.Queue_QMgrName, "O" },
                { MQeQueueAdminMsg.Queue_Description, "O" },
                { MQeQueueAdminMsg.Queue_Mode, "O" },
                { MQeQueueAdminMsg.Queue_FileDesc, "O" },
                { MQeQueueAdminMsg.Queue_Cryptor, "O" },
                { MQeQueueAdminMsg.Queue_Authenticator, "O" },
                { MQeQueueAdminMsg.Queue_Compressor, "O" },
                { MQeQueueAdminMsg.Queue_TargetRegistry, "O" },
                { MQeQueueAdminMsg.Queue_AttrRule, "O" },
                { MQeQueueAdminMsg.Queue_Rule, "O" },
                { MQeQueueAdminMsg.Queue_Expiry, "O" },
                { MQeQueueAdminMsg.Queue_MaxMsgSize, "O" },
                { MQeQueueAdminMsg.Queue_Priority, "O" },
                { MQeQueueAdminMsg.Queue_MaxQSize, "O" }
            };
    }

    /**
     * Get the set of characteristics for the managed object.
     * @param none 
     * @return MQeFields object
     * @exception java.lang.Exception
     */
    public MQeFields getCharacteristics() throws Exception {
        MQeFields fields = super.getCharacteristics();

        fields.putAscii(Queue_Alias, null);

        return fields;
    }

    /**
     * Return the name of resource being managed.
     * @param none
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return "Local Queue";
    }

    /**
     * Configure the valid set of actions.
     * @param none
     * @return An array of String array
     */
    protected String[][] getUpdateCharacteristics() {
        return  new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
                { MQeQueueAdminMsg.Queue_QMgrName, "O" },
                { MQeQueueAdminMsg.Queue_Description, "O" },
                { MQeQueueAdminMsg.Queue_Mode, "O" },
                { MQeQueueAdminMsg.Queue_FileDesc, "O" },
                { MQeQueueAdminMsg.Queue_Cryptor, "O" },
                { MQeQueueAdminMsg.Queue_Authenticator, "O" },
                { MQeQueueAdminMsg.Queue_Compressor, "O" },
                { MQeQueueAdminMsg.Queue_TargetRegistry, "O" },
                { MQeQueueAdminMsg.Queue_AttrRule, "O" },
                { MQeQueueAdminMsg.Queue_Rule, "O" },
                { MQeQueueAdminMsg.Queue_Expiry, "O" },
                { MQeQueueAdminMsg.Queue_MaxMsgSize, "O" },
                { MQeQueueAdminMsg.Queue_Priority, "O" },
                { MQeQueueAdminMsg.Queue_MaxQSize, "O" }
            };
    }

    /**
     * Get the string value of a named field from within a fields object.
     * @param name Name of the field
     * @param fields MQeFields object
     * @return A String array
     * @exception java.lang.Exception
     */
    public String[] getValueFromFields(String name, MQeFields fields) throws Exception {
        if (name.equals(MQeQueueAdminMsg.Queue_CreationDate))
            // Display creation date in readable form ...
            return new String[] { new Date(fields.getLong(name)).toString() };
        else if (name.equals(MQeQueueAdminMsg.Admin_Class))
            // Display full class name
            return new String[] { MQe.abbreviate(fields.getAscii(name), 0) };
        else
            return super.getValueFromFields(name, fields);

    }

    /**
     * Process a queue action request.
     * @param e java.lang.Object
     * @param where An integer indicating the location
     * @param index An integer containing the type of resource to be managed
     * @param choice java.lang.String
     * @param state A boolean
     * @return void
     * @exception java.lang.Exception
     */
    public void resourceAction(Object e, int where, int index, String  choice, boolean state) throws Exception {
        int act = MQeAdminMsg.Action_Unknown;       // Unknown action

        // Decide on which action user selected ?
        switch (index) {
        case 0:
            act = MQeQueueAdminMsg.Action_InquireAll;
            break;

        case 1:
            act = MQeQueueAdminMsg.Action_Create;
            break;

        case 2:
            act = MQeQueueAdminMsg.Action_Update;
            break;

        case 3:
            act = MQeQueueAdminMsg.Action_Delete;
            break;

        case 4:
            act = MQeQueueAdminMsg.Action_AddAlias;
            break;

        case 5:
            act = MQeQueueAdminMsg.Action_RemoveAlias;
            break;

        case 6:
            act = AdminQueue.Action_Browse;
            break;

        case 7:
            act = AdminQueue.Action_PutTestMessage;
            break;

        case 8:
            act = AdminQueue.Action_GetMessage;
            break;
        }

        // Display the message in a window
        activate(new MQeQueueAdminMsg(adminModel.getManageQM(), "name"),
            act, adminModel);

    }

    /**
     * Setup the admin message with the action to be performed and the
     * parameters required by the action.
     * @param parms MQeFields object
     * @return MQeAdminMsg object
     * @exception java.lang.Exception
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {
        switch (new Integer(action).intValue()) {
        case MQeQueueAdminMsg.Action_InquireAll:
            // Inquire
            ((MQeQueueAdminMsg) msg).inquireAll(parms);
            break;

        case MQeQueueAdminMsg.Action_Update:
            // Update
            ((MQeQueueAdminMsg) msg).update(parms);
            break;

        case MQeQueueAdminMsg.Action_Create:
            // Create
            ((MQeQueueAdminMsg) msg).create(parms);
            break;

        case MQeQueueAdminMsg.Action_Delete:
            // Delete
            ((MQeQueueAdminMsg) msg).delete(parms);
            break;

        case MQeQueueAdminMsg.Action_AddAlias:
            // Add an alias
            ((MQeQueueAdminMsg) msg).setName(parms.getAscii(MQeQueueAdminMsg.Admin_Name));
            ((MQeQueueAdminMsg) msg).addAlias(parms.getAscii(AdminQueue.Queue_Alias));
            if (parms.contains(MQeQueueAdminMsg.Queue_QMgrName))
                ((MQeQueueAdminMsg) msg).getInputFields().putAscii(MQeQueueAdminMsg.Queue_QMgrName, parms.getAscii(MQeQueueAdminMsg.Queue_QMgrName));
            break;

        case MQeQueueAdminMsg.Action_RemoveAlias:
            // Remove an alias
            ((MQeQueueAdminMsg) msg).setName(parms.getAscii(MQeQueueAdminMsg.Admin_Name));
            ((MQeQueueAdminMsg) msg).removeAlias(parms.getAscii(AdminQueue.Queue_Alias));
            if (parms.contains(MQeQueueAdminMsg.Queue_QMgrName))
                ((MQeQueueAdminMsg) msg).getInputFields().putAscii(MQeQueueAdminMsg.Queue_QMgrName, parms.getAscii(MQeQueueAdminMsg.Queue_QMgrName));
            break;

        case AdminQueue.Action_Browse:
        case AdminQueue.Action_PutTestMessage:
        case AdminQueue.Action_GetMessage:
            // not actions on an admin message)
            break;
        }
        return (MQeQueueAdminMsg) msg;
    }

    /**
     * Configure the valid set of actions.
     * @param none
     * @return void
     */
    protected void setupActions() {
        super.setupActions();

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_Create),
            getAddCharacteristics());

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_Update),
            getUpdateCharacteristics());

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_InquireAll),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
            }
        );

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_Delete),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
            }
        );

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_AddAlias),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
                { AdminQueue.Queue_Alias, "M" }
            }
        );

        addAction(new String().valueOf(MQeQueueAdminMsg.Action_RemoveAlias),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
                { AdminQueue.Queue_Alias, "M" }
            }
        );

        addAction(new String().valueOf(AdminQueue.Action_Browse),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
            }
        );

        addAction(new String().valueOf(AdminQueue.Action_PutTestMessage),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
            }
        );

        addAction(new String().valueOf(AdminQueue.Action_GetMessage),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
            }
        );

    }

    /**
     * Setup the valid set of characteristics.
     * <p>The sequence is important as this is the sequence that the fields
     * will displayed.
     * @param none
     * @return void
     */
    protected void setupFields() {
        super.setupFields();

        addCharacteristic(MQeQueueAdminMsg.Admin_Name, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_QMgrName, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Description, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Active, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Mode, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Admin_Class, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_FileDesc, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Cryptor, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Authenticator, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Compressor, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_TargetRegistry, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_AttrRule, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Rule, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Expiry, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_MaxMsgSize, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_Priority, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_MaxQSize, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_CurrentSize, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_CreationDate, "TF", null);
        addCharacteristic(MQeQueueAdminMsg.Queue_QAliasNameList, "C", null);
        addCharacteristic(AdminQueue.Queue_Alias, "TF", null);
    }

    /**
     * Setup NLS names.
     * @param none
     * @return void  
     */
    protected void setupNLS() {
        super.setupNLS();

        addNLSName(MQeQueueAdminMsg.Admin_Class, "Class Name *");
        addNLSName(MQeQueueAdminMsg.Admin_Name, "Queue Name");
        addNLSName(MQeQueueAdminMsg.Queue_QMgrName, "Queue Manager Name");
        addNLSName(MQeQueueAdminMsg.Queue_Active, "Active *");
        addNLSName(MQeQueueAdminMsg.Queue_Description, "Description");
        addNLSName(MQeQueueAdminMsg.Queue_Mode, "Access mode (1=Async, 2=Sync)");
        addNLSName(MQeQueueAdminMsg.Queue_FileDesc, "Queue store *");
        addNLSName(MQeQueueAdminMsg.Queue_Cryptor, "Cryptor class *");
        addNLSName(MQeQueueAdminMsg.Queue_Authenticator, "Authenticator class *");
        addNLSName(MQeQueueAdminMsg.Queue_Compressor, "Compressor class *");
        addNLSName(MQeQueueAdminMsg.Queue_TargetRegistry, "Target registry (for WTLS) *");
        addNLSName(MQeQueueAdminMsg.Queue_AttrRule, "Attribute rule class *");
        addNLSName(MQeQueueAdminMsg.Queue_Rule, "Rule class *");
        addNLSName(MQeQueueAdminMsg.Queue_MaxMsgSize, "Maximum message length *");
        addNLSName(MQeQueueAdminMsg.Queue_Priority, "Default priority (0-9) *");
        addNLSName(MQeQueueAdminMsg.Queue_MaxQSize, "Maximum queue depth *");
        addNLSName(MQeQueueAdminMsg.Queue_CurrentSize, "Current queue depth");
        addNLSName(MQeQueueAdminMsg.Queue_Expiry, "Message expiry time *");
        addNLSName(MQeQueueAdminMsg.Queue_CreationDate, "Creation date");
        addNLSName(MQeQueueAdminMsg.Queue_QAliasNameList, "Aliases *");

        addNLSName(AdminQueue.Queue_Alias, "Alias");

        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_InquireAll), "Inquire");
        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_Update), "Update");
        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_Create), "Add");
        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_Delete), "Remove");
        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_AddAlias), "Add alias");
        addNLSName((new String()).valueOf(MQeQueueAdminMsg.Action_RemoveAlias), "Remove alias");
        addNLSName((new String()).valueOf(AdminQueue.Action_Browse), "Browse");
        addNLSName((new String()).valueOf(AdminQueue.Action_PutTestMessage), "Put Test Message");
        addNLSName((new String()).valueOf(AdminQueue.Action_GetMessage), "Get Message");

        addNLSName("cancel", "Cancel");

    }

    /**
     * Send a test message to the specified q on the specified q manager.
     * @param qName String containing the name of the Queue
     * @param qMgrName String containing the name of the QueueManager
     * @return void
     * @exception java.lang.Exception
     */
    protected void sendTestMessage(String qName, String qMgrName
    ) throws Exception {
        MQeMsgObject msg = new MQeMsgObject();

        msg.putUnicode("Test_Unicode", "This is a Unicode String");
        msg.putInt("Test_Int", 123);
        msg.putArrayOfByte("Test_Byte_Array", new byte[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 }
        );
        msg.putBoolean("Test_Boolean", true);

        // Put the message to the queue...
        adminModel.getQM().putMessage(qMgrName                     // Queue manager we're sending to.
            , qName                       // Bridge queue we're sending to.
            , msg                         // Message we're sending.
            , null                        // Attributes of the message.
            , (long) 0                     // We're not sending a confirm.
        );
    }

    /**
     * Get a message from the specified queue on the specified queue manager.
     * @param qName String containing the name of the Queue
     * @param qMgrName String containing the name of the QueueManager
     * @return MQeMsgObject object
     * @exception java.lang.Exception
     */
    protected MQeMsgObject getTestMessage(String qName, String qMgrName
    ) throws Exception {
        // Get the message from the queue...
        return adminModel.getQM().getMessage(qMgrName              // Queue manager we're getting from.
                , qName                // Queue we're getting from.
                , null                 // Filter.
                , null                 // Attributes of the message.
                , (long) 0              // We're not sending a confirm.
            );
    }

    /**
     * Returns a string that provides information about the current frame.
     * This is displayed in the south panel of the main admin gui when a frame is selected.
     * @param none
     * @return String documenting the current frame.
     */
    public String getMessageHelp() {
        return "Local Queues are message stores that are defined within the current Queue Manager.";
    }

}
