// Version mqe_java/source/examples/administration/console/AdminQueueBrowser.java, MQeExamples, la200, la200-L030324.1 1.8

/* -----------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.administration.console;


import com.ibm.mqe.*;
import java.util.*;
import java.awt.*;
import examples.awt.*;


/**                                                                             
 * To provide a simple queue browser. Provides several views of the        
 * queue, including a list of messages plus a dump of the message contents.             
 *                                                                              
 */

public class AdminQueueBrowser extends AwtFrame
    implements MQeMessageListenerInterface, java.beans.PropertyChangeListener {
    public    static short  version[] = { 2, 0, 0, 5 }; 

    protected boolean       active = false;        // Initialised ?
    protected AdminModel    adminModel = null;         // Admin model ...
    protected Vector        fieldMsgs = new Vector(); // Sequence of messages

    /**
     * GUI widgets
     */
    protected java.awt.List  msgList;   // Display messages in list
    protected TextArea  msgArea;        // Display messages in area
    protected Choice    filterChoice;   // Filter message selection
    protected Choice    dispModeChoice; // View style
    protected TextField qMgrTF;         // Name of queue manager
    protected TextField qTF;            // Name of queue being browsed
    protected TextField filterTF;       // The filter
    protected TextField messageTF;      // Messages output here
    protected Label msgCountTF;         // Count of messages being browsed

    protected BorderLayout  southLOM = new BorderLayout(); // South layout manager
    protected Panel         southPan = new Panel();         // Main south panel
    protected Panel         actionPan = new Panel();     // action buttons panel
    protected Panel         msgPan = new Panel();     // Message panel

    /**
     * Variables used in filtering messages.
     * filterValues = Possible fields to filter on
     * filterField  = Filter fields object
     */
    protected static String[]   filterValues = { "MessageID", "CorrelationID" };
    protected MQeFields         filterField = new MQeFields();

    /**
     * Variables used to handle different display modes.
     * dmode:       Possible dislplay modes
     * dispMode:    Current display mode
     */
    protected static String[]   dMode = { "List", "Full", "Both" };
    protected int               dispMode = 0;

    /**
     * Other Variables used to hold user input.
     */
    protected String qMgrName;          // Name of queue manager
    protected String qName;             // Name of queue being browsed
    protected String filter;            // Message filter

    /**
     * AdminLogModel default constructor.
     */
    public AdminQueueBrowser() {
        super("Queue browser");
    }

    /**
     * AdminLogModel constructor.
     * @param admin  an AdminModel to provide base admin functionality
     * @param qMgr   Name of queue manager on which queue resides
     * @param q      Name of queue to browse
     */
    public AdminQueueBrowser(AdminModel admin, String qMgr, String q) {
        super("Queue browser");

        setQMgrName(qMgr);
        setQueueName(q);

        activate(admin);
    }

    /**
     * AdminLogModel constructor.
     * @param title window title
     */
    public AdminQueueBrowser(String title) {
        super(title);
    }

    /**
     * Process an MQe awt event.
     * @param e java.lang.Object
     * @param where An integer indicating the location.
     * @param index An integer containing the type of resource to be managed.
     * @param choice java.lang.String
     * @param state A boolean
     * @return void
     */
    public void action(Object e, int where, int index, String  choice, boolean state) {
        try {
            // get the fields from the GUI and update admin Model
            getInput();

            // setup the filter
            setFilter(filter, filterChoice.getSelectedIndex());

            // clear last message
            message("");

            switch (where) {
            case North:

                /**
                 * Queue manager name, filter, display mode panel
                 */
                if (e == dispModeChoice) {
                    // Change the display mode
                    setDisplayMode(dispModeChoice.getSelectedIndex());
                    refresh();
                }
                break;

            case Center:

                /**
                 * Display message list and details panels
                 */
                if (getDisplayMode() == 0 || getDisplayMode() == 2) {
                    // If message has been selected in the list, display it.
                    MQeFields m = (MQeFields) fieldMsgs.elementAt(msgList.getSelectedIndex());

                    displayMessage(m);
                }
                break;

            case South:

                /**
                 *  Action buttons panel
                 */
                switch (index) {
                case 0:                 // Refresh
                    refresh();
                    break;

                case 1:                 // empty the log
                    emptyQueue();
                    break;

                case 2:                 // cancel
                    close();
                    break;
                }
                break;
            }
        } catch (Exception ex) {
            message(ex.toString());
        }
    }

    /**
     * Initialisation code.
     * @param admin AdminModel object
     * @return void
     */
    public void activate(AdminModel admin) {
        adminModel = admin;     // Remeber which admin model we are using

        if (!active) {

            /**
             * One off initialisation code
             */

            // Setup the queue browse criteria panel
            format(North, new String[][][]
                {
                    { { "L", "Queue"              },              // 0
                        { "TP", getQMgrName()       },              // 1
                        { "TP", getQueueName()      }
                    },            // 2
                    { { "L", "Filter"             },              // 3
                        { "D", filterValues[0], filterValues[1] },  // 4
                        { "T", ""                   }
                    },            // 5
                    { { "L", "Display messages"   },              // 6
                        { "D", dMode[0], dMode[1], dMode[2] },      // 7
                        { "L", "Count"                  }
                    }         // 8
                }
            );

            // Setup the queue/message display panels
            format(Center, new String[][][]
                { {
                        { "S", "logged messages"   },          // 0
                        { "AP", "logged messages"   }
                    }         // 1
                }
            );

            // The south panel for buttons and messages
            southPan.setLayout(southLOM);

            // setup action buttons panel
            AwtFormat southF1 = new AwtFormat(this, South);

            actionPan = southF1.panel(actionPan, new String[][][]
                        {
                            { { "B", "Refresh"      },              // 0
                                { "B", "Empty Queue"  },             // 1
                                { "B", "Cancel"       }
                            },            // 2
                        }
                    );

            // setup message panel
            AwtFormat southF2 = new AwtFormat(this, South);

            msgPan = southF2.panel(msgPan, new String[][][]
                        {
                            { { "TP", "Message"             }
                            }      // 3
                        }
                    );

            // Add action and msg panel to main south panel
            southPan.add(actionPan, java.awt.BorderLayout.NORTH);
            southPan.add(msgPan, java.awt.BorderLayout.SOUTH);

            setPanel(South, southPan);    // Add South panel to frame
            visible(South, true);        // Ensure it can be seen

            // Get references to some of the GUI widgets
            qMgrTF = (java.awt.TextField) getObject('N', 1);
            qTF = (java.awt.TextField) getObject('N', 2);
            filterChoice = (java.awt.Choice) getObject('N', 4);
            filterTF = (java.awt.TextField) getObject('N', 5);
            dispModeChoice = (java.awt.Choice) getObject('N', 7);
            msgList = (java.awt.List) getObject('C', 0);
            msgArea = (java.awt.TextArea) getObject('C', 1);
            msgCountTF = (java.awt.Label) getObject('N', 8);
            messageTF = (java.awt.TextField) southF2.getObject(0);

            msgCountTF.setAlignment(java.awt.Label.CENTER);

            // Use a fixed font for list display so that columns line up
            msgList.setFont(new java.awt.Font("monospaced", 0, 11));
            msgArea.setFont(new java.awt.Font("monospaced", 0, 11));

            // Create a an item listener to handle single clicks
            java.awt.event.ItemListener il = new java.awt.event.ItemListener() {
                    public void itemStateChanged(java.awt.event.ItemEvent e) {
                        if (e.getSource() instanceof java.awt.List) {
                            MQeFields m = (MQeFields) fieldMsgs.elementAt(msgList.getSelectedIndex());

                            msgArea.setText(parseMessageForArea(m));
                        }
                    }
                };

            msgList.addItemListener(il);

            setDisplayMode(0);            // Default to list display mode
            visible(true);                // Display it

            // Display initial contents of queue
            refresh();

            // Start the queue monitor (if local)
            startMonitor();

            // 1st Pass complete ...
            active = true;
        }

        // Listen for changes to any base properties
        adminModel.addPropertyChangeListener(this);
    }

    /**
     * Add a new message to list of messages.
     * @param msg Message to be added
     * @return void
     */
    public synchronized void addMessage(MQeFields msg) {
        fieldMsgs.addElement(msg);        // Add to the list of messages

        if (getDisplayMode() == 0 || getDisplayMode() == 2) {
            // Display a subset of message on a single line
            msgList.add(parseMessage(msg));
        } else if (getDisplayMode() == 1) {
            // Display message contents
            msgArea.append("===== Message: " + getMsgsDisplayed() + " =======");
            msgArea.append(parseMessage(msg));
        }

        // Update message count
        msgCountTF.setText(new Integer(getMsgsDisplayed()).toString());
    }

    /**
     * The browse operations gets admin messages from a queue that match a filter.
     * Messages matching the filter are then displayed.
     * @param match MQeFields object
     * @return void
     */
    public void browse(MQeFields match) {
        try {
            // Get messages that match the filter
            MQeEnumeration recs = getQM().browseMessages(getQMgrName(),
                    getQueueName(),
                    match,
                    null,
                    false);

            // Process them ...
            while (recs.hasMoreElements()) {
                // get the next message
                MQeFields baseRec = (MQeFields) recs.nextElement();

                // add it to display
                addMessage(baseRec);
            }
        } catch (Exception e) {
            message("Browse failed: " + e);
        }
    }

    /**
     * Clear messages from display buffers.
     * @param none
     * @return void
     */
    public void clearMsgs() {
        // Clear the vector of messages
        fieldMsgs.removeAllElements();

        // Clear data from the GUI widgets
        switch (getDisplayMode()) {
        case 0:
            msgList.removeAll();
            break;

        case 1:
            msgArea.setText("");
            break;

        case 2:
            msgList.removeAll();
            msgArea.setText("");
            break;
        }

        // Reset the messages displayed count
        msgCountTF.setText(new Integer(getMsgsDisplayed()).toString());
    }

    /**
     * Close the window.
     * @param none
     * @return void
     */
    public void close() {
        adminModel.removePropertyChangeListener(this);
        stopMonitor();
        dispose();                    // finished with the window
    }

    /**
     * Handel window closing event.
     * @param none
     * @return void
     */
    protected void closeFrame() {
        close();
    }

    /**
     * Open a window to display contents of the message.
     * @param msg Message to be displayed
     * @return void
     * @exception java.lang.Exception
     */
    public void displayMessage(MQeFields msg) throws Exception {
        if (msg instanceof MQeAdminMsg)
            adminModel.displayMsg((MQeAdminMsg) msg, ((MQeAdminMsg) msg).getAction(), null);
        else
            message("Can only display admin messages");
    }

    /**
     * Delete all messages from the queue being browsed.
     * @param none
     * @return void
     */
    public void emptyQueue() {
        try {
            // Get all messages
            MQeEnumeration recs = getQM().browseMessages(getQMgrName(),
                    getQueueName(),
                    null,
                    null,
                    true);

            // Process them ...
            while (recs.hasMoreElements()) {
                // get the next message
                MQeFields match = (MQeFields) recs.nextElement();

                getQM().deleteMessage(getQMgrName(), getQueueName(), match);
            }

            refresh();
        } catch (Exception e) {
            message("Empty queue failed: " + e);
        }
    }

    /**
     * Returns the display mode.
     * @param none
     * @return An integer with one of the following values.
     * <pre>
     *  0: List
     *  1: Text area
     *  2: Both ( 2 panels, 1 with a list and 1 with text area )
     * </pre>
     */
    public int getDisplayMode() {
        return dispMode;
    }

    /**
     * Get user input from GUI widgets.
     * @param none
     * @return void
     */
    public void getInput() {
        filter = filterTF.getText();   // Filter data
        qMgrName = qMgrTF.getText();     // Queue manager name
        qName = qTF.getText();        // Queue name
    }

    /**
     * Returns the number of messages that are in the display buffer.
     * @param none
     * @return An integer value
     */
    public int getMsgsDisplayed() {
        return fieldMsgs.size();
    }

    /**
     * Get a reference to the queue manager.
     * @param none
     * @return MQeQueueManager object
     */
    public MQeQueueManager getQM() {
        return adminModel.getQM();
    }

    /**
     * Gets the queue manager name.
     * @param none
     * @return java.lang.String
     */
    public String getQMgrName() {
        try {
            // If the queue manager name has not been specified, return
            // the local queue manager name
            if (qMgrName == null || qMgrName.equals(""))
                qMgrName = getQM().getName();
        } catch (Exception ex) {}

        return qMgrName;
    }

    /**
     * Gets the queue name.
     * @param none
     * @return java.lang.String
     */
    public String getQueueName() {
        return qName;
    }

    /**
     * Display a message.
     * @param msg java.lang.String containing the message
     * @return void
     */
    public void message(String msg) {
        if (messageTF != null)
            messageTF.setText(msg);
    }

    /**
     * Process a message arrived event.
     * @param e MQeMessageEvent object
     * @return void
     */
    public void messageArrived(MQeMessageEvent e) {
        try {
            // Get the UID of the message
            MQeFields match = e.getMsgFields();

            // setup a match consisting of the UID plus the filter
            match.copy(filterField, true);

            // now browse the message on the queue with the match.
            // ( the browse operation will read the entire message
            // and add it to the display )
            browse(match);
        } catch (Exception ex) {
            message("MessageArrived: dodgy message " + ex);
        }

    }

    /**
     * Parse the message ready for display.
     * @param msg Message object to be parsed
     * @return Parsed message
     */
    public String parseMessage(MQeFields msg) {
        String ps = null;

        switch (getDisplayMode()) {
        case 0:
            ps = parseMessageForList(msg);
            break;

        case 1:
            ps = parseMessageForArea(msg);
            break;

        case 2:
            ps = parseMessageForList(msg);
            break;
        }
        return ps;
    }

    /**
     * Parse a message for display in an area that can
     * handle multiple lines of text.
     * @param msg Message object to be parsed.
     * @return Parsed message
     */
    public String parseMessageForArea(MQeFields msg) {
        String msgS;

        try {
            msgS = msg.dumpToString(" " + "  #0\t#1\t=#2\r\n");
            msgS = msgS + "\r\n";
        } catch (Exception ex) {
            msgS = "Error parsing msg: " + ex;
        }

        return msgS;
    }

    /**
     * Parse a message for display in a single line.
     * @param pmsg Message object to be parsed
     * @return Parsed message
     */
    public String parseMessageForList(MQeFields pmsg) {
        MQeMsgObject msg = (MQeMsgObject) pmsg;      // Case to a message object

        StringBuffer sb = new StringBuffer(150);    // buffer to hold parsed msg

        // Initialise buffer to blanks
        sb.setLength(150);
        for (int i = 0; i < 150; i++) {
            sb.setCharAt(i, ' ');
        }

        String msgID = "no MsgID";
        String correlID = "No CorrelID";
        String startQM = "Origin QM?";
        String style = "= Datagram";

        try {
            // MessageID
            if (msg.contains(MQe.Msg_MsgID))
                msgID = (MQe.byteToHex(msg.getArrayOfByte(MQe.Msg_MsgID)));

            // CorrelID
            if (msg.contains(MQe.Msg_CorrelID))
                correlID = (MQe.byteToHex(msg.getArrayOfByte(MQe.Msg_CorrelID)));

            // initiating queue manager
            if (msg.contains(MQe.Msg_OriginQMgr))
                startQM = (msg.getOriginQMgr());

            // Message style
            if (msg.contains(MQe.Msg_Style))
                if (msg.getInt(MQe.Msg_Style) == MQe.Msg_Style_Request)
                    style = "> Request";
                else if (msg.getInt(MQe.Msg_Style) == MQe.Msg_Style_Reply)
                    style = "< Response";

            String type = MQe.abbreviate(msg.type(), 0);
            int idot = type.lastIndexOf(".");

            type = type.substring(idot + 1, type.length());

            // Put data in buffer
            sb.insert(0, style);
            sb.insert(11, ' ');
            sb.insert(12, startQM);
            sb.insert(39, ' ');
            sb.insert(40, type);
            sb.insert(69, ' ');
            sb.insert(70, msgID);
            sb.insert(89, ' ');
            sb.insert(90, correlID);

            // Reset the buffer length
            sb.setLength(150);
        } catch (Exception e) {
            sb.append("Failed to parse message: " + e.toString());
        }

        // Reset buffer to correct length
        sb.setLength(150);

        return sb.toString();
    }

    /**
     * Method to handle events for the PropertyChangeListener interface.
     * @param evt java.beans.PropertyChangeEvent
     * @return void
     */
    public void propertyChange(java.beans.PropertyChangeEvent evt) {
        if ((evt.getSource() == adminModel) && (evt.getPropertyName().equals("connected"))) {
            // Connection to queue manager state has changed
            if (adminModel.getConnected()) {
                if (adminModel.getMode() == false)
                    startMonitor();
            } else
                stopMonitor();
        }
    }

    /**
     * Refreshes the display and then browses the queue.
     * @param none
     * @return void
     */
    public void refresh() {
        clearMsgs();
        browse(filterField);
    }

    /**
     * Set the display mode to index value.
     * @param index This integer can take one the following values:
     * <pre>
     *   0  display messages in a list. A subset of message fields displayed.
     *   1  dispaly messages in a textarea. All fields in a message are displayed.
     *   2  display 2 panels:  a list of messages in one and a full display of the
     *                         selected message in the other.
     * </pre>
     * @return void
     */
    public void setDisplayMode(int index) {

        dispMode = index;
        Panel cPanel = getPanel(Center);

        switch (dispMode) {
        case 0:
            cPanel.add(msgList, null, 0);
            cPanel.remove(msgArea);
            break;

        case 1:
            cPanel.remove(msgList);
            cPanel.add(msgArea, null, 0);
            break;

        case 2:
            cPanel.add(msgList, null, 0);
            cPanel.add(msgArea, null, 1);
            break;
        }

        // Jog the display to correct layout wierdness
        // should not be necessary in a perfect world !
        if (active)
            setVisible(true);
    }

    /**
     * Setup the filter fields object.
     * @param filter java.lang.String
     * @param index An integer
     * @return void
     * @exception java.lang.Exception
     */
    public void setFilter(String filter, int index) throws Exception {
        filterField.restore(null);    // Clear out the filter

        if (!filter.equals("")) {
            switch (index) {
            case 0:
                // Filter by message id. Assumes that MsgID is a string which
                // is encoded as a byte array
                filterField.putArrayOfByte(MQe.Msg_MsgID, filter.getBytes());
                break;

            case 1:
                // Filter by correlation id. Assumes that CorrelID is a string which
                // is encoded as a byte array
                filterField.putArrayOfByte(MQe.Msg_CorrelID, filter.getBytes());
                break;
            }
        }
    }

    /**
     * Set the name of the queue manager which owns the queue to be browsed.
     * @param qMgrName  String holding the QMgr Name
     * 
     * @return void
     */
    public void setQMgrName(String qMgrName) {
        this.qMgrName = qMgrName;
    }

    /**
     * Set the name of the queue to be browsed.
     * @param qMgrName  String holding the Q Name
     * 
     * @return void
     */
    public void setQueueName(String qMgrName) {
        this.qName = qMgrName;
    }

    /**
     * Start the queue monitor.
     * @param none
     * @return void
     */
    public void startMonitor() {
        try {
            if (adminModel.getConnected()) {
                // Only start the monitor if the queue being monitored is a local queue
                if (getQMgrName().equals(getQM().getName())) {
                    getQM().addMessageListener(this, getQueueName(), null);
                    message("Started monitor: ");
                } else
                    message("Monitor of remote queue not supported");
            } else
                message("Not connected to queue manager");
        } catch (Exception e) {
            message("Failed to start monitor: " + e);
        }
    }

    /**
     * Stop the queue monitor.
     * @param none
     * @return void
     */
    public void stopMonitor() {
        try {
            if (getQM() != null) {
                getQM().removeMessageListener(this, getQueueName(), null);
            }

            message("Stopped monitor: ");
        } catch (Exception e) {
            message("Failed to stop monitor: " + e);
        }
    }
}
