package examples.administration.simple;


// Version mqe_java/source/examples/administration/simple/Example2.java, MQeExamples, la200, la200-L030324.1 1.12
/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.application.*;


/***
 * Adds routes to remote queue managers.
 *
 * Example showing how to:
 * <pre>
 *  . start a queue manager
 *  . add routes to 2 remote queue managers
 *
 * </pre>
 *
 **/
public class Example2 extends ExampleAdminBase {
    public static short[] version = { 2, 0, 0, 5 }; 
    String routeDest1 = "ServerQM1";
    String routeAddress1 = "FastNetwork://127.0.0.1:8082";
    String routeCommand1 = null;
    String routeOptions1 = null;
    String routeDest2 = "ServerQM";
    String routeAddress2 = "FastNetwork://127.0.0.1:8083";
    String routeCommand2 = "/servlet/MQe";
    String routeOptions2 = null;

    /**
     * Constructor that uses the default queue manager.
     * @exception java.lang.Exception
     */
    public Example2() throws Exception {
        super();
    }

    /**
     * Constructor that uses the queue manager passed in on the command line.
     * @param qmName java.lang.String
     * @exception java.lang.Exception
     */
    public Example2(String queueManagerName) throws Exception {
        super(queueManagerName);
    }

    /**
     * Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String, queue manager name.
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMIniFile java.lang.String, ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception java.lang.Exception
     */
    public Example2(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * Creates a new queue.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    protected void addRoutes() throws Exception {
        System.out.println("..Setup an admin message to add some routes ");

        /*
         * Create an empty queue manager admin message and parameters field
         */
        MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();

        /*
         * Prime message with who to reply to and a unique identifier
         */
        MQeFields msgTest = primeAdminMsg(msg);

        /*
         * Set name of queue manager to add routes to
         */
        msg.setName(routeDest1);

        /*
         * Set the admin action to create a new queue
         * The connection is setup to use a default channel. This is an alias
         * which must have be setup on the queue manager for the connection to be
         * valid.
         */
        msg.create(routeAddress1, routeCommand1, routeOptions1, "DefaultChannel", "Example route 1");

        /*
         *  Put the admin message to the admin queue
         */
        System.out.println("..Put admin message to QM/queue: " + qMgrName + "/" + MQe.Admin_Queue_Name);
        myQM.putMessage(qMgrName, MQe.Admin_Queue_Name, msg, null, 0);

        /*
         * Wait a while for the response message
         */
        MQeAdminMsg respMsg = waitForReply(msgTest);

        checkReply(respMsg);

        System.out.println("..Add Routes Successful");
    }

    /**
     * All work is performed in this method.
     * @param none
     * @return void
     * @exception java.lang.Exception
     */
    public void doIt() throws Exception {
        try {

            /*
             * Create a queue
             */
            addRoutes();

            System.out.println(example + " Finished");
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed! " + e);
            throw e;
        } finally {

            /*
             * Clean up
             */
            close();
        }
    }

    /**
     * Starts the application.
     * @param param1  Queue Manager Name to interact with.
     * If queue manager name matches the local QM name then
     * all interactions are local otherwise the local QM acts
     * as a stub QM sending all messages to a remote QM.
     * @param param2 Local queue manager startup parameters ini file
     *
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new Example2().doIt();
            else if (args.length == 1)
                new Example2(args[0]).doIt();
            else
                new Example2(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);
    }

    /**
     * Over-ride default parameter settings.
     * @param none
     * @return void
     */
    public void setParms() {
        super.setParms();
        example = "Example2";
        msgData = example + ": Mary Mary quite contrary";
    }
}
