package examples.application;


// Version mqe_java/source/examples/application/Example2.java, MQeExamples, la200, la200-L030324.1 1.8
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
import com.ibm.mqe.*;


/*
 * <p>Extends ExampleBase,Application example base.
 *
 * <p> Put a series of messages then get a specific message example.
 *
 * <p>Example showing how to:
 * <pre>
 *  . start a queue manager
 *  . put 3 messages to its systems default queue
 *  . get the second of the messages (using a match field)
 *  . browse the queue to locate any remaining messages
 *  . remove any remaining messages.
 *  </pre>
 *  <p>The example relies on the queue containing no messages at the time
 *  when the example is run.
 *
 *
 **/
public class Example2 extends ExampleBase {
    public static short[] version = { 2, 0, 0, 5 }; 
    String msgData1 = example + ":Msg1, Jack & Jill went up the hill...";
    String msgData2 = example + ":Msg2, to fetch a pale of water";
    String msgData3 = example + ":Msg3, Jack fell down and broke his crown";

    /**
     * Constructor that uses the default queue manager.
     * @exception java.lang.Exception
     */
    public Example2() throws Exception {
        super();
    }

    /**
     * Constructor that uses the queue manager passed in on the command line.
     * @param queueManagerName java.lang.String
     * @exception java.lang.Exception
     */
    public Example2(String queueManagerName) throws Exception {
        super(queueManagerName);
    }

    /**
     * Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String,queue manager name
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMgrIniFile java.lang.String - ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception java.lang.Exception
     */
    public Example2(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * All work is performed in this method.
     * @return void
     * @exception java.lang.Exception
     */
    public void doIt() throws Exception {
        try {

            /*
             * Create a msg with some data
             */
            System.out.println("..Create a message with data:" + msgData1);
            MQeMsgObject msgObj1 = new MQeMsgObject();

            msgObj1.putAscii(msgField, msgData1);

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj1, null, 0);

            /*
             * Create a msg with some data
             */
            System.out.println("..Create a message with data:" + msgData2);
            MQeMsgObject msgObj2 = new MQeMsgObject();

            msgObj2.putAscii(msgField, msgData2);

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj2, null, 0);

            /*
             * Create a msg with some data
             */
            System.out.println("..Create a message with data:" + msgData3);
            MQeMsgObject msgObj3 = new MQeMsgObject();

            msgObj3.putAscii(msgField, msgData3);

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj3, null, 0);

            /*
             * Create a Fields object to match message 2 based on its unique id
             */
            MQeFields match = msgObj2.getMsgUIDFields();

            /*
             * Retrieve the the second message
             */
            System.out.println("..Get 2nd message QM/queue: " + qMgrName + "/" + queueName);
            MQeMsgObject msgObj4 = myQM.getMessage(qMgrName, queueName, match, null, 0);

            System.out.println("..Message retrieved contains data " + msgObj4.getAscii(msgField));

            /*
             * Tidy up by removing the 2 remaining messages
             */
            emptyQueue();

            /*
             * Check if it contains the expected data
             */
            if (msgObj4.getAscii(msgField).equals(msgData2))
                System.out.println(example + " Finished");
            else
                System.out.println(example + " Failed: message data does not match");

        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed! " + e);
            throw e;
        } finally {

            /*
             * Clean up
             */
            close();
        }
    }

    /**
     * <p>Starts the application.
     * @param param1 Queue Manager Name to interact with.
     *                      If queue manager name matches the local QM name then
     *                      all interactions are local otherwise the local QM acts
     *                      as a stub QM sending all messages to a remote QM.
     * @param param2 Local queue manager startup parameters ini file.
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new Example2().doIt();
            else if (args.length == 1)
                new Example2(args[0]).doIt();
            else
                new Example2(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);
    }

    /**
     * Over-ride default parameter settings.
     */
    public void setParms() {
        example = "Example2";
    }
}
