// Version mqe_java/source/examples/application/Example5.java, MQeExamples, la200, la200-L030324.1 1.9

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.application;


import com.ibm.mqe.*;


/****
 * <p>Extends ExampleBase, Application example base.
 *
 * <p>Message Locking example.
 *
 * <p>Example showing how to:
 * <pre>
 *  . start a queue manager
 *  . put 3 messages
 *  . browse and lock the messages
 *  . get first locked message
 *  . delete locked message
 *  . unlock message
 *  . get unlocked message to clean up
 * </pre>
 *
 ****/

public class Example5 extends ExampleBase {
    public static short[] version = { 2, 0, 0, 5 }; 
    String msgData1 = example + ": first locked message";
    String msgData2 = example + ": second locked message";
    String msgData3 = example + ": third locked message";
    String id1 = "1";
    String id2 = "2";
    String id3 = "3";

    /**
     * Constructor that uses the default queue manager.
     * @exception java.lang.Exception
     */
    public Example5() throws Exception {
        super();
    }

    /**
     * Constructor that uses the queue manager passed in on the command line.
     * @param queueManagerName java.lang.String
     * @exception  java.lang.Exception
     */
    public Example5(String queueManagerName) throws Exception {
        super(queueManagerName);
    }

    /**
     * Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String, queue manager name
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMgrIniFile java.lang.String, ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception  java.lang.Exception
     */
    public Example5(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * All work is performed in this method.
     * @return void
     * @exception  java.lang.Exception
     */
    public void doIt() throws Exception {
        try {

            /*
             *  Get a unique identifer for use in this example.
             */
            long unique = MQe.uniqueValue();
            MQeFields uniqueID = new MQeFields();

            uniqueID.putLong("Unique", unique);

            /*
             * Create a msg with some data and well known identifier
             */
            System.out.println("..Create a message with id: " + id1 + " and data:" + msgData1);
            MQeMsgObject msgObj1 = new MQeMsgObject();

            msgObj1.putAscii(msgField, msgData1);
            msgObj1.putLong("Unique", unique);
            msgObj1.putArrayOfByte(MQe.Msg_MsgID, asciiToByte(id1));

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj1, null, 0);

            /*
             * Create a msg with some data and well known identifier
             */
            System.out.println("..Create a message with id: " + id2 + " and data:" + msgData2);
            MQeMsgObject msgObj2 = new MQeMsgObject();

            msgObj2.putAscii(msgField, msgData2);
            msgObj2.putLong("Unique", unique);
            msgObj2.putArrayOfByte(MQe.Msg_MsgID, asciiToByte(id2));

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj2, null, 0);

            /*
             * Create a msg with some data and well known identifier
             */
            System.out.println("..Create a message with id: " + id3 + " and data:" + msgData3);
            MQeMsgObject msgObj3 = new MQeMsgObject();

            msgObj3.putAscii(msgField, msgData3);
            msgObj3.putLong("Unique", unique);
            msgObj3.putArrayOfByte(MQe.Msg_MsgID, asciiToByte(id3));

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj3, null, 0);

            /*
             * Lock the messages put on the queue by this example
             */
            System.out.println("..Lock the 3 messages: " + qMgrName + "/" + queueName);
            // Generate a confirmID that can be used to "undo" the lock in the
            // queue managers undo() method.
            long confirmID = MQe.uniqueValue();
            MQeEnumeration msgs = myQM.browseMessagesAndLock(qMgrName, queueName,
                    uniqueID, null,
                    confirmID, false);

            /*
             * Get the lock identifier
             */
            long lock = msgs.getLockId();

            /*
             * Get the unique Ids for each message and add in the lock id
             */
            MQeFields[] uniqueIds = new MQeFields[3];
            int i = 0;

            while (msgs.hasMoreElements()) {
                uniqueIds[i] = ((MQeMsgObject) msgs.nextElement()).getMsgUIDFields();
                uniqueIds[i].putLong(MQe.Msg_LockID, lock);
                i++;
            }

            /*
             * Get first locked message
             */
            MQeMsgObject  msg1 = null;

            System.out.println("..Get 1st locked message QM/queue: " + qMgrName + "/" + queueName);
            msg1 = myQM.getMessage(qMgrName, queueName, uniqueIds[0], null, 0);
            System.out.println("..Message retrieved contains data " + msg1.getAscii(msgField));

            /*
             * Delete the second locked message
             */
            System.out.println("..Delete the second locked message QM/queue: " + qMgrName + "/" + queueName);
            myQM.deleteMessage(qMgrName, queueName, uniqueIds[1]);
            System.out.println("..Deleted the second locked message");

            /*
             * Unlock the third message
             * - must specify lock id and unique id
             */
            System.out.println("..Unlock the third locked message QM/queue: " + qMgrName + "/" + queueName);
            myQM.unlockMessage(qMgrName, queueName, uniqueIds[2]);
            System.out.println("..Unlocked the third locked message");

            /*
             * Tidy up
             */
            emptyQueue();

            System.out.println(example + " Finished");
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed!");
            throw e;
        } finally {

            /*
             * Clean up
             */
            close();
        }
    }

    /**
     * Starts the application.
     * @param param1  Queue Manager Name to interact with.
     *                      If queue manager name matches the local QM name then
     *                      all interactions are local otherwise the local QM acts
     *                      as a stub QM sending all messages to a remote QM.
     * @param param2  Local queue manager startup parameters ini file.
     * @return void
     * @exception  java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new Example5().doIt();
            else if (args.length == 1)
                new Example5(args[0]).doIt();
            else
                new Example5(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);
    }

    /**
     * Over-ride default parameter settings.
     */
    public void setParms() {
        example = "Example5";
    }
}
