package examples.attributes;


// Version mqe_java/source/examples/attributes/NTAuthenticator.java, MQeExamples, la200, la200-L030324.1 1.11
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeFields;
import examples.awt.AwtDialog;
import examples.nativecode.JavaNT;


/**
 *  Provides NT Authentication support.
 *
 * <p><b>NOTE:</b> Requires the following :
 * <ul>
 * <li>The User to be authenticated to be a valid NT User on the m/c owning
 * the target resource.
 * ( use Program->Administration Tools(common)->User Manager ).
 * <li>Ensure the NT Guest user account is disabled.
 * ( otherwise invalid user ids will be able to authenticate via the
 * Guest account ).
 * <li>This User to have the following User Rights set:
 * <pre>
 *         -Act as part of the operating system.
 *         -Logon as locally.
 *         -Logon as service.
 *         ( use Program->Administration Tools(common)->Policies->User Rights,
 *           Show Advanced User Rights ).
 * </pre>
 * <li>JavaNT.dll to be in the PATH (not CLASSPATH) of the m/c owning
 * the target resource.
 * </ul>
 *
 */
public class NTAuthenticator extends LogonAuthenticator {
    public static short version[] = { 2, 0, 0, 5 }; 

    protected final static String NTU_Domain = "D";
    protected final static String NTU_UserId = "U";
    protected final static String NTU_Password = "P";

    protected static examples.nativecode.JavaNT ntDll = null;
    protected int handle = 0;

    /**
     * Closes the authenticator.
     * @param none
     * @return void
     */
    public void close() {
        if ((handle != 0) && (ntDll != null)) {
            ntDll.logoffUser(handle);
        }
        handle = 0;
        super.close();
    }

    /**
     * (Master) Prompt to enter userid an password data.
     * @param fields MQeFields object
     * @return void
     * @exception java.lang.Exception
     */
    protected void prompt(MQeFields fields) throws Exception {
        String domain = ".";
        String userid = "";
        String password = null;

        // display a Dialog - prompting for Domain, Userid and password
        while (userid.equals("")) {

            // Dialog with implicit "OK" and "Cancel" buttons in South panel
            AwtDialog ntd =
                new AwtDialog(
                    "NT Authenticate",
                    AwtDialog.Show_OK_Cancel,
                    new String[][][] { { { "L", "Domain" }, {
                                "T", domain }
                        }, {
                            {
                                "L", "Userid" }, {
                                "T", userid }
                        }, {
                            {
                                "L", "Password" }, {
                                "T*", "" }
                        }
                    }
                );

            if (ntd.getActionIndex(ntd.South) != ntd.Button_OK) {
                throw new MQeException(Except_Stopped, "Canceled");
            }
            domain = mask(ntd.getText(AwtDialog.Center, 1));
            userid = mask(ntd.getText(AwtDialog.Center, 3));
            password = mask(ntd.getText(AwtDialog.Center, 5));
            ntd.dispose();
        }

        // add the Domain, Userid and password data to the fields to be transmitted
        fields.putAscii(NTU_Domain, domain);
        fields.putAscii(NTU_UserId, userid);
        fields.putAscii(NTU_Password, password);
    }

    /**
     * (Slave) Validate the userid and password.
     * @param fields MQeFields object
     * @return String Authenticated ID
     * @exception java.lang.Exception
     */
    protected String validate(MQeFields fields) throws Exception {

        // extract the Domain, Userid and password data from the received fields
        String domain = fields.getAscii(NTU_Domain);
        String userid = fields.getAscii(NTU_UserId);
        String password = fields.getAscii(NTU_Password);

        // make sure that the NT interface DLL is loaded
        if (ntDll == null) {
            ntDll = new examples.nativecode.JavaNT();
        }

        // try to logon using the recieved domain, userid and password
        handle = ntDll.logonUser(mask(domain), mask(userid), mask(password), ntDll.NETWORK);
        if (handle != 0) {
            // set the authenticated id
            return mask(userid);
        }
        throw new MQeException(MQe.Except_Authenticate, "Authenticate failed");
    }

    /**
     * Mask a string.
     * @param data String to be masked
     * @return Masked String
     */
    public String mask(String data) {
        byte temp[] = MQe.asciiToByte(data);

        for (int i = temp.length - 1; i >= 0; i = i - 1) {
            temp[i] = (byte) (temp[i] ^ (i + 0x00A5));
        }
        return MQe.byteToAscii(temp);
    }
}
