package examples.attributes;


// Version mqe_java/source/examples/attributes/UseridAuthenticator.java, MQeExamples, la200, la200-L030324.1 1.12
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeFields;
import examples.awt.AwtDialog;


/**
 *  Provides Authentication support.
 *  <p><b>NOTE:</b>
 *  Needs predefined UserIDS.txt file of format:
 *  <pre>
 *      [UserIDs]
 *      User1Name=User1Password
 *      ..
 *      UserNName=UserNPassword
 * </pre>
 */
public class UseridAuthenticator extends LogonAuthenticator {
    public static short version[] = { 2, 0, 0, 5 }; 

    // change this line to point to the correct userid file
    protected final static String fileName = "." + File.separator + "UserIDs.txt";
    protected final static String userIDname = "UserIDs";

    protected final static String Usr_UserId = "U";
    protected final static String Usr_Password = "P";
    protected final static String Usr_NewPswd = "N";

    /**
     * (Master) prompt to enter userid an password data.
     * 
     * @param fields MQeFields object
     * @return void
     * @exception java.lang.Exception
     */
    protected void prompt(MQeFields fields) throws Exception {
        String userid = "";
        String password = "";
        String newPW = "";
        String cmpPW = "";

        // Pormpt for userid and password - optional change of password
        while (userid.equals("") || (!newPW.equals(cmpPW))) {

            // Dialog with implicit "OK" and "Cancel" buttons in South panel
            AwtDialog ua =
                new AwtDialog(
                    "Userid Authenticate",
                    AwtDialog.Show_OK_Cancel,
                    new String[][][] { { { "L", "Userid" }, {
                                "T", userid }, {
                                "L", "Password" }, {
                                "T*", password }
                        }, {
                            {
                                "L", "New Password" }, {
                                "T*", "" }, {
                                "L", "Confirm     " }, {
                                "T*", "" }
                        }
                    }
                );

            if (ua.getActionIndex(ua.South) != ua.Button_OK) {
                throw new MQeException(Except_Stopped, "Canceled");
            }
            userid = ua.getText(AwtDialog.Center, 1);
            password = ua.getText(AwtDialog.Center, 3);
            newPW = ua.getText(AwtDialog.Center, 5);
            cmpPW = ua.getText(AwtDialog.Center, 7);
            ua.dispose();
        }
        fields.putAscii(Usr_UserId, mask(userid));
        fields.putAscii(Usr_Password, mask(password));

        // only send change of password data if one was entered
        if (!newPW.equals("")) {
            fields.putAscii(Usr_NewPswd, mask(newPW));
        }
    }

    /**
     * (Slave) Validate the userid and password.
     * 
     * @param fields MQeFields object
     * @return Validated string
     * @exception java.lang.Exception
     */
    protected String validate(MQeFields fields) throws Exception {
        String userid = mask(fields.getAscii(Usr_UserId));
        String password = mask(fields.getAscii(Usr_Password));

        // check if the user is defined in the IDs file
        File diskFile = new File(fileName);
        byte data[] = new byte[(int) diskFile.length()];
        FileInputStream inputFile = new FileInputStream(diskFile);

        inputFile.read(data);
        inputFile.close();

        MQeFields section =
            MQeFields.restoreFromString("\r\n", "[#0]", "#1=#2", MQe.byteToAscii(data) + "\r\n");

        MQeFields UserIDs = section.getFields(userIDname);
        String chkPswd = UserIDs.getAscii(userid);

        // compare the passwords
        if (!chkPswd.equals(password)) {
            throw new MQeException(MQe.Except_Authenticate, "Authenticate failed");
        }

        // If there is a new password - update the IDs file
        if (fields.contains(Usr_NewPswd)) {

            UserIDs.putAscii(userid, mask(fields.getAscii(Usr_NewPswd)));
            diskFile = new File(fileName);
            FileOutputStream outputFile = new FileOutputStream(diskFile);
            String result = section.dumpToString("\r\n", "[#0]", "#1=#2");

            outputFile.write(MQe.asciiToByte(result));
            outputFile.close();
        }
        return (userid);
    }

    /**
     * Mask a string.
     * 
     * @param data String to be masked
     * @return Masked string
     */
    public String mask(String data) {
        byte temp[] = MQe.asciiToByte(data);

        for (int i = temp.length - 1; i >= 0; i = i - 1) {
            temp[i] = (byte) (temp[i] ^ (i + 0x0053));
        }
        return (MQe.byteToAscii(temp));
    }
}
