// Version mqe_java/source/examples/awt/AwtDialog.java, MQeExamples, la200, la200-L030324.1 1.6

/* -----------------------------------------------------------------------------*/ 

/*                    WebSphere MQ Everyplace for Multiplatforms                */ 

/*                                                                              */ 

/* Licensed Materials - Property of IBM                                         */ 

/*                                                                              */ 

/* 5765-E63                                                                     */ 

/*                                                                              */ 

/* (C) Copyright IBM Corp. 1999                                                 */ 

/* -----------------------------------------------------------------------------*/ 
package examples.awt;


import  java.awt.*;
import  java.awt.event.*;


/*
 * <p>Displays a Dialog.                                                                                                                                    
 * <p>Notes:                                                                        
 *                                                                               
 *      Dependent on a subset of the function provided by the following          
 *      AWT components:                                                          
 *                                                                               
 *                BorderLayout                                                   
 *                Panel                                                          
 *                                                                                                                         
 ***/ 
public class AwtDialog extends Dialog implements AwtEvent {
    public       static short version[] = { 2, 0, 0, 5 }; 

    public final static char  Center = 'C';
    public final static char  East = 'E';
    public final static char  North = 'N';
    public final static char  South = 'S';
    public final static char  West = 'W';

    public final static int   Button_OK = 1; /* index number of OK button    */
    public final static int   Button_Cancel = 3; /* index number of Cancel button*/
    public final static int   Show_None = 0; /* show no buttons              */
    public final static int   Show_OK = 1; /* show "OK" button             */
    public final static int   Show_OK_Cancel = 2; /* show both buttons            */

    protected BorderLayout    borderLayout1 = new BorderLayout();
    protected AwtFormat       format[] = new AwtFormat[5];
    protected boolean         modalState = false;
    protected Panel           panel[] = new Panel[5];

    protected static String   BorderWhere[] = { BorderLayout.CENTER,
            BorderLayout.NORTH,
            BorderLayout.SOUTH,
            BorderLayout.EAST,
            BorderLayout.WEST };

    /*
     * Constructor ( owner ).                                                     
     *                                                                            
     * @param  Owner  Owning frame                                                  
     *                                                                            
     * @param  Title  The Dialog box window title                                   
     **/
    public AwtDialog(Frame owner, String title) {                                           /*                              */
        super(owner, false);                      /* not modal                    */
        activate(false, title);                   /* initialise                   */
    }                                           /*                              */

    /*
     * Constructor (not owner) Modal - South panel with OK and Cancel buttons.   
     *                                                                            
     * @param    Title  The Dialog box window title                            
     *                                                                            
     * @param   Option The following :
     * <pre> 
     *              Show_None      - no buttons in South panel             
     *              Show_OK        - OK button  added in South panel              
     *              Show_OK_Cancel - both OK and Cancel added in South panel     
     * </pre>                                                                          
     * @param   Items The data for the components in the center of the Dialog      
     **/
    public AwtDialog(String title, int option, String items[][][]) {                                           /*                              */
        this(new Frame(title), title, option, items);   /* create frame         */
    }                                           /*                              */

    /*
     * Constructor (owner) - Modal - South panel with OK and Cancel buttons.
     *                                                                            
     * @param   Owner   Owning frame                                                  
     *                                                                            
     * @param  Title  The Dialog box window title                                   
     *                                                                            
     * @param   Option The following : 
     * <pre>
     *              Show_None      - no buttons in South panel                    
     *              Show_OK        - OK button  added in South panel              
     *              Show_OK_Cancel - both OK and Cancel added in South panel      
     * </pre>                                                                          
     * @param  Items The data for the components in the center of the Dialog.       
     **/
    public AwtDialog(Frame owner, String title, int option, String items[][][]) {                                           /*                              */
        super(owner);                             /* owner frame                  */
        activate(true, title);                    /* initialise                   */
        format(Center, items);                    /* format the center panel      */
        switch (option)                           /* based on the option          */ {                                         /*                              */
        case Show_OK:
            format(South, new String[][][] { { {}, { "B", "OK" },     /* Index 1 */
                        {}
                    }
                }
            );
            break;                                  /* done                         */

        case Show_OK_Cancel:
            format(South, new String[][][] { { {}, { "B", "OK" },     /* Index 1 */
                        {}, { "B", "Cancel" }, /* Index 3 */
                        {}
                    }
                }
            );
            break;                                  /* done                         */
        }                                         /*                              */
        visible(true, true);                      /* show the data                */
    }                                           /*                              */

    /*
     * <p>Called whenever an action occurs.                                           
     *                                                                            
     * <p>Override to provide "event handling".
     * @param e Object
     * @param where Integer indicating where
     * @param index Integer containing the  index value  
     * @param choice String containing the choice 
     * @param state  boolean  containing state  
     * @return void
     *                                    
     **/
    public void action(Object e, int where, int index, String choice, boolean state) {                                           /*                              */
        if (modalState && (e instanceof Button))  /* modal dialog ?               */
            closeDialog();                           /* ... yes, loose the window    */
    }                                           /*                              */

    /*
     * Activate method.                                                           
     *                                                                            
     * @param   modal  boolean.
     * <pre>
     *                  true  = make dialog box modal             
     *                  false = non-modal (i.e. overlapped)          
     * </pre>                                                                           
     * @param   Title   The Dialog box window title    
     * @return void                               
     **/
    protected void activate(boolean modal, String title) {                                           /*                              */
        modalState = modal;                         /* save modal state             */
        enableEvents(AWTEvent.WINDOW_EVENT_MASK); /* get widow events             */
        this.setTitle(title);                     /* set window title             */
        this.setLayout(borderLayout1);            /* set layout                   */
        this.setBackground(Color.lightGray);      /* Dialog background colour     */
    }                                           /*                              */

    /*
     * Handle a window event.
     * @return void                       
     */
    protected synchronized void closeDialog() {                                           /*                              */
        setVisible(false);                        /* hide the window              */
        dispose();                                 /* loose the window             */
    }                                           /*                              */

    /*
     * Format a menu or a panel.
     * @param where Integer indicating where
     * @param items The data for the components 
     * @return void
     */
    public void format(char where, String items[][][]) {                                           /*                              */
        if (items == null)                        /* any data ?                   */
            return;                                   /* ... no                       */
        format[index(where)] = new AwtFormat(this, where);      /* new element  */
        visible(where, true);                     /* make panel visible           */
        format[index(where)].panel(panel[index(where)], items);     /* panel  */
        pack();                                    /* re-pack the window           */
    }                                           /*                              */

    /*
     * Find the index number for the requested panel.
     * @param where Integer indicating where
     * @return an index value.          
     **/
    protected static int index(int where) {                                           /*                              */
        int index = -1;                             /* no index set                 */

        switch (where)                            /* based on where               */ {                                         /*                              */
        case Center:
            index = 0;
            break;           /* place in Center panel        */

        case North:
            index = 1;
            break;           /* place in North  panel        */

        case South:
            index = 2;
            break;           /* place in South  panel        */

        case East:
            index = 3;
            break;           /* place in East   panel        */

        case West:
            index = 4;
            break;           /* place in West   panel        */
        }                                         /*                              */
        return (index);                            /* return the index no.         */
    }                                           /*                              */

    /*
     * Get the Index number of the pressed button. 
     * @param where Integer indicating where
     * @return Index number of the selected button.                              
     **/
    public int getActionIndex(int where) {                                           /*                              */
        return (format[index(where)].getActionIndex()); /* get index number     */
    }                                           /*                              */

    /*
     * Get the String data from the pressed button a Index. 
     * @param where Integer indicating where
     * @return A String containing the selected action.                    
     **/
    public String getActionString(int where) {                                           /*                              */
        return (format[index(where)].getActionString());        /* Action string*/
    }                                           /*                              */

    /*
     * Get the boolean state from the Checkbox at Index. 
     * @param where Integer indicating where
     * @param index Integer containing the  index value    
     * @return A boolean value to know whether the checkbox was checked or not.                         
     **/
    public boolean getCheckState(int where, int index) {                                           /*                              */
        return (format[index(where)].getCheckState(index));    /* Checkbox     */
    }                                           /*                              */

    /*
     * Get the selected item from the Choice box at Index.
     * @param where Integer indicating where
     * @param index Integer containing the  index value    
     * @return A String containing the choice                       
     **/
    public String getChoice(int where, int index) {                                           /*                              */
        return (format[index(where)].getChoice(index));        /* Choise text  */
    }                                           /*                              */

    /*
     * Get the object at the specified Index. 
     * @param where Integer indicating where
     * @param index Integer containing the  index value    
     * @return An object                                 
     **/
    public Object getObject(int where, int index) {                                           /*                              */
        return (format[index(where)].getObject(index));        /* Object ref   */
    }                                           /*                              */

    /*
     * Get the selected panel object. 
     * @param where Integer indicating where  
     * @return A Panel object                                        
     */
    public Panel getPanel(int where) {                                           /*                              */
        return (panel[index(where)]);            /* get panel object             */
    }                                           /*                              */

    /*
     * Get the String data from the TextField at Index.   
     * @param where Integer indicating where
     * @param index Integer containing the  index value    
     * @return A String containing the text.                  
     **/
    public String getText(int where, int index) {                                           /*                              */
        return (format[index(where)].getText(index));  /* Text data            */
    }                                           /*                              */

    /*
     * Handle a window event.
     * @param e   WindowEvent
     * @return void                                                     
     **/
    protected void processWindowEvent(WindowEvent e) {                                           /*                              */
        if (e.getID() == WindowEvent.WINDOW_CLOSING)      /* loose window         */
            closeDialog();                           /* ... yes                      */
        else                                        /* ... no                       */
            super.processWindowEvent(e);            /* ancestor                     */
    }                                           /*                              */

    /*
     * Set text in to a Text area at the specified Index.
     * @param where Integer indicating where
     * @param index Integer containg the index value
     * @param text A String containg the text which should be set.
     * @return void                        
     **/
    public void setText(int where, int index, String text) {                                           /*                              */
        format[index(where)].setText(index, text);      /* new text             */
    }                                           /*                              */

    /*
     * Show or hide the frame. 
     * @param state boolean containing the state.
     * @param modal boolean modal state.
     * @return void                                                              
     **/
    public void visible(boolean state, boolean modal) {                                           /*                              */
        pack();                                     /* pack all controls            */
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();

        setLocation((screenSize.width - frameSize.width) / 2,
            (screenSize.height - frameSize.height) / 2);
        modalState = modal;                        /* save modal state             */
        setModal(modalState);                     /* possible set modal state     */
        setVisible(state);                        /* make visible                 */
    }                                           /*                              */

    /*
     * add/remove a panel object.  
     * @param where An integer indicating where
     * @param state boolean  containing state                                              
     **/
    public void visible(int where, boolean state) {                                           /*                              */
        int i = index(where);                     /* convert to index             */

        if ((i < 0) || (i >= panel.length))       /* allowed ?                    */
            return;                                   /* ... no                       */
        if (state)                                /* add or remove                */ {                                         /* ... add                      */
            if (panel[i] == null)                   /* panel exist ?                */
                panel[i] = new Panel();                /* ... no, new panel            */
            add(panel[i], BorderWhere[i]);          /* add to frame                 */
        }                                         /*                              */ else                                        /* remove                       */
            remove(panel[i]);                       /* remove the panel             */
        pack();                                     /* re-pack the window           */
    }                                           /*                              */

}

