// Version mqe_java/source/examples/awt/AwtFormat.java, MQeExamples, la200, la200-L030324.1 1.5

/* -----------------------------------------------------------------------------*/ 

/*                    WebSphere MQ Everyplace for Multiplatforms                */ 

/*                                                                              */ 

/* Licensed Materials - Property of IBM                                         */ 

/*                                                                              */ 

/* 5765-E63                                                                     */ 

/*                                                                              */ 

/* (C) Copyright IBM Corp. 1999                                                 */ 

/* -----------------------------------------------------------------------------*/ 
package examples.awt;


import  java.awt.event.*;
import  java.util.*;


/*
 /* <p>Format Components in a Panel or menu.                                                                        
 * Notes:                                                                        
 *                                                                               
 *      Dependent on a subset of the function provided by the following          
 *      AWT components:                                                          
 *                                                                               
 *      for Dialogs and Frames                                                   
 *                Button                                                        
 *                Checkbox                                                       
 *                Choice                                                         
 *                GridLayout                                                     
 *                Label                                                          
 *                List                                                           
 *                Panel                                                          
 *                TextArea                                                       
 *                TextField                                                      
 *                                                                               
 *      for Menus:                                                               
 *                CheckboxMenuItem                                               
 *                Menu                                                           
 *                MenuBar                                                        
 *                MenuItem                                                       
 *                                                                                                                    
 **/ 
public class AwtFormat {
    public    static short    version[] = { 2, 0, 0, 5 }; 

    protected Object           actionObject = null;
    protected String           actionString = null;
    protected AwtEvent         eventAWT = null;
    protected ItemListener     checkAction = null;
    protected int              maskIndex = 0;
    protected KeyAdapter       keyAction = null;
    protected Vector           objectTable = new Vector();
    protected ActionListener   selectAction = null;

    /**
     * Constructor.
     * @param awtEvent  An AwtEvent object
     * @param indexMask An integer containg the index mask.                                                              
     **/
    public AwtFormat(AwtEvent awtEvent, int indexMask) {                                           /*                              */
        super();
        eventAWT = awtEvent;
        maskIndex = indexMask;

        /* Button event handler                                                     */
        selectAction = new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        actionString = e.getActionCommand();
                        actionObject = e.getSource();
                        int Index = objectTable.indexOf(actionObject);

                        if (Index > -1)
                            eventAWT.action(e.getSource(), maskIndex,
                                Index,
                                actionString,
                                false);
                    }
                };

        /* Key release event handler                                                */
        keyAction = new KeyAdapter() {
                    public void keyReleased(KeyEvent e) {
                        actionString = KeyEvent.getKeyText(e.getKeyCode());
                        actionObject = e.getSource();
                        int Index = objectTable.indexOf(actionObject);

                        eventAWT.action(e.getSource(), maskIndex,
                            Index,
                            actionString,
                            false);
                    }
                };

        /* Checkbox event handler                                                   */
        checkAction = new ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        int Index = objectTable.indexOf(e.getSource());

                        if (e.getSource() instanceof java.awt.Checkbox)
                            eventAWT.action(e.getSource(), maskIndex, Index,
                                ((java.awt.Checkbox) e.getSource()).getLabel(),
                                ((java.awt.Checkbox) e.getSource()).getState());
                        else
                        if (e.getSource() instanceof java.awt.CheckboxMenuItem)
                            eventAWT.action(e.getSource(), maskIndex, Index,
                                ((java.awt.CheckboxMenuItem) e.getSource()).getLabel(),
                                ((java.awt.CheckboxMenuItem) e.getSource()).getState());
                        else
                        if (e.getSource() instanceof java.awt.Choice)
                            eventAWT.action(e.getSource(), maskIndex, Index,
                                ((java.awt.Choice) e.getSource()).getSelectedItem(),
                                false);
                    }
                };
    }

    /*----------------------------------------------------------------------------*/
    
    /*                                                                            */
    
    /*----------------------------------------------------------------------------*/
    protected void addToMenu(java.awt.Menu m, String items[], int subIndex) {                                           /*                              */
        switch (items[subIndex].charAt(0))      /* based on option              */ {                                         /*                              */

        /* menu - sub-menu                                                        */
        case 'S': {                                       /*                              */
                java.awt.Menu mi = new java.awt.Menu(items[subIndex + 1]);      /*      */

                m.add(mi);                            /* add to owning menu           */
                for (int i = subIndex + 2; i < items.length; i = i + 2)
                    addToMenu(mi, items, i);            /* add sub-menu items           */
                break;                                  /* done                         */
            }                                       /*                              */

            /* menu - seperator                                                       */
        case '-':
            m.addSeparator();                       /* add seperator line           */
            break;                                  /* done                         */

            /* menu - checkable item                                                  */
        case 'C': {                                       /*                              */
                java.awt.CheckboxMenuItem mi = new java.awt.CheckboxMenuItem(
                        items[subIndex + 1],
                        (items[subIndex].indexOf('!') > -1));

                mi.addItemListener(checkAction);      /* listener for event           */
                m.add(mi);                            /* add to menu                  */
                objectTable.addElement(mi);           /* save in object table         */
                break;                                  /* done                         */
            }                                       /*                              */

            /* menu - action item                                                     */
        default: {                                       /*                              */
                java.awt.MenuItem mi = new java.awt.MenuItem(items[subIndex + 1]);

                mi.addActionListener(selectAction);   /* listener for event           */
                m.add(mi);                            /* add to menu                  */
                objectTable.addElement(mi);           /* save in object table         */
                break;                                  /* done                         */
            }                                       /*                              */
        }                                         /*                              */
    }                                           /*                              */

    /*
     * Get the Index number of the pressed button.  
     * @return An integer containing the index value.                        
     */
    public int getActionIndex() {                                           /*                              */
        if (actionObject == null)                 /* any action ?                 */
            return (-1);                             /* ... no                       */
        return (objectTable.indexOf(actionObject));      /* find in vector       */
    }                                           /*                              */

    /*
     * Get the String data from the pressed button at Index.
     * @return The data String 
     **/
    public String getActionString() {                                           /*                              */
        return (actionString);                     /* return the action            */
    }                                           /*                              */

    /*
     * Get the boolean state from the Checkbox at Index.  
     * @param index Integer, index value.
     * @return A boolean value indicating whether the box was checked or not.                         
     **/
    public boolean getCheckState(int index) {                                           /*                              */
        Object o = objectTable.elementAt(index);  /* get correct object           */
        boolean result = false;                     /* assume not set               */

        if (o instanceof java.awt.Checkbox)       /* is it a checkbox             */
            result = ((java.awt.Checkbox) o).getState();     /* get state           */
        else                                        /* ... no                       */
        if (o instanceof java.awt.CheckboxMenuItem)      /* is it a check item  */
            result = ((java.awt.CheckboxMenuItem) o).getState();   /* get state   */
        return (result);                           /* return the result            */
    }                                           /*                              */

    /*
     * Get the selected item from the Choice box at Index. 
     * @param index Integer, index value.
     * @return The choice String 
     **/
    public String getChoice(int index) {                                           /*                              */
        return (((java.awt.Choice) objectTable.elementAt(index)).getSelectedItem());
    }                                           /*                              */

    /*
     * Get the object at the specified Index. 
     * @param index Integer, index value.
     * @return An object at the index value.                                
     **/
    public Object getObject(int index) {                                           /*                              */
        return (objectTable.elementAt(index));   /* find Object for Index number */
    }                                           /*                              */

    /*
     * Get the String data from the TextField at Index.
     * @param index Integer, index value.
     * @return  Text String                         
     **/
    public String getText(int index) {                                           /*                              */
        String result = null;
        Object o = objectTable.elementAt(index);

        if (o instanceof java.awt.TextField)
            result = ((java.awt.TextField) o).getText();
        else
        if (o instanceof java.awt.TextArea)
            result = ((java.awt.TextArea) o).getText();
        else
        if (o instanceof java.awt.MenuItem)
            result = ((java.awt.MenuItem) o).getLabel();
        else
        if (o instanceof java.awt.Button)
            result = ((java.awt.Button) o).getLabel();
        else
        if (o instanceof java.awt.Checkbox)
            result = ((java.awt.Checkbox) o).getLabel();
        else
        if (o instanceof java.awt.Choice)
            result = ((java.awt.Choice) o).getSelectedItem();
        else
        if (o instanceof java.awt.List)
            result = ((java.awt.List) o).getSelectedItem();
        return (result);
    }                                           /*                              */

    /**
     * Format the components in a Menu.                                           
     *                                                                            
     * @param Items   The Items to be displayed in the dialog box.                  
     * This is an array of array of array.                           
     * The first dimention defines the number of rows.               
     * The second dimension defines the number of columns            
     * The third dimension defins the components of the form:        
     * { type, Data {, Data, { ... } } }  
     * <pre>           
     *              where :                                                       
     *                    type  - is the component type                           
     *                            " "  - normal menu item                         
     *                            "C"  - CheckItem - unchecked                    
     *                                   "!"  - checked                           
     *                            "-"  - seperator                                
     *                            any thing else is treated as a Label            
     *                                                                            
     *                    Data  - the text to be used by the component            
     * </pre>                                                                           
     * <p>For Example:                                                                   
     * <pre>                                                                           
     *    Format( 'C', new String[][][]                                           
     *               { { { "File" },                                            
     *                   { " ", "Open"   },                            // Index 0 
     *                   { " ", "Close"  }                             // Index 1 
     *                   { "-"    }                                    //         
     *                   { "C", "Log"   }                              // Index 2 
     *                   { "-"    }                                    //         
     *                   { " ", "Exit"   }                             // Index 3 
     *                 },                                                         
     *                 { { "Help" },                                            
     *                   { " ", "About"  } } } );                      // Index 4 
     * </pre>
     ***/
    public java.awt.MenuBar menuBar(String items[][][]) {                                           /*                              */
        java.awt.MenuBar mb = new java.awt.MenuBar();      /* create new menuBar   */

        /* add Label's and CheckItem's                                              */
        for (int i = 0; i < items.length; i = i + 1)      /* for all rows         */ {                                         /*                              */
            java.awt.Menu m = new java.awt.Menu(items[i][0][0]);      /* new menu   */

            mb.add(m);                              /* add to menu bar              */
            for (int j = 1; j < items[i].length; j = j + 1) /* decode sub items     */
                addToMenu(m, items[i][j], 0);         /* process                      */
        }                                         /*                              */
        return (mb);                               /* return the complete menubar  */
    }                                           /*                              */

    /**
     * Format the components in a panel.                                         
     * @param p java.awt.Panel                                                                          
     * @param   Items The Items to be displayed in the dialog box or frame.         
     * This is an array of array of array.                           
     * The first dimention defines the number of rows.               
     * The second dimension defines the number of columns            
     * The third dimension defins the components of the form:       
     * { type, Data {, Data, { ... } } } 
     * <pre>            
     *              where :                                                      
     *                    type  - is the component type                           
     *                            "A"  - TextArea                                 
     *                                   "P" - protected - not editable          
     *                                   "K" - give Key released action events 
     *                            "B"  - Button                                  
     *                            "C"  - Checkbox - unchecked                     
     *                                   "!"  - checked                          
     *                            "D"  - Choice    (drop down list)               
     *                            "L"  - Label                                    
     *                            "S"  - Selection list (list box)                
     *                            "T"  - TextField                                
     *                                   "P" - protected - not editable           
     *                                   "K" - give Key released action events    
     *                                   "*" - masked input                       
     *                            any thing else is treated as a Label            
     *                                                                            
     *                    Data  - the text to be used by the component            
     * </pre>                                                                           
     * <p>For Example:                                                                  
     * <pre>                                                                          
     *    Format( 'C', new String[][][]                                          
     *               { { { "L",  "Zero"  },                            // Index 0 
     *                   { "T",  "One"   },                            // Index 1 
     *                   { "T*", "Two"   }                             // Index 2 
     *                 },                                                         
     *                 { { "C!", "Three" },                            // Index 3 
     *                   { "C",  "Four"  },                            // Index 4 
     *                   { "TP", "Five"  },                            // Index 5 
     *                 },                                                         
     *                 { { "D",  "Six-0", "Six-1", "Six-2", "Six-3" }, // Index 6 
     *                   { "B",  "OK"    },                            // Index 7 
     *                   { "B",  "Abort" }                             // Index 8 
     *               }                                                            
     *             } );                                                           
     *    System.out.println( " input    field 1=" + D.GetTextField( 1 ) );       
     *    System.out.println( " Checkbox field 3=" + D.GetCheckBox ( 3 ) );       
     *    System.out.println( " Choice   field 6=" + D.GetChoice   ( 6 ) );       
     *    System.out.println( " Action   String="  + D.GetActionString(  ) );     
     *    System.out.println( " Action   Index="   + D.GetActionIndex (  ) ); 
     * </pre>   
     * @return java.awt.Panel 
     **/
    public java.awt.Panel panel(java.awt.Panel p, String items[][][]) {                                           /*                              */
        if (p == null)                            /* supplied panel ?             */
            p = new java.awt.Panel();                /* ... no                       */
        else                                        /* ... yes                      */
            p.removeAll();                           /* clear the panel              */

        /* set Grid layout rows and columns                                         */
        java.awt.GridLayout grid = new java.awt.GridLayout();        /* grid layout*/

        grid.setColumns(items[0].length);         /* set grid layout              */
        grid.setRows(items.length);         /* set grid layout              */
        p.setLayout(grid);                        /* apply layout                 */

        /* add Button's, Label's, Checkboxe's and TextField's                       */
        for (int i = 0; i < items.length; i = i + 1)      /* for all rows         */
            for (int j = 0; j < items[i].length; j = j + 1)         /* sub array    */ {                                       /*                              */
                Object o;                               /* work variable                */

                if (items[i][j].length < 2)           /* enough elements ?            */ {                                     /* ... no                       */
                    o = new java.awt.Label("");         /* an empty label               */
                    p.add((java.awt.Label) o, null);    /* add to panel                 */
                }                                     /*                              */ else                                    /* ... yes                      */ {                                     /*                              */
                    String item0 = items[i][j][0];        /* to reduce generated code     */
                    String item1 = items[i][j][1];        /* to reduce generated code     */

                    switch (item0.charAt(0))          /* what type of component ?     */ {                                   /*                              */

                    /* TextArea                                                         */
                    case 'A':
                        o = new java.awt.TextArea();     /* new button                   */
                        if (item0.indexOf('K') > -1)  /* key released event ?         */
                            ((java.awt.TextArea) o).addKeyListener(keyAction);     /* yes */
                        if (item0.indexOf('P') > -1)  /* protected ?                  */
                            ((java.awt.TextArea) o).setEditable(false);    /* ... yes     */
                        ((java.awt.TextArea) o).setText(item1); /* set the text        */
                        p.add((java.awt.TextArea) o, null);      /* add to panel        */
                        break;                            /* done                         */

                        /* Button                                                           */
                    case 'B':
                        o = new java.awt.Button();       /* new button                   */
                        ((java.awt.Button) o).setLabel(item1);  /* set the text        */
                        ((java.awt.Button) o).addActionListener(selectAction);  /*     */
                        p.add((java.awt.Button) o, null);        /* add to panel        */
                        break;                            /* done                         */

                        /* Checkbox (checked/unchecked)                                     */
                    case 'C':
                        o = new java.awt.Checkbox(item1, (item0.indexOf('!') > -1));
                        ((java.awt.Checkbox) o).addItemListener(checkAction);    /* proc*/
                        p.add((java.awt.Checkbox) o, null);      /* add to panel        */
                        break;                            /* done                         */

                        /* Choice (drop down list box)                                      */
                    case 'D':
                        o = new java.awt.Choice();       /* new Choice list              */
                        for (int k = 1; k < items[i][j].length; k = k + 1)
                            ((java.awt.Choice) o).add(items[i][j][k]);     /* add item    */
                        ((java.awt.Choice) o).addItemListener(checkAction);      /* proc*/
                        ((java.awt.Choice) o).select(0);         /* first one selected  */
                        p.add((java.awt.Choice) o, null);        /* add to panel        */
                        break;                            /* done                         */

                        /* Selection List (list box)                                        */
                    case 'S':
                        o = new java.awt.List();         /* new list                     */
                        for (int k = 1; k < items[i][j].length; k = k + 1)
                            ((java.awt.List) o).add(items[i][j][k]);   /* add item        */
                        ((java.awt.List) o).addActionListener(selectAction);  /* event  */
                        p.add((java.awt.List) o, null); /* add to panel                 */
                        break;                            /* done                         */

                        /* TextField                                                        */
                    case 'T':
                        o = new java.awt.TextField(item1);       /* new TextField       */
                        if (item0.indexOf('K') > -1)  /* want key released event ?    */
                            ((java.awt.TextField) o).addKeyListener(keyAction);    /* yes */
                        if (item0.indexOf('P') > -1)  /* protected ?                  */
                            ((java.awt.TextField) o).setEditable(false);   /* ... yes     */
                        if (item0.indexOf('*') > -1)  /* mask input ?                 */
                            ((java.awt.TextField) o).setEchoChar('*');     /* set mask    */
                        p.add((java.awt.TextField) o, null);     /* remember the object */
                        break;                            /* done                         */

                        /* anything else is treated as a label                              */
                    case 'L':
                    default:
                        o = new java.awt.Label(item1);           /* new Label           */
                        p.add((java.awt.Label) o, null);         /* add to pannel       */
                        break;                             /* done                         */
                    }                                   /* end switch                   */
                }                                     /*                              */
                objectTable.addElement(o);            /* add object to vector         */
            }                                       /*                              */
        return (p);                                /* return the panel             */
    }                                           /*                              */

    /**
     * Set text in to the specified Index.
     * @param index Integer index value
     * @param msg The Text String containing the msg.               
     **/
    public void setText(int index, String msg) {                                           /*                              */
        Object o = objectTable.elementAt(index);

        if (o instanceof java.awt.TextField)
            ((java.awt.TextField) o).setText(msg);
        else
        if (o instanceof java.awt.TextArea)
            ((java.awt.TextArea) o).setText(msg);
        else
        if (o instanceof java.awt.MenuItem)
            ((java.awt.MenuItem) o).setLabel(msg);
        else
        if (o instanceof java.awt.Button)
            ((java.awt.Button) o).setLabel(msg);
        else
        if (o instanceof java.awt.Checkbox)
            ((java.awt.Checkbox) o).setLabel(msg);
        else
        if (o instanceof java.awt.Choice)
            ((java.awt.Choice) o).add(msg);
        else
        if (o instanceof java.awt.List)
            ((java.awt.List) o).add(msg);
    }                                           /*                              */

}

