// Version 1.29 mqe_java/source/examples/awt/AwtMQeTrace.java, MQeExamples, la200, la200-L030409

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.awt;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import com.ibm.mqe.trace.*;
import com.ibm.mqe.*;
import java.util.*;


/** 
 * An AWT control panel for collecting and storing trace information.
 */
public class AwtMQeTrace {

    /**
     * A  version so we can find out what version of this class
     * is on the classpath.
     */
    public static short[] version = { 2, 0, 0, 5 }; 

    static AwtMQeTrace me = null;

    /**
     * Closes the trace window an all associated objects.
     */
    public static void close() {
        if (me != null) {
            me.closeMe();
        }
        me = null;
    }

    /**
     * Opens the trace window and displays it.
     * 
     * <p>If the trace window is already created, then it is made visible.
     * 
     * <p>Trace handlers are plugged-in to the feed of trace information 
     * coming from the MQeTrace class.
     */
    public static final void open() {
        if (me == null) {
            me = new AwtMQeTrace();
        } else {
            me.openMe();
        }
    }

    /**
     * A private instance method, so we only have a single static "me" reference.
     */
    private void closeMe() {
        if (traceWindow != null) {
            traceWindow.closeMe();
            traceWindow = null;
        }
    }

    /**
     * A private constructor, as the inner internal class is only available to 
     * the open() factory method.
     */
    private AwtMQeTrace() {
        openMe();
    }

    /**
     * When the trace class is opened, this method gets called to create the
     * window and trace resources.
     */
    private void openMe() {
        if (traceWindow == null) {
            traceWindow = new InternalAwtMQeTrace();
        } else {
            traceWindow.setVisible(true);
        }
    }

    /**
     * A private reference to the internal trace window, which does all the work.
     */
    private InternalAwtMQeTrace traceWindow = null;

    /**
     * A private class which does all the windowing and trace function.
     * 
     * <p>The function is hidden in a private class to make the interface to the 
     * outer class very simple.
     */
    private class InternalAwtMQeTrace extends Frame {

        private static final String WINDOW_FRAME_TITLE = "MQe Trace";

        private static final String MENU_LABEL_FILE = "File";

        private static final String MENU_ITEM_LABEL_QUIT = "Close";
        private static final char MENU_ITEM_LABEL_QUIT_SHORTCUT = 'l';

        private static final String MENU_ITEM_LABEL_TO_STDOUT = "Trace to stdout";
        private static final String MENU_ITEM_LABEL_TO_BINARY_FILE =
            "Trace to binary file.";
        private static final String MENU_ITEM_LABEL_TO_ASCII_FILE =
            "Trace to ascii file.";

        private static final String MENU_LABEL_VIEW = "View";
        private static final String MENU_ITEM_LABEL_CAPTURE_STDOUT = "Trace stdout";
        private static final String MENU_ITEM_LABEL_CAPTURE_STDERR = "Trace stderr";

        private static final String MENU_ITEM_LABEL_TO_SCREEN = "Trace to screen";
        private static final String MENU_ITEM_LABEL_FILTER_ALL = "Trace all";
        private static final String MENU_ITEM_LABEL_FILTER_NONE = "Trace nothing";
        private static final String MENU_ITEM_LABEL_FILTER_USER_ALL =
            "Trace all user groups";
        private static final String MENU_ITEM_LABEL_FILTER_USER_NONE =
            "Trace no user groups";
        private static final String MENU_ITEM_LABEL_FILTER_OTHER_ALL =
            "Trace all other groups";
        private static final String MENU_ITEM_LABEL_FILTER_OTHER_NONE =
            "Trace no other groups";

        private static final int CENTER_AREA_ROWS = 20;
        private static final int CENTER_AREA_COLUMNS = 100;

        private static final int COMMAND_MENU_ITEM_QUIT_SELECTED = 0;
        private static final int COMMAND_MENU_ITEM_TO_STDOUT_SELECTED = 1;
        private static final int COMMAND_MENU_ITEM_CAPTURE_STDOUT_SELECTED = 2;
        private static final int COMMAND_MENU_ITEM_TO_SCREEN_SELECTED = 3;
        private static final int COMMAND_MENU_ITEM_TO_BINARY_FILE_SELECTED = 4;
        private static final int COMMAND_MENU_ITEM_TO_ASCII_FILE_SELECTED = 5;
        private static final int COMMAND_MENU_ITEM_FILTER_ALL_SELECTED = 6;
        private static final int COMMAND_MENU_ITEM_FILTER_EXCEPTIONS_SELECTED = 7;
        private static final int COMMAND_MENU_ITEM_FILTER_NONE_SELECTED = 8;
        private static final int COMMAND_MENU_ITEM_FILTER_ERRORS_SELECTED = 9;
        private static final int COMMAND_MENU_ITEM_FILTER_WARNINGS_SELECTED = 10;
        private static final int COMMAND_MENU_ITEM_FILTER_INFO_SELECTED = 11;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_ALL_SELECTED = 12;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_NONE_SELECTED = 13;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_1_SELECTED = 14;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_2_SELECTED = 15;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_3_SELECTED = 16;
        private static final int COMMAND_MENU_ITEM_FILTER_USER_4_SELECTED = 17;
        private static final int COMMAND_MENU_ITEM_FILTER_OTHER_ALL_SELECTED = 18;
        private static final int COMMAND_MENU_ITEM_FILTER_OTHER_NONE_SELECTED = 19;
        private static final int COMMAND_MENU_ITEM_FILTER_BRIDGE_SELECTED = 20;
        private static final int COMMAND_MENU_ITEM_CAPTURE_STDERR_SELECTED = 21; 
        private static final int COMMAND_MENU_ITEM_FILTER_MQSERIES_SELECTED = 22;
        private static final int COMMAND_MENU_ITEM_FILTER_API_SELECTED = 23;
        private static final int COMMAND_MENU_ITEM_FILTER_ENTRY_SELECTED = 24;
        private static final int COMMAND_MENU_ITEM_FILTER_EXIT_SELECTED = 25;

        private java.awt.MenuBar menuBar;

        private java.awt.Menu fileMenu;

        private java.awt.CheckboxMenuItem toStdOutMenuItem;
        private java.awt.CheckboxMenuItem toScreenMenuItem;
        private java.awt.CheckboxMenuItem toBinaryFileMenuItem;
        private java.awt.CheckboxMenuItem toAsciiFileMenuItem;
        private java.awt.CheckboxMenuItem captureStdOutMenuItem;
        private java.awt.CheckboxMenuItem captureStdErrMenuItem;

        private java.awt.MenuItem exitMenuItem;

        private java.awt.Menu viewMenu;
        private java.awt.CheckboxMenuItem viewAllMenuItem;
        private java.awt.CheckboxMenuItem viewNoneMenuItem;
        private java.awt.CheckboxMenuItem filterAllMenuItem;
        private java.awt.CheckboxMenuItem filterNoneMenuItem;

        private java.awt.CheckboxMenuItem filterUserAllMenuItem;
        private java.awt.CheckboxMenuItem filterUserNoneMenuItem;

        private java.awt.CheckboxMenuItem filterOtherAllMenuItem;
        private java.awt.CheckboxMenuItem filterOtherNoneMenuItem;
        private java.awt.CheckboxMenuItem filterApiMenuItem;
        private java.awt.CheckboxMenuItem filterEntryMenuItem;
        private java.awt.CheckboxMenuItem filterExitMenuItem;

        private java.awt.TextArea panelContent;

        private static final boolean CHECKED = true;
        private static final boolean UNCHECKED = false;
		
        private static final boolean DIALOG_MODALITY_MODAL = true;

        private static final long INITIAL_FILTER = MQeTrace.GROUP_MASK_DEFAULT;

        /** 
         * A hash table holding all our "filter" menu items.
         * 
         * The key to the hash is the command number it can process, expressed as a string.
         */
        private Hashtable filterFlags = new Hashtable();

        private static final long GROUP_USER_ALL = MQeTrace.GROUP_MASK_USER_DEFINED;
        private static final long GROUP_USER_NONE = 0L;

        private static final long GROUP_OTHER_ALL =
            MQeTrace.GROUP_MASK_ALL
            & (~GROUP_USER_ALL)
            & (~MQeTrace.GROUP_ERROR)
            & (~MQeTrace.GROUP_WARNING)
            & (~MQeTrace.GROUP_EXCEPTION)
            & (~MQeTrace.GROUP_INFO)
            & (~MQeTrace.GROUP_BRIDGE)
            & (~MQeTrace.GROUP_MQSERIES)
            & (~MQeTrace.GROUP_API)
            & (~MQeTrace.GROUP_ENTRY)
            & (~MQeTrace.GROUP_EXIT);
				
        private static final long GROUP_NONE_OTHER = 0L;

        /**
         * Which trace points are we filtering on ?
         */
        private long traceFilter = MQeTrace.GROUP_MASK_DEFAULT;

        /**
         * Constructor allowing a default frame title to be set.
         */
        public InternalAwtMQeTrace() {
            this(WINDOW_FRAME_TITLE);
        }

        /**
         * Constructor for InternalAwtMQeTrace
         */
        public InternalAwtMQeTrace(String title) {
            super(title);

            activeTraceHandlers = new MQeTraceSplitter();
            MQeTrace.setHandler((MQeTraceHandler) activeTraceHandlers);

            buildFrameCenter();

            outputStreamToCentralPanel = new AwtOutputStream(panelContent);

            // Build the menu bar.
            buildMenuBar();

            setFilter(INITIAL_FILTER);
			
            // Pack the components into the frame.
            pack();

            // Collect window events.
            enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);

            // Allow people to see the window.    			                                               
            setVisible(true);
        }
		
        /**
         * Capture window events arriving at this frame.
         * 
         * <P>Process window closing events. Pass all others on to the super class.
         * @param e  WindowEvent
         * @return void
         */
        protected final void processWindowEvent(WindowEvent e) {
            int event = e.getID();

            switch (event) {
            case WindowEvent.WINDOW_CLOSING:
                // User has used the system menu events to close the window.
                close();
                break;

            default:
                super.processWindowEvent(e);
                break;
            }
        }

        /**
         * Constructs the menu bar for the frame.
         */
        private void buildMenuBar() {

            // Create a menu bar.
            menuBar = new java.awt.MenuBar();

            // Populate the menu bar with the file menu.
            buildFileMenu();
            menuBar.add(fileMenu);

            // Populate the menu bar with the view menu.
            buildViewMenu();
            menuBar.add(viewMenu);

            // Set the menu bar into the frame.
            this.setMenuBar(menuBar);
        }

        /**
         * Constructs the "File" menu.
         */
        private void buildFileMenu() {
            // Construct a File menu
            fileMenu = new java.awt.Menu(MENU_LABEL_FILE);
            boolean traceToScreenDefault = false;

            toScreenMenuItem =
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_TO_SCREEN,
                        traceToScreenDefault,
                        COMMAND_MENU_ITEM_TO_SCREEN_SELECTED,
                        this);
            if (traceToScreenDefault) {
                traceToScreenStart();
            }

            boolean captureStdOutDefault = false;

            captureStdOutMenuItem =
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_CAPTURE_STDOUT,
                        captureStdOutDefault,
                        COMMAND_MENU_ITEM_CAPTURE_STDOUT_SELECTED,
                        this);
            if (captureStdOutDefault) {	
                captureStdoutStart();
            }
			
            boolean captureStdErrDefault = false;

            captureStdErrMenuItem = 
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_CAPTURE_STDERR,
                        captureStdErrDefault,
                        COMMAND_MENU_ITEM_CAPTURE_STDERR_SELECTED,
                        this);
            if (captureStdErrDefault) {						
                captureStdErrStart();					
            }

            toStdOutMenuItem =
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_TO_STDOUT,
                        false,
                        COMMAND_MENU_ITEM_TO_STDOUT_SELECTED,
                        this);

            toAsciiFileMenuItem =
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_TO_ASCII_FILE,
                        false,
                        COMMAND_MENU_ITEM_TO_ASCII_FILE_SELECTED,
                        this);

            toBinaryFileMenuItem =
                    createCheckboxMenuItem(
                        fileMenu,
                        MENU_ITEM_LABEL_TO_BINARY_FILE,
                        false,
                        COMMAND_MENU_ITEM_TO_BINARY_FILE_SELECTED,
                        this);

            createMenuSeparator(fileMenu);

            MenuItem quitMenuItem =
                createMenuItem(
                    fileMenu,
                    MENU_ITEM_LABEL_QUIT,
                    MENU_ITEM_LABEL_QUIT_SHORTCUT,
                    COMMAND_MENU_ITEM_QUIT_SELECTED,
                    this);
        }

        /**
         * Creates a menu divider, a horizontal line separating other menu items.	
         */
        void createMenuSeparator(java.awt.Menu owningMenu) {
            owningMenu.add(new MenuItem("-"));
        }

        /**
         * Constructs the "View" menu.
         */
        private void buildViewMenu() {
            // Construct the view menu
            viewMenu = new java.awt.Menu(MENU_LABEL_VIEW);

            filterAllMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_ALL,
                        true,
                        COMMAND_MENU_ITEM_FILTER_ALL_SELECTED,
                        this);

            filterNoneMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_NONE,
                        false,
                        COMMAND_MENU_ITEM_FILTER_NONE_SELECTED,
                        this);

            createMenuSeparator(viewMenu);

            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_ERRORS_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_ERROR,
                    viewMenu,
                    "Errors",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_ERRORS_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_WARNINGS_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_WARNING,
                    viewMenu,
                    "Warnings",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_WARNINGS_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_INFO_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_INFO,
                    viewMenu,
                    "Information",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_INFO_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_EXCEPTIONS_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_EXCEPTION,
                    viewMenu,
                    "Exceptions",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_EXCEPTIONS_SELECTED,
                    this));

            createMenuSeparator(viewMenu);

            filterUserAllMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_USER_ALL,
                        true,
                        COMMAND_MENU_ITEM_FILTER_USER_ALL_SELECTED,
                        this);
            filterUserAllStart();

            filterUserNoneMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_USER_NONE,
                        false,
                        COMMAND_MENU_ITEM_FILTER_USER_NONE_SELECTED,
                        this);

            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_USER_1_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_USER_DEFINED_1,
                    viewMenu,
                    "User group 1 events",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_USER_1_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_USER_2_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_USER_DEFINED_2,
                    viewMenu,
                    "User group 2 events",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_USER_2_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_USER_3_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_USER_DEFINED_3,
                    viewMenu,
                    "User group 3 events (+captured stdout)",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_USER_3_SELECTED,
                    this));
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_USER_4_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_USER_DEFINED_4,
                    viewMenu,
                    "User group 4 events (+captured stderr)",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_USER_4_SELECTED,
                    this));

            createMenuSeparator(viewMenu);

            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_BRIDGE_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_BRIDGE,
                    viewMenu,
                    "Bridge",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_BRIDGE_SELECTED,
                    this));
					
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_MQSERIES_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_MQSERIES,
                    viewMenu,
                    "MQSeries",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_MQSERIES_SELECTED,
                    this));					

            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_API_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_API,
                    viewMenu,
                    "Api",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_API_SELECTED,
                    this));	
                         
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_ENTRY_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_ENTRY,
                    viewMenu,
                    "Entry",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_ENTRY_SELECTED,
                    this));
					
            filterFlags.put(
                String.valueOf(COMMAND_MENU_ITEM_FILTER_EXIT_SELECTED),
                new FilterFlag(
                    MQeTrace.GROUP_EXIT,
                    viewMenu,
                    "Exit",
                    CHECKED,
                    COMMAND_MENU_ITEM_FILTER_EXIT_SELECTED,
                    this));					                         
                                        	
            createMenuSeparator(viewMenu);

            filterOtherAllMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_OTHER_ALL,
                        true,
                        COMMAND_MENU_ITEM_FILTER_OTHER_ALL_SELECTED,
                        this);

            filterOtherNoneMenuItem =
                    createCheckboxMenuItem(
                        viewMenu,
                        MENU_ITEM_LABEL_FILTER_OTHER_NONE,
                        false,
                        COMMAND_MENU_ITEM_FILTER_OTHER_NONE_SELECTED,
                        this);

        }

        /**
         * A menu item which represents a single filter bitmask.
         * 
         */
        private class FilterFlag {
            long filterValue;
            java.awt.CheckboxMenuItem menuItem;

            /**
             * Constructs a filter flag inner class.
             */
            FilterFlag(
                long filterValue,
                java.awt.Menu owningMenu,
                String menuLabel,
                boolean isCheckedInitially,
                int commandNumberWhenSelected,
                InternalAwtMQeTrace commandProcessor) {
                this.menuItem =
                        createCheckboxMenuItem(
                            owningMenu,
                            menuLabel,
                            isCheckedInitially,
                            commandNumberWhenSelected,
                            commandProcessor);
                this.filterValue = filterValue;

            }

            /**
             * The filter is being turned on.
             * Set the trace filter accordingly.
             */
            void start() {
                setFilter(getFilter() | filterValue);
            }

            /**
             * The filter is being turned off.
             * Set the trace filter accordingly.
             */
            void stop() {
                setFilter(getFilter() & (~filterValue));
            }

            /**
             * The trace filter has been set. Check the filter and see if this 
             * menu item needs to be ticked or not.
             */
            void isSet() {
                if (menuItem != null) {
                    if ((getFilter() & filterValue) != 0L) {
                        menuItem.setState(true);
                    } else {
                        menuItem.setState(false);
                    }
                }
            }

            /**
             * The menu item has been selected. Process the event 
             * according to whether the menu checkbox is being ticked on or off.
             */
            void doCommand(int command) {
                if (this.menuItem.getState()) {
                    start();
                } else {
                    stop();
                }
            }
        }

        private void filterOtherAllStart() {
            setFilter(getFilter() | GROUP_OTHER_ALL);
        }

        private void filterUserAllStart() {
            setFilter(getFilter() | GROUP_USER_ALL);
        }

        private void filterOtherAllStop() {
            filterOtherNoneStart();
        }

        private void filterOtherNoneStart() {
            setFilter(getFilter() & (~GROUP_OTHER_ALL));
        }

        private void filterOtherNoneStop() {
            filterOtherNoneStart();
        }

        private void filterUserAllStop() {
            filterUserNoneStart();
        }

        private void filterUserNoneStart() {
            setFilter(getFilter() & (~GROUP_USER_ALL));
        }

        private void filterUserNoneStop() {
            filterUserAllStart();
        }

        private void filterNoneStart() {
            setFilter(MQeTrace.GROUP_MASK_NONE);
        }

        private void filterNoneStop() {
            filterAllStart();
        }

        private void filterOtherNoneIsSet() {
            if (filterOtherNoneMenuItem != null) {
                if ((getFilter() & GROUP_OTHER_ALL) == 0L) {
                    filterOtherNoneMenuItem.setState(true);
                    filterOtherNoneMenuItem.setEnabled(false);
                } else {
                    filterOtherNoneMenuItem.setState(false);
                    filterOtherNoneMenuItem.setEnabled(true);
                }
            }
        }

        private void filterOtherAllIsSet() {
            if (filterOtherAllMenuItem != null) {
                if ((getFilter() & GROUP_OTHER_ALL) == GROUP_OTHER_ALL) {
                    filterOtherAllMenuItem.setState(true);
                    filterOtherAllMenuItem.setEnabled(false);
                } else {
                    filterOtherAllMenuItem.setState(false);
                    filterOtherAllMenuItem.setEnabled(true);
                }
            }
        }

        private void filterUserNoneIsSet() {
            if (filterUserNoneMenuItem != null) {
                if ((getFilter() & GROUP_USER_ALL) == 0L) {
                    filterUserNoneMenuItem.setState(true);
                    filterUserNoneMenuItem.setEnabled(false);
                } else {
                    filterUserNoneMenuItem.setState(false);
                    filterUserNoneMenuItem.setEnabled(true);
                }
            }
        }

        private void filterNoneIsSet() {
            if (filterNoneMenuItem != null) {
                if ((getFilter() & MQeTrace.GROUP_MASK_ALL) == 0L) {
                    filterNoneMenuItem.setState(true);
                    filterNoneMenuItem.setEnabled(false);
                } else {
                    filterNoneMenuItem.setState(false);
                    filterNoneMenuItem.setEnabled(true);
                }
            }
        }

        private void filterAllStart() {
            setFilter(MQeTrace.GROUP_MASK_ALL);
        }

        private void filterAllStop() {
            filterNoneStart();
        }

        private void filterAllIsSet() {
            if (filterAllMenuItem != null) {
                if (getFilter() == MQeTrace.GROUP_MASK_ALL) {
                    filterAllMenuItem.setState(true);
                    filterAllMenuItem.setEnabled(false);
                } else {
                    filterAllMenuItem.setState(false);
                    filterAllMenuItem.setEnabled(true);
                }
            }
        }

        /**
         * Are all the user trace groups being collected ?
         * Adjust the filter menu item accordingly.
         */
        private void filterUserAllIsSet() {
            if (filterUserAllMenuItem != null) {
                if ((getFilter() & GROUP_USER_ALL) == GROUP_USER_ALL) {
                    filterUserAllMenuItem.setState(true);
                    filterUserAllMenuItem.setEnabled(false);
                } else {
                    filterUserAllMenuItem.setState(false);
                    filterUserAllMenuItem.setEnabled(true);
                }
            }
        }

        /**
         * The filter is being set by some code inside this class.
         * Most likely as a result of one of the menu items being selected.
         */
        private void setFilter(long newFilter) {
            if (getFilter() != newFilter) {
                MQeTrace.setFilter(newFilter);
            }
            filterHasChanged();
        }
		
        /**
         * The filter has just changed. Make sure all our menu items are 
         * up to date.
         */
        private void filterHasChanged() {
            filterAllIsSet();
            filterNoneIsSet();
            filterUserAllIsSet();
            filterUserNoneIsSet();
            filterOtherAllIsSet();
            filterOtherNoneIsSet();
            Enumeration filterFlagWalker = filterFlags.elements();

            while (filterFlagWalker.hasMoreElements()) {
                FilterFlag thisFlag = (FilterFlag) filterFlagWalker.nextElement();

                thisFlag.isSet();
            }
        }

        /**
         * The filter is being got by some code inside this class.
         * Most likely as a result of one of the menu items being selected.
         */
        private long getFilter() {
            return MQeTrace.getFilter();
        }

        /**
         * Constructs the text area widget to put into the central part of the frame.
         */
        private void buildFrameCenter() {
            // Put a text area in the middle of the panel.
            panelContent = new java.awt.TextArea(CENTER_AREA_ROWS, CENTER_AREA_COLUMNS);
            this.add(panelContent, java.awt.BorderLayout.CENTER);
        }

        /**
         * Creates a menu item, and assigns it a handler for selection events.
         * 
         * <p>When the Awt event is detected for this menu item, the specified
         * command handler will be passed the specified commandCode.
         * 
         * @param label The text displayed by this menu item.
         * @param shortCut The part of the label which is a shortcut.
         * @param commandCode The command code value passed on the call-back to the 
         * command handler when the menu item is selected.
         * @param handler The object which will be called-back when the menu item is 
         * selected.
         * 
         * @return The constructed menu item object.
         */
        MenuItem createMenuItem(
            Menu owningMenu,
            String label,
            char shortCut,
            int commandCode,
            InternalAwtMQeTrace handler) {

            /**
             * A local class class to serve as an action listener.
             * Converting an Awt event into a "command".
             */
            class MenuItemListener implements java.awt.event.ActionListener {

                /**
                 * A local reference to the command handler we are to send the command to.
                 */
                InternalAwtMQeTrace myCommandHandler;

                /**
                 * The value of the command to pass back to the command handler when the 
                 * menu item is used.
                 */
                int myCommandCode;

                /**
                 * Constructs a menu item listener.
                 * @parma commandHandler A reference to the object which will process the commands.
                 */
                MenuItemListener(InternalAwtMQeTrace commandHandler, int commandCode) {
                    this.myCommandHandler = commandHandler;
                    this.myCommandCode = commandCode;
                }

                /**
                 * Called by Awt when the menu item is used.
                 * Causes a call-back to the command handler. 
                 */
                public void actionPerformed(ActionEvent e) {
                    this.myCommandHandler.doCommand(this.myCommandCode);
                }
            }

            // Create the menu item.
            MenuItem item = new MenuItem(label, new MenuShortcut(shortCut));
            ActionListener selectionEventHandler =
                (ActionListener) new MenuItemListener(handler, commandCode);

            // Allow capture of selection events for this menu item.	
            item.addActionListener(selectionEventHandler);

            // Add the item to the menu
            owningMenu.add(item);

            return item;
        }

        /**
         * Creates a check box menu item, and assigns it a handler for selection events.
         * 
         * <p>When the Awt event is detected for this menu item, the specified
         * command handler will be passed the specified commandCode.
         * 
         * @param label The text displayed by this menu item.
         * @param shortCut The part of the label which is a shortcut.
         * @param commandCode The command code value passed on the call-back to the 
         * command handler when the menu item is selected.
         * @param handler The object which will be called-back when the menu item is 
         * selected.
         * 
         * @return The constructed menu item object.
         */
        CheckboxMenuItem createCheckboxMenuItem(
            Menu owningMenu,
            String label,
            boolean initialState,
            int commandCode,
            InternalAwtMQeTrace handler) {

            /**
             * A local class class to serve as an action listener.
             * Converting an Awt event into a "command".
             */
            class CheckboxMenuItemListener implements java.awt.event.ItemListener {

                /**
                 * A local reference to the command handler we are to send the command to.
                 */
                InternalAwtMQeTrace myCommandHandler;

                /**
                 * The value of the command to pass back to the command handler when the 
                 * menu item is used.
                 */
                int myCommandCode;

                /**
                 * Constructs a menu item listener.
                 * @parma commandHandler A reference to the object which will process the commands.
                 */
                CheckboxMenuItemListener(InternalAwtMQeTrace commandHandler, int commandCode) {
                    this.myCommandHandler = commandHandler;
                    this.myCommandCode = commandCode;
                }

                /**
                 * Called by Awt when the menu item is used.
                 * Causes a call-back to the command handler. 
                 */
                public void itemStateChanged(ItemEvent e) {
                    this.myCommandHandler.doCommand(this.myCommandCode);
                }
            }

            // Create the menu item.
            CheckboxMenuItem item = new CheckboxMenuItem(label, initialState);

            ItemListener selectionEventHandler =
                (ItemListener) new CheckboxMenuItemListener(handler, commandCode);

            // Allow capture of selection events.			
            item.addItemListener(selectionEventHandler);

            // Add the menu item to the owning menu.
            owningMenu.add(item);

            return item;
        }

        /**
         * Close everything down in good order from within the 
         */
        public void closeMe() {

            // Cut the feed of trace information to our trace handler(s)
            MQeTrace.setHandler(null);

            // Free up the collection of trace handlers.
            activeTraceHandlers = null;

            // Free up each handler if it's been allocated.
            handlerToAsciiFile = null;
            handlerToBinaryFile = null;
            handlerToReadable = null;
            handlerToScreen = null;

            this.hide();
            this.disableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);
            this.dispose();
            toldToClose = true;
        }

        /**
         * Set when the close() routing has been asked to shut down.
         */
        boolean toldToClose = false;

        /**
         * A menu item or widget has noticed an event. Process the event.
         * 
         * <p>All such events are passed to this routine for processing.
         */
        void doCommand(int command) {
            switch (command) {
            // When the quit menu item is selected.
            case COMMAND_MENU_ITEM_QUIT_SELECTED:
                close();
                break;

            case COMMAND_MENU_ITEM_TO_STDOUT_SELECTED:
                if (toStdOutMenuItem.getState()) {
                    traceToStdOutStart();
                } else {
                    traceToStdOutStop();
                }
                break;

            case COMMAND_MENU_ITEM_TO_SCREEN_SELECTED:
                if (toScreenMenuItem.getState()) {
                    traceToScreenStart();
                } else {
                    traceToScreenStop();
                }
                break;

            case COMMAND_MENU_ITEM_CAPTURE_STDOUT_SELECTED:
                if (captureStdOutMenuItem.getState()) {
                    captureStdoutStart();
                } else {
                    captureStdoutStop();
                }
                break;

            case COMMAND_MENU_ITEM_CAPTURE_STDERR_SELECTED:
                if (captureStdErrMenuItem.getState()) {
                    captureStdErrStart();
                } else {
                    captureStdErrStop();
                }
                break;					
										
            case COMMAND_MENU_ITEM_TO_BINARY_FILE_SELECTED:
                if (toBinaryFileMenuItem.getState()) {
                    toBinaryFileStart();
                } else {
                    toBinaryFileStop();
                }
                break;

            case COMMAND_MENU_ITEM_TO_ASCII_FILE_SELECTED:
                if (toAsciiFileMenuItem.getState()) {
                    toAsciiFileStart();
                } else {
                    toAsciiFileStop();
                }
                break;

            case COMMAND_MENU_ITEM_FILTER_ALL_SELECTED:
                if (filterAllMenuItem.getState()) {
                    filterAllStart();
                } else {
                    filterAllStop();
                }
                break;

            case COMMAND_MENU_ITEM_FILTER_NONE_SELECTED:
                if (filterNoneMenuItem.getState()) {
                    filterNoneStart();
                } else {
                    filterNoneStop();
                }
                break;

            case COMMAND_MENU_ITEM_FILTER_USER_NONE_SELECTED:
                if (filterUserNoneMenuItem.getState()) {
                    filterUserNoneStart();
                } else {
                    filterUserNoneStop();
                }
                break;

            case COMMAND_MENU_ITEM_FILTER_USER_ALL_SELECTED:
                if (filterUserAllMenuItem.getState()) {
                    filterUserAllStart();
                } else {
                    filterUserAllStop();
                }
                break;										

            case COMMAND_MENU_ITEM_FILTER_OTHER_ALL_SELECTED:
                if (filterOtherAllMenuItem.getState()) {
                    filterOtherAllStart();
                } else {
                    filterOtherAllStop();
                }
                break;

            case COMMAND_MENU_ITEM_FILTER_OTHER_NONE_SELECTED:
                if (filterOtherNoneMenuItem.getState()) {
                    filterOtherNoneStart();
                } else {
                    filterOtherNoneStop();
                }
                break;

            default:
                FilterFlag flag = (FilterFlag) filterFlags.get(String.valueOf(command));

                if (flag != null) {
                    // The command came from a regulare filter flag menu item.
                    flag.doCommand(command);
                }
            }
        }

        /**
         * Reference to the trace handler which puts information into a binary file.
         */
        private com.ibm.mqe.trace.MQeTraceToBinaryFile handlerToBinaryFile = null;

        /**
         * Starts tracing to binary file.
         */
        private void toBinaryFileStart() {

            // Create and use the "open" file dialog.
            java.awt.FileDialog openDialog = new java.awt.FileDialog(this, 
                    "Open traceToBinary file", java.awt.FileDialog.LOAD 
                );
																
            // Indicate that this is a modal dialog box.																
            openDialog.setModal(DIALOG_MODALITY_MODAL);																
            // Cause the dialog to be actioned by the user.
            openDialog.setVisible(true);
		
            // Gather the name of the selected traceToBinary file...
            String fileName = openDialog.getFile();
            String directory = openDialog.getDirectory();
            String suffix = null;

            // Discard the dialog.
            openDialog.setVisible(false);
            openDialog.dispose();
		
            // Create a to-readable trace handler which writes to that print stream.
            if (handlerToBinaryFile == null) {
                handlerToBinaryFile = new com.ibm.mqe.trace.MQeTraceToBinaryFile(directory, fileName, suffix, 0, -1);
                activeTraceHandlers.addHandler(handlerToBinaryFile);
            }
			 
        }

        /** 
         * Stops tracing to binary file.
         */
        private void toBinaryFileStop() {
            // Only do anything if we are already tracing to binary file.
            if (handlerToBinaryFile != null) {
                activeTraceHandlers.removeHandler(handlerToBinaryFile);
            }
            // Free up the binary file trace handler.
            handlerToBinaryFile = null;
        }

        /**
         * Reference to the trace handler which puts information to an ascii readable file.
         */
        private com.ibm.mqe.trace.MQeTraceToReadable handlerToAsciiFile = null;

        /**
         * Start tracing to an ascii file.
         */
        private void toAsciiFileStart() {
            // Make sure that the tracing to ascii file has stopped.
            toAsciiFileStop();
            // Create and use the "open" file dialog.
            java.awt.FileDialog openDialog = new java.awt.FileDialog(this, 
                    "Open traceToAscii file", java.awt.FileDialog.LOAD 
                );
																
            // Indicate that this is a modal dialog box.																
            openDialog.setModal(DIALOG_MODALITY_MODAL);																
            // Cause the dialog to be actioned by the user.
            openDialog.setVisible(true);
		
            // Gather the name of the selected traceToBinary file...
            String fileName = openDialog.getFile();
            String directory = openDialog.getDirectory();
	        
            // Discard the dialog.
            openDialog.setVisible(false);
            openDialog.dispose();
			
            // Open a file output stream...
            File outputAsciiFile = new File(directory + fileName);

            if (!outputAsciiFile.exists()) {
                try {
					
                    java.io.FileOutputStream outputAsciiStream =
                        new java.io.FileOutputStream(directory + fileName);
                } catch (IOException ioex) {
                    // Should not have a problem here.
                    // If we do, print the error to stderr.
                    System.err.println("Failure to create file " + fileName);
                    ioex.printStackTrace();
                    ioex.printStackTrace(new PrintStream(outputStreamToCentralPanel));
                }
            }
            // Get an output stream to write to that file.
            try {
                java.io.FileOutputStream outputAsciiStream =
                    new java.io.FileOutputStream(outputAsciiFile);
 
                // Create a print stream which writes to the output stream.
                java.io.PrintStream outputAsciiPrintStream = new PrintStream(outputAsciiStream);

                // Create a to-readable trace handler which writes to that print stream.
                handlerToAsciiFile =
                        new com.ibm.mqe.trace.MQeTraceToReadable(
                            outputAsciiPrintStream,
                            new MQeTraceRenderer());

                activeTraceHandlers.addHandler(handlerToAsciiFile);
            } catch (FileNotFoundException fnfex) {// We should have created the file already, and never get this error.
            }
        }                     

        /**
         * Stop tracing to an ascii file.
         */
        private void toAsciiFileStop() {
            if (handlerToAsciiFile != null) {
                activeTraceHandlers.removeHandler(handlerToAsciiFile);
            }
            handlerToAsciiFile = null;
        }

        /**
         * A memory of what stdout used to be set to before we started.
         */
        private PrintStream oldStdOut = System.out;
		
        /**
         * A memory of what stderr used to be set to before we started.
         */
        private PrintStream oldStdErr = System.err;

        /**
         * A list of trace handlers which are active.
         */
        private MQeTraceSplitter activeTraceHandlers = null;

        /**
         * A trace handler which outputs to stdout.
         */
        private com.ibm.mqe.trace.MQeTraceToReadable handlerToReadable = null;

        /**
         * Start tracing readable ascii to stdout.
         */
        private void traceToStdOutStart() {
            // Make sure tracing using this handler has stopped previously.
            traceToStdOutStop();
            handlerToReadable =
                    new com.ibm.mqe.trace.MQeTraceToReadable(oldStdOut, new MQeTraceRenderer());
            activeTraceHandlers.addHandler(handlerToReadable);
        }

        /**
         * Stop tracing readable ascii to stdout.
         */
        private void traceToStdOutStop() {
            if (handlerToReadable != null) {
                activeTraceHandlers.removeHandler(handlerToReadable);
            }
            // Free up the handler we were using.
            handlerToReadable = null;
        }

        /**
         * A reference to the trace handler which logs information to the screen.
         */
        com.ibm.mqe.trace.MQeTraceToReadable handlerToScreen = null;

        /**
         * Start tracing information to the central panel in the trace window.
         */
        private void traceToScreenStart() {
            // Make sure tracing to the screen has stopped previously.
            traceToScreenStop();
            handlerToScreen =
                    new com.ibm.mqe.trace.MQeTraceToReadable(
                        new PrintStream(outputStreamToCentralPanel),
                        new MQeTraceRenderer());
            activeTraceHandlers.addHandler(handlerToScreen);
        }

        /**
         * Stop putting trace information to the middle frame of the trace panel.
         */
        private void traceToScreenStop() {
            if (handlerToScreen != null) {
                activeTraceHandlers.removeHandler(handlerToScreen);
            }
            handlerToScreen = null;
        }

        /**
         * A reference to the output stream which writes to the central panel in
         * the trace window.
         */
        private AwtOutputStream outputStreamToCentralPanel;

        /**
         * Start capturing stdout to the MQe trace mechanism.
         */
        private void captureStdoutStart() {
            OutputStream toTraceOutStream = (OutputStream) new com.ibm.mqe.trace.MQeTraceOutputStream(MQeTrace.GROUP_USER_DEFINED_3, (short) 0);

            System.setOut(new PrintStream(toTraceOutStream));
        }
		
        /**
         * Start capturing stderr to the MQe trace mechanism.
         */
        private void captureStdErrStart() {
            OutputStream toTraceOutStream = (OutputStream) new com.ibm.mqe.trace.MQeTraceOutputStream(MQeTrace.GROUP_USER_DEFINED_4, (short) 0);

            System.setErr(new PrintStream(toTraceOutStream));
        }	

        /**
         * Stop capturing stdout to the screen widget.
         */
        private void captureStdoutStop() {
            System.setOut(oldStdOut);
        }
		
        /**
         * Stop capturing stderr to the screen widget.
         */
        private void captureStdErrStop() {
            System.setErr(oldStdErr);
        }
		
    }
}
