// Version mqe_java/source/examples/eventlog/LogToUnixEventLog.java, MQeExamples, la200, la200-L030324.1 1.4

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/*                                                                              */

/* AIX: 5765-E63, Solaris: 5639-M17                                             */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.eventlog;


import  examples.nativecode.*;
import  java.util.*;
import  com.ibm.mqe.*;


/*
 * <p>To define the MQe log interface.                                               
 *                                                                              
 */
public class LogToUnixEventLog implements MQeEventLogInterface {   /* start class                                                            */
    public       static short version[] = { 2, 0, 0, 5 }; 

    protected static JavaUnix unixInterface = null;

    /*
     * Default Constructor.
     * @exception java.lang.Exception                                                              
     **/
    public LogToUnixEventLog() throws Exception {                                           /*                              */}                                           /*                              */

    /*
     * Constructor.                                                              
     *                       
     * @param logName The { { drive } Path } file name    
     * @exception java.lang.Exception         
     **/
    public LogToUnixEventLog(String logName) throws Exception {                                           /*                              */
        activate(logName);                        /* activate                     */
    }                                           /*                              */

    /*
     * Activate.                                                       
     *                       
     * @param logName The { { drive } Path } file name   
     * @return void 
     * @exception java.lang.Exception  
     */       
    public void activate(String logName) throws Exception {                                           /*                              */
        if (logName == null)                      /* name specified ?             */
            logName = "MQeLog";                       /* ... no set default           */
        unixInterface = new JavaUnix(logName);    /* create the object            */
        MQe.setEventLogHandler(this);             /* allow MQe to use it          */
    }                                           /*                              */

    /*
     * Write a record to the log file.  
     * @param logType A byte to indicate the type of log.
     * @param logNumber An integer
     * @param logMsg An Object 
     * @return void
     **/
    public void logOutput(byte logType, int logNumber, Object logMsg) {                                           /*                              */
        if (logMsg instanceof String)             /* string type ?                */
            unixInterface.record(logType, logNumber, (String) logMsg, null);
        else                                        /* ... no                       */
        
        /*                                                                          */
        if (logMsg instanceof byte[])             /* byte array type              */
            unixInterface.record(logType, logNumber, null, (byte[]) logMsg);
        else                                        /* ... no                       */
            
            /*                                                                          */
            unixInterface.record(logType,
                logNumber,
                "<<Invalid type>> " + logMsg.toString(),
                null);
    }                                           /*                              */

    /*
     * Closes the interface. 
     * @return void        
     **/
    public void close() {                                           /*                              */
        if (unixInterface != null)                /* initialised ?                */ {                                         /* ... yes                      */
            MQe.setEventLogHandler(null);           /* stop using it                */
            unixInterface.close();                   /* close it down                */
            unixInterface = null;                     /* lose the object              */
        }                                         /*                              */
    }                                           /*                              */

    /*
     * De-register an Unix event source.
     * @return void
     */
    public void finalize() {                                           /*                              */
        close();                                   /* ensure it's closed           */
    }                                           /*                              */

}   /* end class                                                              */

