// Version 1.3 mqe_java/source/examples/helloworld/Configure.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.helloworld;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;
import java.util.*;


/**
 * Configures the HelloWorldQM on your disk in the QueueManagers directory.
 *
 * <p>The resultant queue manager can then be used by the
 * {@link examples.helloworld.Run} program, and destroyed using the
 * {@link examples.helloworld.UnConfigure} program.
 *
 * <p>Most solutions separate the configuration of their disk-resident queue managers
 * from the use of those queue managers. The hello world java example does this also.
 */
public class Configure {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /**
     * Entry point invoked from the command line.
     */
    public static void main(String[] args) {
        try {
            define();
        } catch (Exception e) {
            System.out.println("Failure: " + e.toString());
            e.printStackTrace();
        }
    }

    /**
     * Defines the queue manager.
     */
    public static void define() throws Exception {

        System.out.println("Defining the queue manager.");

        String queueManagerName = "HelloWorldQM";
        String baseDirectoryName = "./QueueManagers/" + queueManagerName;

        System.out.println("Using a queue manager name of " + queueManagerName);

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(MQeRegistry.Adapter, "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(MQeRegistry.DirName, baseDirectoryName + "/Registry");
        config.putFields("Registry", registrySection);

        // Construct a queue manager configuration utility object.
        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure(config, "com.ibm.mqe.adapters.MQeDiskFieldsAdapter" + ":" + baseDirectoryName + "/Queues");

        // Define a queue manager.
        configurator.defineQueueManager();

        // Define some queues on the queue manager.
        configurator.defineDefaultAdminQueue();
        configurator.defineDefaultAdminReplyQueue();
        configurator.defineDefaultDeadLetterQueue();
        configurator.defineDefaultSystemQueue();

        // Close the queue manager configuration utility class.
        configurator.close();

        System.out.println("Defined queue manager " + queueManagerName);
    }
}
