// Version 1.3 mqe_java/source/examples/helloworld/UnConfigure.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.helloworld;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;
import java.util.*;


/**
 * Removes a queue manager created using the {@link examples.helloworld.Configure} program.
 */
public class UnConfigure {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    public static void main(String[] args) {
        try {
            undefine();
        } catch (Exception e) {
            System.out.println("Failure: " + e.toString());
            e.printStackTrace();
        }
    }

    /**
     * Removes any definitions on the queue manager which were previously defined.
     */
    public static void undefine() throws Exception {

        String queueManagerName = "HelloWorldQM";
        String baseDirectoryName = "./QueueManagers/" + queueManagerName;

        System.out.println("Using a queue manager name of " + queueManagerName);

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(MQeRegistry.Adapter, "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(MQeRegistry.DirName, baseDirectoryName + "/Registry");
        config.putFields("Registry", registrySection);

        System.out.println("Queue manager " + queueManagerName + " will now be deleted.");

        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure();

        configurator.activate(config, "com.ibm.mqe.adapters.MQeDiskFieldsAdapter" + ":" + baseDirectoryName + "/Queues");

        // Remove the configured entries for the queues we defined.
        configurator.deleteSystemQueueDefinition();
        configurator.deleteDeadLetterQueueDefinition();
        configurator.deleteAdminReplyQueueDefinition();
        configurator.deleteAdminQueueDefinition();

        // Remove the queue manager itself.
        configurator.deleteQueueManagerDefinition();

        configurator.close();

        System.out.println("Queue manager is now deleted.");
    }
}
