package examples.install;


// Version 1.3 mqe_java/source/examples/install/AwtConfigUtils.java, MQeExamples, la200, la200-L030324.1
/* -----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
import examples.awt.AwtDialog;
import examples.awt.*;
import java.io.*;
import java.awt.*;


/**
 * Collection of utility methods which all use Awt for popping dialogs
 *
 * <p>Implemented in a separate class to the other utility methods for config because
 * J2ME profiles don't have awt, but still should be able to use the SimpleCreateQM
 * example class.
 *
 */
public class AwtConfigUtils {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // file variable
    private static final String FILESPEC = "*.ini";

    /**
     * allow the user to browse for a file
     *
     * @param  frame      The parent frame.
     * @param  title      Title for the dialog box.
     * @param  filename   Start point for browsing.
     * @return            The selected file, or the original
     *                    filename if the user cancelled,
     **/
    public static String browseFile(Frame frame, String title, String filename) {
        String parent = null;

        // get the parent directory from the parameter
        if (filename != null && filename.length() > 0) {
            File f = new File(filename);

            parent = f.getParent();
        }

        // Instantiate a file dialog for a .ini configuration file
        FileDialog iniFD = new FileDialog(frame, title, FileDialog.LOAD);

        iniFD.setFile(FILESPEC);
        if (parent != null)
            iniFD.setDirectory(parent);

        // show the dialog
        iniFD.show();

        // Validate the input
        if (iniFD.getFile() == null)
            return filename;
        else
            return iniFD.getDirectory() + iniFD.getFile();
    }

    /**
     * Display a confirmation dialog box
     *
     * @param title    Title for the dialog box.
     * @param message  The message to be displayed.
     * @return         'true' if the user clicks OK, 'false' otherwise
     **/
    public static boolean confirmDialog(String title, String message) {
        AwtDialog confirmDialog =
            new AwtDialog(title,
                AwtDialog.Show_OK_Cancel,
                new String[][][]
                { { { "AP", message }
                    }
                }
            );

        // Check the dialog response - and return a boolean accordingly
        return (confirmDialog.getActionIndex(confirmDialog.South) == confirmDialog.Button_OK);
    }

    /**
     * Display a message dialog box.
     *
     * @param title    Title for the dialog box.
     * @param message  The message to be displayed.
     **/
    public static void messageDialog(String title, String message) {
        AwtDialog dialog =
            new AwtDialog(title,
                AwtDialog.Show_OK,
                new String[][][]
                { { {"L", " "}
                    },
                    { {"L", "  " + message + "  "}
                    },
                    { {"L", " " }
                    }
                }
            );
        // this will return when the user clicks OK
    }

    /**
     * Display a message dialog box.
     *
     * @param title    Title for the dialog box.
     * @param message1 The first part of a two part message to be displayed.
     * @param message2 The second part of a two part message to be displayed.
     * @return void
     **/
    public static void messageDialog(String title, String message1, String message2) {
        AwtDialog dialog =
            new AwtDialog(title,
                AwtDialog.Show_OK,
                new String[][][]
                { { {"L", " "}
                    },
                    { {"L", "  " + message1 + "  "}
                    },
                    { {"L", " "}
                    },
                    { {"L", "  " + message2 + "  "}
                    },
                    { {"L", " " }
                    }
                }
            );
        // this will return when the user clicks OK
    }

}
