// Version mqe_java/source/examples/install/CreateQueueManager.java, MQeExamples, la200, la200-L030324.1 1.9

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.install;


import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;

import examples.awt.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;


/*
 *                                                     
 * <p>This extends the SimpleCreateQM class and adds a simple window          
 * interface for creating a queue manager.                                 
 *                                                                              
 * <p>This class is concerned with constructing and managing the GUI,         
 * creating the queue manager is handled by methods in the base class.  
 *                                                              
 **/

public class CreateQueueManager extends SimpleCreateQM implements KeyListener {
    public       static short version[] = { 2, 0, 0, 5 }; 
    // Queue Manager name
    String qmName = null;

    /*
     * GUI variables
     */

    // title for frames, etc
    String titleStr;

    // Configuration File
    String    configFileLblStr;
    TextField configFileTF;
    String    configFileValue = "";

    String    browseLblStr;
    Button    browseB;

    // Directory for the queues
    String    queueDirLblStr;
    TextField queueDirTF;
    String    queueDirValue;

    // OK and Cancel buttons
    Button okB;
    String okLabelStr;
    Button cancelB;
    String cancelLabelStr;

    // Other Strings, for NLS
    String configFileStr;
    String confirmQMStr;
    String createdStr;
    String errorStr;
    String qMgrStr;
    String qMgrNameStr;
    String queueDirStr;
    String regDirStr;
    String selectCFStr;

    // has the user typed into the Queue Directory text field
    boolean queueDirChanged = false;

    /**
     * CreateQueueManager constructor.
     * Construct the frame to collect the user input, and show it.
     *
     * @param configFile  Name of the configuration file; can be null
     **/
    public CreateQueueManager(String configFile) {
        // base class constructor; this calls initStrings()
        super();

        // was a configuration file specified ?
        if (configFile != null) {
            configFileValue = configFile;
            queueDirValue = getDefaultQueuePath(configFileValue);
        } else
            queueDirValue = new File("").getAbsolutePath() + defQueueDir + File.separator;
        // build the frame
        createQMFrame QMD = buildFrame();

        // display it and let the GUI do the rest
        QMD.show();
    }

    /**
     * buildFrame - build the GUI Frame and populate it.
     * This uses a createQMFrame, which is an AwtFrame extended
     * with an Action routine. It is defined in an inner class.
     *
     * @return    The built frame.
     **/
    createQMFrame buildFrame() {
        // Create the frame
        createQMFrame QMD = new createQMFrame(titleStr);

        // and populate it
        QMD.format(AwtFrame.Center, new String[][][]
            {
                { {"L", ""}, {"L", ""}
                },                                 /* Index  0,  1 */
                { {"L", configFileLblStr}, {"T", configFileValue}
                },     /* Index  2,  3 */
                { {"L", ""}, {"B", browseLblStr}
                },                       /* Index  4,  5 */
                { {"L", ""}, {"L", ""}
                },                                 /* Index  6,  7 */
                { {"L", queueDirLblStr}, {"T", queueDirValue}
                },         /* Index  8,  9 */
                { {"L", ""}, {"L", ""}
                },                                 /* Index 10, 11 */
            }
        );

        // add the OK and Cancel buttons
        QMD.format(AwtFrame.South, new String[][][]
            { { {"L", "       "},        /* Index  0 */
                    {"B", okLabelStr},      /* Index  1 */
                    {"L", "      "},        /* Index  2 */
                    {"B", cancelLabelStr},  /* Index  3 */
                    {"L", "      "}         /* Index  4 */
                }
            }
        );

        // Use the index numbers to locate the GUI components we need
        configFileTF = (TextField) QMD.getObject(AwtFrame.Center, 3);
        browseB = (Button) QMD.getObject(AwtFrame.Center, 5);
        queueDirTF = (TextField) QMD.getObject(AwtFrame.Center, 9);

        // add a Key Listener to the queue dir TextField
        queueDirTF.addKeyListener(this);

        // The buttons are in the South Panel
        okB = (Button) QMD.getObject(AwtFrame.South, 1);
        cancelB = (Button) QMD.getObject(AwtFrame.South, 3);

        return QMD;
    }

    /**
     * Do the work: check details, ask the user for confirmation, and
     * create the queue manager. Most of this is done in the superclass method.
     *
     * @return    'true' if successful, 'false' to return to the UI
     **/
    boolean createQMgr() {
        boolean result = true;

        // get the values we need
        configFileValue = configFileTF.getText().trim();
        queueDirValue = queueDirTF.getText().trim();

        if (queueDirValue.length() == 0)
            queueDirValue = new File("").getAbsolutePath();

        // call the superclass method
        try {
            result = createQMgr(configFileValue, queueDirValue);

            if (result) // confirm that the Queue Manager definition has been created
                AwtConfigUtils.messageDialog(titleStr, qMgrStr + "  '" + qmName + "'  " + createdStr);
        } catch (Exception e) {
            // display an error dialog and return false, to return to the UI
            AwtConfigUtils.messageDialog(titleStr + " - " + errorStr, e.getMessage());
            result = false;
        }

        return result;
    }

    /**
     * confirmDetails - display the details and seek confirmation
     * This is called from the superclass method createQMgr(...)
     *
     * @return  'true' if the user confirms, 'false' otherwise
     **/
    boolean confirmDetails() {
        // Find the queue manager name.
        // We already know this is a valid configuration file, so there
        // shouldn't be any errors getting the name.
        try {
            MQeFields qmFields = configFields.getFields(MQeQueueManager.QueueManager);

            qmName = qmFields.getAscii(MQeQueueManager.Name).trim();
        } catch (Exception e) {}

        // find the registry directory
        String regDir = null;

        try {
            MQeFields regFields = configFields.getFields(MQeQueueManager.Registry);

            regDir = regFields.getAscii(MQeRegistry.DirName).trim();
        } catch (Exception e) {}

        // construct the details in a String Buffer
        StringBuffer qmDetails = new StringBuffer();
        Properties pLineSeparator = new Properties(System.getProperties());
        String NL = pLineSeparator.getProperty("line.separator");
        String colon = ":  ";

        if (qmName != null)
            qmDetails.append(qMgrNameStr).append(colon).append(qmName).append(NL).append(NL);

        qmDetails.append(configFileStr).append(colon).append(configFileValue).append(NL);

        if (regDir != null)
            qmDetails.append(regDirStr).append(colon).append(regDir).append(NL);

        qmDetails.append(queueDirStr).append(colon).append(queueDirValue).append(NL);

        // Display the details, and seek confirmation
        return AwtConfigUtils.confirmDialog(confirmQMStr, qmDetails.toString());
    }

    /**
     * Key Listener methods.
     **/
    public void keyTyped(KeyEvent e) {
        queueDirChanged = true;
    }

    public void keyPressed(KeyEvent e) {}

    public void keyReleased(KeyEvent e) {}

    /**
     * Tidy up and exit.
     **/
    private void close() {
        // no tidying up to do, just exit
        System.exit(0);
    }

    /**
     * Initialise string values using the resource bundle.
     **/
    void initStrings() {
        super.initStrings();

        ResourceBundle resources = utils.getResourceBundle();

        browseLblStr = resources.getString("browse");
        cancelLabelStr = resources.getString("cancelButton");
        configFileLblStr = resources.getString("configFile");
        configFileStr = configFileLblStr;
        confirmQMStr = resources.getString("confirmQM");
        createdStr = resources.getString("created");
        errorStr = resources.getString("error");
        okLabelStr = resources.getString("okButton");
        qMgrStr = resources.getString("qMgr");
        qMgrNameStr = resources.getString("qMgrName");
        queueDirLblStr = resources.getString("queueDir");
        queueDirStr = queueDirLblStr;
        regDirStr = resources.getString("regDir");
        selectCFStr = resources.getString("selectCF");
        titleStr = resources.getString("createTitle");
    }

    /**
     * Start the application.
     **/
    public static void main(java.lang.String[] args) {
        new CreateQueueManager(args.length == 0 ? null : args[0]);
    }

    /**********************************************************************
     * Inner Class to extend Awtframe, overriding the Action method
     **********************************************************************/
    class createQMFrame extends AwtFrame {
        String frameTitle;

        createQMFrame(String title) {
            super(title);
            frameTitle = title;
        }

        /**
         * Overide the Action method to provide interaction between the
         * components of the dialog
         **/
        public void action(Object e, int where, int index, String  choice, boolean state) {
            // exit if cancel is clicked
            if (e == cancelB)
                close();

            // if browse button is clicked, display browse dialog
            if (e == browseB) {
                configFileValue = AwtConfigUtils.browseFile(this,
                            frameTitle + " - " + selectCFStr,
                            configFileTF.getText());
                configFileTF.setText(configFileValue);
                // if the user has not changed the queue directory default, reset it
                if (!queueDirChanged)
                    queueDirTF.setText(getDefaultQueuePath(configFileValue));
            } // check the details and create the queue manager if OK is clicked
            else if (e == okB) {
                if (createQMgr())
                    close();      // completed everything, so exit
                else
                    return;       // error in details, or user does not confirm
            }
        }

    } // end of inner class createQMFrame

}

