// Version mqe_java/source/examples/install/DeleteQueueManager.java, MQeExamples, la200, la200-L030324.1 1.6

/* -----------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.install;


import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;

import examples.awt.*;

import java.awt.*;
import java.io.*;
import java.util.ResourceBundle;


/*
 * <p>This extends the SimpleDeleteQM class and adds a simple window          
 * interface for deleting a queue manager.                                 
 *                                                                              
 * <p>This class is concerned with constructing and managing the GUI,         
 * deleting the queue manager is handled by methods in the base class.                                                                  
 **/

public class DeleteQueueManager extends SimpleDeleteQM {

    public       static short version[] = { 2, 0, 0, 5 }; 
    // Queue Manager name
    String qmName = null;

    /*
     * GUI variables
     */

    // Title string for dialogs, etc
    String titleStr;

    // Configuration File
    String    configFileLblStr;
    TextField configFileTF;
    String    configFileValue = "";

    Button    browseB;
    String    browseLblStr;

    // Delete checkbox
    String    deleteLblStr;
    Checkbox  deleteCB;
    boolean   deleteValue = false;

    // OK and Cancel buttons
    Button okB;
    String okLabelStr;
    Button cancelB;
    String cancelLabelStr;

    // Other Strings, for NLS
    String configDel1Str;
    String configDel2Str;
    String configFileStr;
    String confirmQMStr;
    String deletedStr;
    String errorStr;
    String qMgrNameStr;
    String qMgrStr;
    String regDirStr;
    String selectCFStr;

    /**
     * DeleteQueueManager constructor.
     * Construct the frame to collect the user input, and show it.
     *
     * @param configFile  Name of the configuration file; can be null
     **/
    public DeleteQueueManager(String configFile) {
        // base class constructor; this calls initStrings()
        super();

        // was a configuration file specified ?
        if (configFile != null)
            configFileValue = configFile;

        // build the frame
        deleteQMFrame QMD = buildFrame();

        // display it and let the GUI do the rest
        QMD.show();
    }

    /**
     * buildFrame - build the GUI Frame and populate it.
     * This uses a deleteQMFrame, which is an AwtFrame extended
     * with an Action routine. It is defined in an inner class.
     *
     * @return    The built frame.
     **/
    public deleteQMFrame buildFrame() {
        // Create the frame
        deleteQMFrame QMD = new deleteQMFrame(titleStr);

        // and populate it
        QMD.format(AwtFrame.Center, new String[][][]
            {
                { {"L", ""}, {"L", ""}
                },                                 /* Index  0,  1 */
                { {"L", configFileLblStr}, {"T", configFileValue}
                },     /* Index  2,  3 */
                { {"L", ""}, {"B", browseLblStr}
                },                       /* Index  4,  5 */
                { {"L", ""}, {"L", ""}
                },                                 /* Index  6,  7 */
                { {"C", deleteLblStr}, {"L", ""}
                },                       /* Index  8,  9 */
                { {"L", ""}, {"L", ""}
                },                                 /* Index 10, 11 */
            }
        );

        // add the OK and Cancel buttons
        QMD.format(AwtFrame.South, new String[][][]
            { { {"L", "       "},        /* Index  0 */
                    {"B", okLabelStr},      /* Index  1 */
                    {"L", "      "},        /* Index  2 */
                    {"B", cancelLabelStr},  /* Index  3 */
                    {"L", "      "}         /* Index  4 */
                }
            }
        );

        // Use the index numbers to locate the GUI components we need
        configFileTF = (TextField) QMD.getObject(AwtFrame.Center, 3);
        browseB = (Button) QMD.getObject(AwtFrame.Center, 5);
        deleteCB = (Checkbox) QMD.getObject(AwtFrame.Center, 8);

        // The buttons are in the South Panel
        okB = (Button) QMD.getObject(AwtFrame.South, 1);
        cancelB = (Button) QMD.getObject(AwtFrame.South, 3);

        return QMD;
    }

    /**
     * Do the work: check details, ask the user for confirmation, and
     * delete the configuration. Most of this is done in the superclass method.
     *
     * @return    'true' if successful, 'false' to return to the UI
     **/
    boolean deleteConfiguration() {
        boolean result = true;

        // get the values we need
        configFileValue = configFileTF.getText().trim();
        deleteValue = deleteCB.getState();

        // call the superclass method
        try {
            result = deleteConfiguration(configFileValue, deleteValue);

            if (result) // confirm that the Queue Manager definition has been deleted
                AwtConfigUtils.messageDialog(titleStr, qMgrStr + "  '" + qmName + "'  " + deletedStr);
        } catch (Exception e) {
            // display an error dialog and return false, to return to the UI
            AwtConfigUtils.messageDialog(titleStr + " - " + errorStr, e.getMessage());
            result = false;
        }

        return result;
    }

    /**
     * confirmDetails - display the details and seek confirmation.
     * This is called from the superclass method deleteConfiguration(...)
     *
     * @return  'true' if the user confirms, 'false' otherwise
     **/
    public boolean confirmDetails() {
        // Find the queue manager name.
        // We already know it is a valid configuration file, so there
        // shouldn't be any errors getting the name.
        try {
            MQeFields qmFields = configFields.getFields(MQeQueueManager.QueueManager);

            qmName = qmFields.getAscii(MQeQueueManager.Name);
        } catch (Exception e) {}

        // find the registry directory
        String regDir = null;

        try {
            MQeFields regFields = configFields.getFields(MQeQueueManager.Registry);

            regDir = regFields.getAscii(MQeRegistry.DirName);
        } catch (Exception e) {}

        // construct the details in a String Buffer
        StringBuffer qmDetails = new StringBuffer();
        String NL = "\r\n";
        String colon = ":  ";

        if (qmName != null)
            qmDetails.append(qMgrNameStr).append(colon).append(qmName).append(NL).append(NL);

        qmDetails.append(configFileStr).append(colon).append(configFileValue).append(NL);

        if (regDir != null)
            qmDetails.append(regDirStr).append(colon).append(regDir).append(NL);

        qmDetails.append(NL);
        qmDetails.append(deleteValue ? configDel1Str : configDel2Str);

        // Display the details, and seek confirmation
        return AwtConfigUtils.confirmDialog(confirmQMStr, qmDetails.toString());
    }

    /**
     * Tidy up and exit.
     **/
    void close() {
        // no tidying up to do, just exit
        System.exit(0);
    }

    /**
     * Initialise string values using the resource bundle.
     **/
    void initStrings() {
        super.initStrings();

        ResourceBundle resources = utils.getResourceBundle();

        browseLblStr = resources.getString("browse");
        cancelLabelStr = resources.getString("cancelButton");
        configDel1Str = resources.getString("configDel1");
        configDel2Str = resources.getString("configDel2");
        configFileLblStr = resources.getString("configFile");
        configFileStr = configFileLblStr;
        confirmQMStr = resources.getString("confirmQM");
        deletedStr = resources.getString("deleted");
        deleteLblStr = resources.getString("deleteLbl");
        errorStr = resources.getString("error");
        okLabelStr = resources.getString("okButton");
        qMgrNameStr = resources.getString("qMgrName");
        qMgrStr = resources.getString("qMgr");
        regDirStr = resources.getString("regDir");
        selectCFStr = resources.getString("selectCF");
        titleStr = resources.getString("deleteTitle");
    }

    /**
     * Start the application.
     **/
    public static void main(java.lang.String[] args) {
        new DeleteQueueManager(args.length == 0 ? null : args[0]);
    }

    /**********************************************************************
     * Inner Class to extend Awtframe, overriding the Action method
     **********************************************************************/
    class deleteQMFrame extends AwtFrame {
        String frameTitle;

        deleteQMFrame(String title) {
            super(title);
            frameTitle = title;
        }

        /**
         * Overide the Action method to provide interaction with the
         * components of the frame
         **/
        public void action(Object e, int where, int index, String  choice, boolean state) {
            // exit if cancel is clicked
            if (e == cancelB)
                close();

            // if the browse button is clicked, display browse dialog
            if (e == browseB) {
                configFileValue = AwtConfigUtils.browseFile(this,
                            frameTitle + " - " + selectCFStr,
                            configFileTF.getText());
                configFileTF.setText(configFileValue);
            } // check the details and delete the queue manager if OK is clicked
            else if (e == okB) {
                if (deleteConfiguration())
                    close();      // completed everything, so exit
                else
                    return;       // error in details, or user does not confirm
            }
        }

    } // end of inner class deleteQMFrame

}

