// Version mqe_java/source/examples/install/GetCredentials.java, MQeExamples, la200, la200-L030324.1 1.9

/* -----------------------------------------------------------------------------*/

/*                  WebSphere MQ Everyplace for Multiplatforms                  */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.install;


import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;
import examples.queuemanager.MQeQueueManagerUtils;
import examples.awt.*;

import java.awt.*;
import java.io.*;
import java.util.*;


/*
 * <p>This Class uses a GUI to gather the information required to obtain      
 * new credentials for a Queue Manager's Private Registry. This must       
 * be done before the registry can be used securely.                                                          
 *                                                                              
 **/

public class GetCredentials {
    public       static short version[] = { 2, 0, 0, 5 }; 

    // Utility methods
    ConfigUtils utils;

    // Queue Manager name
    String qmName = null;

    /*
     * GUI variables
     */
    configGCFrame GCF;
    String titleStr;

    // Configuration File
    String    configFileLblStr;
    TextField configFileTF = null;
    String    configFileValue = "";

    Button    browseB;
    String    browseLblStr;

    // Private Registry's PIN
    String    prPinLblStr;
    TextField prPinTF;
    String    prPinValue = "";

    // Mini-Certificate Server (Issuance Service) Adapter
    String    prCertAdaptLblStr;
    TextField prCertAdaptTF;
    String    prCertAdaptValue = "com.ibm.mqe.adapters.MQeTcpipHttpAdapter";

    // Mini-Certificate Server (Issuance Service) Address
    String    prCertAddrLblStr;
    TextField prCertAddrTF;
    String    prCertAddrValue = "";

    // Mini-Certificate Server (Issuance Service) Port Number
    String    prCertPortLblStr;
    TextField prCertPortTF;
    String    prCertPortValue = "8085";

    // Mini-Certificate Server (Issuance Service) PIN
    String    prCertPinLblStr;
    TextField prCertPinTF;
    String    prCertPinValue = "";

    // Confirm Mini-Certificate Server (Issuance Service) PIN
    String    prVCertPinLblStr;
    TextField prVCertPinTF;
    String    prVCertPinValue = "";

    // OK and Cancel buttons
    Button okB;
    String okLabelStr;
    Button cancelB;
    String cancelLabelStr;

    // Configuration Parameters
    MQeFields configFields;
    String    prCertAddr = "";

    // Other Strings, for NLS
    String badServerPinStr;
    String certServerStr;
    String configFileStr;
    String confirmQMStr;
    String confirmServerPinStr;
    String continueStr;
    String credExistStr;
    String credRenamedStr;
    String enterForPortStr;
    String errorStr;
    String gotCredStr;
    String noFileStr;
    String noPRPinStr;
    String noServerAddrStr;
    String noServerPinStr;
    String noServerPortStr;
    String notConfigFileStr;
    String notPRStr;
    String qMgrNameStr;
    String regDirStr;
    String selectCFStr;

    /**
     * GetCredentials constructor.
     * Construct the frame to collect the user input, and show it.
     *
     * This constructor takes a filename as parameter. In this case
     * we prompt for user for a PIN etc, and perform more checks.
     *
     * @param configFile  Name of the configuration file; can be null
     **/
    public GetCredentials(String configFile) {
        // instantiate the utility class
        utils = new ConfigUtils();

        // initialise the NLS strings
        initStrings();

        // was a configuration file specified ?
        if (configFile != null)
            configFileValue = configFile;

        // build the larger frame
        GCF = buildFrame(true);

        // display it and let the GUI do the rest
        GCF.show();
    }

    /**
     * GetCredentials constructor.
     * Construct the frame to collect the user input, and show it.
     *
     * This constructor takes a MQeFields object as parameter.
     * In this case we trust the information in the fields object
     * and perform fewer checks.
     *
     * @param configFile  Name of the configuration file; can be null
     **/
    public GetCredentials(MQeFields configParms) {
        // instantiate the utility class
        utils = new ConfigUtils();

        // initialise the NLS strings
        initStrings();

        // save the parameter
        configFields = configParms;

        // build the smaller frame
        GCF = buildFrame(false);

        // display it and let the GUI do the rest
        GCF.show();
    }

    /**
     * buildFrame - build the GUI Frame and populate it.
     * This uses a configQMFrame, which is an AwtFrame extended
     * with an Action routine. It is defined in an inner class.
     *
     * @param full  If true, build a complete frame, otherwise
     *              build a cut-down frame.
     * @return      The built frame.
     **/
    public configGCFrame buildFrame(boolean full) {
        // Create the frame
        configGCFrame GCF = new configGCFrame(titleStr);

        // and populate it
        if (full) {
            GCF.format(AwtFrame.Center, new String[][][]
                {
                    { {"L", ""}, {"L", ""}
                    },                              /* Index  0,  1 */
                    { {"L", configFileLblStr}, {"T", configFileValue}
                    },  /* Index  2,  3 */
                    { {"L", ""}, {"B", browseLblStr}
                    },                    /* Index  4,  5 */
                    { {"L", ""}, {"L", ""}
                    },                              /* Index  6,  7 */
                    { {"L", prPinLblStr}, {"T*", prPinValue}
                    },            /* Index  8,  9 */
                    { {"L", ""}, {"L", ""}
                    },                              /* Index 10, 11 */
                    { {"L", prCertAdaptLblStr}, {"T", prCertAdaptValue}
                    }, /* Index 12, 13 */
                    { {"L", prCertAddrLblStr}, {"T", prCertAddrValue}
                    },   /* Index 14, 15 */
                    { {"L", prCertPortLblStr}, {"T", prCertPortValue}
                    },   /* Index 16, 17 */
                    { {"L", prCertPinLblStr}, {"T*", prCertPinValue}
                    },    /* Index 18, 19 */
                    { {"L", prVCertPinLblStr}, {"T*", prVCertPinValue}
                    },  /* Index 20, 21 */
                    { {"L", ""}, {"L", ""}
                    },                              /* Index 22, 22 */
                }
            );

            // Use the index numbers to locate the GUI components we need
            configFileTF = (TextField) GCF.getObject(AwtFrame.Center, 3);
            browseB = (Button) GCF.getObject(AwtFrame.Center, 5);
            prPinTF = (TextField) GCF.getObject(AwtFrame.Center, 9);
            prCertAdaptTF = (TextField) GCF.getObject(AwtFrame.Center, 13);
            prCertAddrTF = (TextField) GCF.getObject(AwtFrame.Center, 15);
            prCertPortTF = (TextField) GCF.getObject(AwtFrame.Center, 17);
            prCertPinTF = (TextField) GCF.getObject(AwtFrame.Center, 19);
            prVCertPinTF = (TextField) GCF.getObject(AwtFrame.Center, 21);
        } else {
            GCF.format(AwtFrame.Center, new String[][][]
                {
                    { {"L", ""}, {"L", ""}
                    },                              /* Index  0,  1 */
                    { {"L", prCertAdaptLblStr}, {"T", prCertAdaptValue}
                    }, /* Index  2,  3 */
                    { {"L", prCertAddrLblStr}, {"T", prCertAddrValue}
                    },   /* Index  4,  5 */
                    { {"L", prCertPortLblStr}, {"T", prCertPortValue}
                    },   /* Index  6,  7 */
                    { {"L", prCertPinLblStr}, {"T*", prCertPinValue}
                    },    /* Index  8,  9 */
                    { {"L", prVCertPinLblStr}, {"T*", prVCertPinValue}
                    },  /* Index 10, 11 */
                    { {"L", ""}, {"L", ""}
                    },                              /* Index 12, 13 */
                }
            );

            // Use the index numbers to locate the GUI components we need
            configFileTF = null;
            browseB = null;
            prPinTF = null;
            prCertAdaptTF = (TextField) GCF.getObject(AwtFrame.Center, 3);
            prCertAddrTF = (TextField) GCF.getObject(AwtFrame.Center, 5);
            prCertPortTF = (TextField) GCF.getObject(AwtFrame.Center, 7);
            prCertPinTF = (TextField) GCF.getObject(AwtFrame.Center, 9);
            prVCertPinTF = (TextField) GCF.getObject(AwtFrame.Center, 11);
        }

        // add the OK and Cancel buttons to the South Panel
        GCF.format(AwtFrame.South, new String[][][]
            { { {"L", "       "},       /* Index  0 */
                    {"B", okLabelStr},     /* Index  1 */
                    {"L", "      "},       /* Index  2 */
                    {"B", cancelLabelStr}, /* Index  3 */
                    {"L", "      "}        /* Index  4 */
                }
            }
        );

        okB = (Button) GCF.getObject(AwtFrame.South, 1);
        cancelB = (Button) GCF.getObject(AwtFrame.South, 3);

        return GCF;
    }

    /**
     * Check that the details the user entered are complete and consistent.
     * If this method returns true, all the values we need have been extracted
     * from the GUI.
     *
     * @return   'true' if the details are correct, 'false' if there is an error.
     **/
    private boolean checkDetails() {
        String errorMsg = null;

        // extract the values
        if (configFileTF != null) {
            // full frame
            configFileValue = configFileTF.getText();
            prPinValue = prPinTF.getText().trim();

            // Configuration file
            if (configFileValue.equals(""))
                errorMsg = noFileStr;

            // Private registry PIN
            else if (prPinValue.equals(""))
                errorMsg = noPRPinStr;

            // Check that the configuration file is valid
            else {
                configFields = utils.verifyIniFile(configFileValue);
                if (configFields == null)
                    errorMsg = "'" + configFileValue + "' " + notConfigFileStr;
            }
        }

        // always have to check these:
        if (errorMsg == null) {
            prCertAdaptValue = prCertAdaptTF.getText().trim();
            prCertAddrValue = prCertAddrTF.getText().trim();
            prCertPortValue = prCertPortTF.getText().trim();
            prCertPinValue = prCertPinTF.getText().trim();
            prVCertPinValue = prVCertPinTF.getText().trim();

            // Mini-Certificate Server (Issuance Service ) Adapter, Address and Port
            if (prCertAddrValue.equals(""))
                errorMsg = noServerAddrStr;
            else if (prCertPortValue.equals(""))
                errorMsg = noServerPortStr;
            else if ((errorMsg = utils.checkPortNumber(prCertPortValue)) != null)
                errorMsg = errorMsg + enterForPortStr;

            // Mini-Certificate Server PIN and confirmed value
            else if (prCertPinValue.equals(""))
                errorMsg = noServerPinStr;
            else if (prVCertPinValue.equals(""))
                errorMsg = confirmServerPinStr;
            else if (!prCertPinValue.equals(prVCertPinValue))
                errorMsg = badServerPinStr;
        }

        if (errorMsg == null)
            return true;     // Indicate details are acceptable
        else {
            // throw up a dialog box with the error
            AwtConfigUtils.messageDialog(titleStr + " - " + errorStr, errorMsg);

            // indicate an error by returning false
            return false;
        }
    }

    /**
     * confirmDetails - display the details and seek confirmation.
     *
     * @return  'true' if the user confirms, 'false' otherwise
     **/
    public boolean confirmDetails() {
        // Find the queue manager name.
        // We already know this is a valid configuration file, so there
        // shouldn't be any errors getting the name.
        try {
            MQeFields qmFields = configFields.getFields(MQeQueueManager.QueueManager);

            qmName = qmFields.getAscii(MQeQueueManager.Name);
        } catch (Exception e) {}

        // find the registry directory
        String regDir = null;

        try {
            MQeFields regFields = configFields.getFields(MQeQueueManager.Registry);

            regDir = regFields.getAscii(MQeRegistry.DirName);
        } catch (Exception e) {}

        // construct the details in a String Buffer
        StringBuffer qmDetails = new StringBuffer();
        Properties pLineSeparator = new Properties(System.getProperties());
        String NL = pLineSeparator.getProperty("line.separator");

        String colon = ":  ";

        if (qmName != null)
            qmDetails.append(qMgrNameStr).append(colon).append(qmName).append(NL).append(NL);

        if (configFileValue.length() > 0)
            qmDetails.append(configFileStr).append(colon).append(configFileValue).append(NL);

        if (regDir != null)
            qmDetails.append(regDirStr).append(colon).append(regDir).append(NL).append(NL);

        if (prCertAdaptValue != null && prCertAdaptValue.length() > 0)
            prCertAddr = prCertAdaptValue + ":";

        prCertAddr = prCertAddr + prCertAddrValue + ":" + prCertPortValue;

        qmDetails.append(certServerStr).append(colon).append(prCertAddr).append(NL);

        // Display the details, and seek confirmation
        return AwtConfigUtils.confirmDialog(confirmQMStr, qmDetails.toString());
    }

    /**
     * Do the work: check details, ask the user for confirmation, and
     * obtain the credentials.
     *
     * @return    'true' if successful, 'false' to return to the UI
     **/
    boolean getCredentials() {
        boolean result = true;
        String renamePrefix = null;
        Cursor frameC = null;

        try {
            // extract the registry parameters and save a copy of them
            MQeFields regFields = configFields.getFields(MQeQueueManager.Registry);

            // Use the the PIN if it was obtained from the UI
            String regPIN = (configFileTF != null) ? prPinValue : null;

            // Establish any aliases defined by the .ini file
            MQeQueueManagerUtils.processAlias(configFields);

            // open the registry configuration utility
            MQePrivateRegistryConfigure regConfig =
                new MQePrivateRegistryConfigure(qmName, regFields, regPIN);

            // check if it is a Private Registry
            if (!regConfig.isPrivateRegistry()) {
                regConfig.close();
                throw new Exception(notPRStr);
            }

            // check for existing credentials
            if (regConfig.credentialsExist()) {
                if (continueDialog())   // does the user want to continue?
                    // yes - generate a prefix for renaming
                    renamePrefix = Long.toString(new Date().getTime()) + "_";
                else
                    result = false;      // return to the UI
            }

            if (result) {
                // get the credentials
                // this could take a long time
                okB.setEnabled(false);
                cancelB.setEnabled(false);
                frameC = GCF.getCursor();
                GCF.setCursor(new Cursor(Cursor.WAIT_CURSOR));

                regConfig.getCredentials(regFields,
                    regPIN,
                    prCertAddr,
                    prCertPinValue,
                    renamePrefix);

                // confirm that the credentials have been obtained
                String msg1 = gotCredStr + "  '" + qmName + "'";

                if (renamePrefix == null)
                    AwtConfigUtils.messageDialog(titleStr, msg1);
                else
                    AwtConfigUtils.messageDialog(titleStr, msg1, credRenamedStr + ": " + renamePrefix);
            }
            // finished
            regConfig.close();
        } catch (Exception e) {
            // display an error dialog and return false, to return to the UI
            AwtConfigUtils.messageDialog(titleStr + " - " + errorStr, e.getMessage());
            result = false;
        }

        GCF.setCursor(frameC);
        okB.setEnabled(true);
        cancelB.setEnabled(true);

        return result;
    }

    /**
     * Display a dialog box to say that credentials already exist.
     *
     * @return   'true' to continue, 'false' to quit
     **/
    private boolean continueDialog() {
        AwtDialog contDialog =
            new AwtDialog(titleStr,
                AwtDialog.Show_OK_Cancel,
                new String[][][]
                { { {"L", " "}
                    },
                    { {"L", "  " + credExistStr + "  '" + qmName + "'  "}
                    },
                    { {"L", " "}
                    },
                    { {"L", "  " + continueStr + "  "}
                    },
                    { {"L", " "}
                    }
                }
            );

        return (contDialog.getActionIndex(contDialog.South) == contDialog.Button_OK);
    }

    /**
     * Tidy up and exit.
     **/
    private void close() {
        // no tidying up to do, just exit
        System.exit(0);
    }

    /**
     * Initialise all the strings using a ResourceBundle.
     **/
    void initStrings() {
        // Resources
        ResourceBundle resources = utils.getResourceBundle();

        badServerPinStr = resources.getString("badServerPin");
        browseLblStr = resources.getString("browse");
        cancelLabelStr = resources.getString("cancelButton");
        certServerStr = resources.getString("certServer");
        configFileLblStr = resources.getString("configFile");
        configFileStr = configFileLblStr;
        confirmQMStr = resources.getString("confirmQM");
        confirmServerPinStr = resources.getString("confirmServerPin");
        continueStr = resources.getString("continue");
        credExistStr = resources.getString("credExist");
        credRenamedStr = resources.getString("credRenamed");
        enterForPortStr = resources.getString("enterForPort");
        errorStr = resources.getString("error");
        gotCredStr = resources.getString("gotCred");
        noFileStr = resources.getString("noFile");
        noPRPinStr = resources.getString("noPRPin");
        noServerAddrStr = resources.getString("noServerAddr");
        noServerPinStr = resources.getString("noServerPin");
        noServerPortStr = resources.getString("noServerPort");
        notConfigFileStr = resources.getString("notConfigFile");
        notPRStr = resources.getString("notPR");
        okLabelStr = resources.getString("okButton");
        prCertAdaptLblStr = resources.getString("prCertAdaptLbl");
        prCertAddrLblStr = resources.getString("prCertAddrLbl");
        prCertPinLblStr = resources.getString("prCertPinLbl");
        prCertPortLblStr = resources.getString("prCertPortLbl");
        prPinLblStr = resources.getString("prPinLbl");
        prVCertPinLblStr = resources.getString("confirm") + prCertPinLblStr;
        qMgrNameStr = resources.getString("qMgrName");
        regDirStr = resources.getString("regDir");
        selectCFStr = resources.getString("selectCF");
        titleStr = resources.getString("getCredTitle");
    }

    /**
     * Start the application.
     **/
    public static void main(java.lang.String[] args) {
        new GetCredentials(args.length == 0 ? null : args[0]);
    }

    /**********************************************************************
     * Inner Class to extend Awtframe, overriding the Action method.
     **********************************************************************/
    class configGCFrame extends AwtFrame {
        String frameTitle;

        configGCFrame(String title) {
            super(title);
            frameTitle = title;
        }

        /**
         * Overide the Action method to provide interaction between the
         * components of the dialog.
         **/
        public void action(Object e, int where, int index, String  choice, boolean state) {
            // exit if cancel is clicked
            if (e == cancelB)
                close();

            // if browse button is clicked, display browse dialog
            if (e == browseB) {
                configFileValue = AwtConfigUtils.browseFile(this,
                            frameTitle + " - " + selectCFStr,
                            configFileTF.getText());
                configFileTF.setText(configFileValue);
                prPinTF.setText("");
            } // check the details and configure the queue manager if OK is clicked
            else if (e == okB) {
                if (checkDetails() && confirmDetails() && getCredentials())
                    close();      // completed everything, so exit
                else
                    return;       // error in details, or user does not confirm
            }
        }

    } // end of inner class configGCFrame

}

