package examples.midp.exampleapp.msgpump;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;


/**
 * This class aims to be a simple commandline user interface.
 * It captures user input on System.in and sends the information to a controller.
 * It is aimed to be run as a thread in the background to ensure that blocking
 * functions do not prevent the user from inputting command.
 *
 * <p>Start and prompt texts are also allowed - the starttext is shown when the class
 * is started and the prompttext is displayed after every command the user enters.
 *
 * <p>The thread will keep running until the user enters the command 'quit' at which
 * point this final command will be sent to the controller to process and this
 * thread will stop().
 *
 * <p>This will only be used by commandline interfaces and not by MIDP UIs.
 */

public class InputThread implements Runnable {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version
  
    // The thread's controller, which will be used to handle any user requests
    private NormalClient controller;

    // String to prompt the user for text with, or initially display
    private String starttext;
    private String prompttext;

    /**
     * Constructor for InputThread.
     */
    public InputThread(NormalClient controller, String starttext, String prompttext) {
        this.controller = controller;
        this.starttext = starttext;
        this.prompttext = prompttext;
    }

    /**
     *     Waits for user input and then sends it to the controller.
     *     Stops after the user enters the command 'quit' (this command is sent
     *     to the controller before it stops).
     *
     * @return void
     */
    public void run() {

        /* Create new Buffered Reader to read user input from */
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

        controller.processMsg(starttext);
        controller.processMsg(prompttext);
        String input = "";

        try {
            while (!input.equals("quit")) {
                input = reader.readLine();
                controller.processInput(input);
                controller.processMsg(prompttext);
            }
        } catch (IOException e) {
            controller.processMsg("IOException on input reader");
            e.printStackTrace();
        }
        controller.processMsg("[Input thread] Finished");
    }
}
