package examples.midp.exampleapp.textapp;



/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import com.ibm.mqe.MQeMsgObject;



/**
 * The aim of the client model is to register with the registration server and
 * be prepared to send messages to the gateway server. Unlike the message pump,
 * messages will only be sent by the request of the user.
 *
 * <p>To achieve these goals, remote queues to the gateway and registration reply
 * and request queues are required as well as a connection to both those servers.
 *
 * <p>Since everything is driven by user input, almost all the work is done by the
 * client controller. The model mainly sets the queue manager up correctly to
 * allow it to perform its functions.
 */
public class AppModelClient extends AppModel {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /* These are the IPs where messages are send */
    private String regServer = DEFAULT_REGISTRATION;
    private String gatewayServer = DEFAULT_GATEWAY;


    /**
     *   This is not a server.
     * @see examples.midp.exampleapp.textapp.AppModel#isServer
     */
    protected boolean isServer() {
        return false;
    }

    /**
     *   Sets a property of the model.
     */
    protected void setProperty(String property, String value) {
        if (property.equals("RegistrationIP")) {
            if (value != null) {
                regServer = "Network://" + value + ":" + DEFAULT_REGISTRATION_LISTENER_PORT;
            }
        }
        if (property.equals("GatewayIP")) {
            if (value != null) {
                gatewayServer = "Network://" + value + ":" + DEFAULT_LISTENER_PORT;
            }
        }
    }


    /**
     *    A home server queue on the gateway is used by the client.
     * @see examples.midp.exampleapp.textapp.AppModel#initializeOtherQueues
     */
    protected void initializeOtherQueues() throws Exception {
        getMsgService().createHomeServerQueue(
            getLocalQMname(),
            DEFAULT_GATEWAY_QM,
            DEFAULT_HOMESERVER_Q,
            0);
    }


    /**
     * Configures the model - clients need to register with the central server.
     * @see examples.midp.exampleapp.textapp.AppModel#initialConfiguration
     */
    protected void initialConfiguration() throws Exception {
        super.initialConfiguration();
        registerWithCentral(regServer, DEFAULT_REGISTRATION_Q);
    }

    /**
     *   We require the ability to send to the request queue of the gateway server and
     *   to request a registration (and check the reply) from the reg server.
     * @see examples.midp.exampleapp.textapp.AppModel#chooseRemoteQueues
     */
    protected String[] chooseRemoteQueues() {
        return new String[] {
                DEFAULT_GATEWAY_QM,
                DEFAULT_REQUEST_Q,
                "async",
                DEFAULT_REGISTRATION_QM,
                DEFAULT_REGISTRATION_Q,
                "sync",
                DEFAULT_REGISTRATION_QM,
                DEFAULT_REGISTRATION_REPLY_Q,
                "sync" };
    }


    /**
     *   Need connections to the reg and gateway servers.
     * @see examples.midp.exampleapp.textapp.AppModel#chooseConnections
     */
    protected String[] chooseConnections() {
        return new String[] {
                DEFAULT_REGISTRATION_QM,
                regServer,
                RegistrationCommand,
                RegistrationOptions,
                DEFAULT_GATEWAY_QM,
                gatewayServer,
                GatewayCommand,
                GatewayOptions };
    }


    /**
     * @see examples.midp.exampleapp.textapp.AppModel#processDataGramMsg(MQeMsgObject, String, String)
     */
    public void processDataGramMsg(MQeMsgObject message, String qMgrName, String qName) {
        processStatus("*ERROR* Client received message from " + qName + "@" + qMgrName);
    }


    /**
     * @see examples.midp.exampleapp.textapp.AppModel#processReplyMsg(MQeMsgObject, String, String)
     */
    public void processReplyMsg(MQeMsgObject message, String qMgrName, String qName) {
        processStatus("*ERROR* Client received message from " + qName + "@" + qMgrName);
    }


    /**
     * @see examples.midp.exampleapp.textapp.AppModel#processRequestMsg(MQeMsgObject, String, String)
     */
    public void processRequestMsg(MQeMsgObject message, String qMgrName, String qName) {
        processStatus("*ERROR* Client received message from " + qName + "@" + qMgrName);
    }


    /**
     * (client shouldn't have a listener but returns something anyway).
     * @see examples.midp.exampleapp.textapp.AppModel#getListenerPort
     */
    protected int getListenerPort() {
        return DEFAULT_LISTENER_PORT;
    }
}
