// Version mqe_java/source/examples/mqbridge/administration/commandline/MQBridgeQueueCreator.java, MQe, la200, la200-L030324.1 1.7

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.mqbridge.administration.commandline; 


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;
 

/**
 * A command-line tool which creates an MQ bridge queue on a queue manager.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQBridgeQueueCreator 
 * &lt;bridgeName&gt;
 * &lt;proxyName&gt;
 * &lt;connectionName&gt; 
 * &lt;qName&gt; 
 * &lt;transformerClass&gt; 
 * &lt;compressor&gt; 
 * &lt;cryptor&gt; 
 * &lt;authenticator&gt;
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * 
 * <P><B>Parameters</B></P>
 * 
 * <DL>
 * <DT>bridgeName</DT>
 * <DD>The name of the parent bridge.</DD>
 * 
 * <DT>proxyName</DT>
 * <DD>The name of the WMQ queue manager proxy</DD>
 * 
 * <DT>connectionName</DT>
 * <DD>The name of the connection. This must match the name of a server-channel
 * connection on the WMQ queue manager.\n</DD>
 * 
 * <DT>qName</DT>
 * <DD>The name of the WMQ queue to which this bridge queue remotely referrs.</DD>
 * 
 * <DT>transformerClass</DT>
 * <DD>The name of the java class used to convert between message formats.
 * Must include the .class extension.
 * Specify "null" if you want to use the default transformer for this bridge.</DD>
 * 
 * <DT>compressor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * compressor the queue should use to store messages on the disk.
 * Use "null" if you don't want a compressor.</DD>
 * 
 * <DT>cryptor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * cryptor the queue should use to store messages on the disk.
 * Use "null" if you don't want a cryptor.</DD>
 * 
 * <DT>authenticator</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * authenticator the queue should use to store messages on the disk.
 * Use "null" if you don't want an authenticator.</DD>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the WMQe queue manager to which the administration message is
 * to be sent to, and which will have the new object configured as a result.
 * The queue manager must be a bridge-enabled queue manager.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path of the .ini file which describes a queue manager which will be
 * be used to issue the administration message</DD>
 * </DL>
 */
public class MQBridgeQueueCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 
    
    public       static short version[] = { 2, 0, 0, 5 }; 

    static final String syntax = 
        "Syntax: java MQBridgeQueueCreator <bridgeName> <proxyName> <connectionName>"
        + "<qName> <transformerClass> <compressor> <cryptor> <authenticator> <qMgr> <iniFile>\n"
        + "Where...\n"
        + "bridgeName\n"
        + "  The name of the parent bridge.\n"
        + "proxyName\n"
        + "  The name of the WMQ queue manager proxy.\n"
        + "connectionName\n"
        + "  The name of the connection. This must match the name of a server-channel\n"
        + "  connection on the WMQ queue manager.\n"
        + "qName\n"
        + "  The name of the WMQ queue to which this bridge queue remotely referrs.\n"
        + "transformerClass\n"
        + "  The name of the java class used to convert between message formats.\n"
        + "  Must include the .class extension.\n"
        + "  Specify \"null\" if you want to use the default transformer for this bridge.\n"
        + "compressor\n"
        + "  The name of the compressor class to use when transmitting data to this MQbridge queue\n"
        + "  between WMQe queue managers\n"
        + "cryptor\n"
        + "  The name of the cryptor class to use when transmitting data to this MQbridge queue\n"
        + "  between WMQe queue managers\n"
        + "authenticator\n"
        + "  The name of the cryptor class to use when transmitting data to this MQbridge queue\n"
        + "  between WMQe queue managers\n"
        + "qMgr\n"
        + "  The name of the WMQe queue manager to which the administration message is\n"
        + "  to be sent to, and which will have the new object configured as a result.\n"
        + "  The queue manager must be a bridge-enabled queue manager.\n"
        + "iniFile\n"
        + "  The fully-qualified path of the .ini file which describes a queue manager which will be\n"
        + "  be used to issue the administration message.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQBridgeQueueCreator me = new MQBridgeQueueCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method to create the bridge queue.
     * 
     * @param args
     * @return void 
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {
        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
        String bridgeName = null;   // The name of the bridge to create
        String proxyName = null;   // The name of the proxy to create
        String connectionName = null;// The name of the MQ client connection to use.
        String qName = null;        // The name of the queue to create.
        String transformerClass = null; // Name of the class used to convert between msg formats.
        String compressorClass = null; // Compressor to use when sending data to this queue.
        String cryptorClass = null; // The cryptor class to use when sending data to this queue.
        String authenticatorClass = null; // The authenticator class to use when sending data to this queue.

        // Check for the number of parameters.
        if (args.length < 10) {
            throw new Exception("Error: Not enough parameters.\n" + syntax);
        }
        if (args.length > 10) {
            throw new Exception("Error: Too many parameters.\n" + syntax);
        }

        // Get the parameters from the system properties...
        bridgeName = args[0].trim();
        proxyName = args[1].trim();
        connectionName = args[2].trim();
        qName = args[3].trim();
        transformerClass = args[4].trim();
        compressorClass = args[5].trim();
        cryptorClass = args[6].trim();
        authenticatorClass = args[7].trim();
        targetQMgr = args[8].trim();
        iniFile = args[9].trim();

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Create the queue...
        createBridgeQueue(bridgeName, proxyName, connectionName, qName, transformerClass, compressorClass, cryptorClass, authenticatorClass, targetQMgr, myQMgr 
        );

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will create a bridge, and sends it
     * to a named queue manager.
     * 
     * @param bridgeName The name of the bridge which the queue will use to communicate with MQ
     * 
     * @param proxyName The name of the MQ queue manager proxy which will be used to connect to the
     *                  WMQ queue manager when this MQBridge queue is used.
     * 
     * @param connectionName
     *                   The name of the client connection which will be used to get a connection
     *                   when the new bridge queue wishes to talk to the WMQ system.
     * 
     * @param qName  The name of the bridge queue. Normally corresponds to the name of the queue which
     *               exists on the WMQ system.
     * 
     * @param transformerClass
     *                   The name of the transformer class to be used by the new bridge queue when messages
     *                   are converted between WMQe and WMQ format.
     *                   A special value of "null" will cause the bridge queue to use the default transformer
     *                   set up in the bridge object configuration.
     * 
     * @param targetQMgr The name of the queue manager on which the bridge queue will be created.
     * 
     * @param myQMgr A reference to an active WMQe queue manager, which will be used to send and receive administration
     *               messages.
     * @return void
     * @exception Exception
     */
    protected void createBridgeQueue(String bridgeName, String proxyName, String connectionName, String qName, String transformerClass, String compressorClass, String cryptorClass, String authenticatorClass, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        MQeMQBridgeQueueAdminMsg msg = new MQeMQBridgeQueueAdminMsg(bridgeName, proxyName, connectionName, 3 // hold idle channels in the queue for 3 minutes
            );

        // Create a fields object in which we can pre-initialise the object...
        MQeFields params = new MQeFields();

        params.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Synchronous);

        if (transformerClass.equals("null")) {// Don't specify a transformer... it will use the default transformer set 
            // at the bridge configuration level.
        } else {
            params.putUnicode(MQeMQBridgeQueueAdminMsg.Queue_Transformer, transformerClass);
        }

        // Fill the params with any other information we need for our queue definition.
        if (!cryptorClass.equals("null")) {
            params.putAscii(msg.Queue_Cryptor, cryptorClass);
        }
        if (!authenticatorClass.equals("null")) {
            params.putAscii(msg.Queue_Authenticator, authenticatorClass);
        }
        if (!compressorClass.equals("null")) {
            params.putAscii(msg.Queue_Compressor, compressorClass);
        }

        // Set the command in the admin msg to "create".
        msg.create(params);

        // Set the name of the queue into the admin msg...
        msg.setName(proxyName, qName);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
    } 
}
