// Version mqe_java/source/examples/mqbridge/administration/commandline/MQListenerCreator.java, MQe, la200, la200-L030324.1 1.7

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.mqbridge.administration.commandline; 


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;
 
                           
/**
 * A command-line tool to create an WMQ transmission queue listener on
 * an WMQe queue manager.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>
 * java examples.mqbridge.administration.commandline.MQListenerCreator 
 * &lt;bridgeName&gt;
 * &lt;proxyName&gt; 
 * &lt;connectionName&gt; 
 * &lt;listenerName&gt; 
 * &lt;transformer&gt; 
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * 
 * <DL>
 * <DT>bridgeName</DT>
 * <DD>The name of the parent bridge.</DD>
 * 
 * <DT>proxyName</DT>
 * <DD>The name of the WMQ queue manager proxy.</DD>
 * 
 * <DT>connectionName</DT>
 * <DD>The name of the connection. This must match the name of a server-channel
 * connection on the WMQ queue manager.</DD>
 * 
 * <DT>listenerName</DT>
 * <DD>The name of the listener. This must match the name of a transmission queue on WMQ
 * transformer.
 * The name of the transformer to use to convert messages pulled from the WMQ transmission queue
 * This is a class name, including the .class extension.
 * Specify "null" if you wish to default the transformer to use the one specified on the bridge object.</DD>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the WMQe queue manager to which the administration message is
 * to be sent to, and which will have the new object configured as a result.
 * The queue manager must be a bridge-enabled queue manager.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path of the .ini file which describes a queue manager which will be
 * be used to issue the administration message.</DD>
 * 
 * </DL>
 */
public class MQListenerCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public       static short version[] = { 2, 0, 0, 5 }; 

    static final String syntax = 
        "Syntax: java MQListenerCreator <bridgeName> <proxyName> <connectionName> <listenerName> <transformer> <qMgr> <iniFile>\n"
        + "Where...\n"
        + "bridgeName\n"
        + "  The name of the parent bridge.\n"
        + "proxyName\n"
        + "  The name of the WMQ queue manager proxy.\n"
        + "connectionName\n"
        + "  The name of the connection. This must match the name of a server-channel\n"
        + "  connection on the WMQ queue manager.\n"
        + "listenerName\n"
        + "  The name of the listener. This must match the name of a transmission queue on WMQ\n"
        + "transformer\n"
        + "  The name of the transformer to use to convert messages pulled from the WMQ transmission queue\n"
        + "  This is a class name, including the .class extension.\n"
        + "  Specify \"null\" if you wish to default the transformer to use the one specified on the bridge object\n"
        + "qMgr\n"
        + "  The name of the WMQe queue manager to which the administration message is\n"
        + "  to be sent to, and which will have the new object configured as a result.\n"
        + "  The queue manager must be a bridge-enabled queue manager.\n"
        + "iniFile\n"
        + "  The fully-qualified path of the .ini file which describes a queue manager which will be\n"
        + "  be used to issue the administration message.\n";

    /**
     * Entry point when the tool is invokde from the comand-line. 
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) {
        int rc = 0;
        MQListenerCreator me = new MQListenerCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Collects command-line parameters into variables, and calls the command which creates the listener.
     * 
     * @param args
     * @return void
     * @exception java.lang.Exception
     *
     */
    public void activate(String args[]) throws Exception {
        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
        String bridgeName = null;   // The name of the bridge to create
        String proxyName = null;   // The name of the proxy to create
        String connectionName = null;// The name of the WMQ client connection to use.
        String listenerName = null;// The name of the transmission queue to read from.
        String transformerClass = null; // The name of the transformer class to use during msg conversions.

        if (args.length < 7) {
            throw new Exception("Error: Not enough parameters.\n" + syntax);
        }

        // Get the parameters from the system properties...
        bridgeName = args[0].trim();
        proxyName = args[1].trim();
        connectionName = args[2].trim();
        listenerName = args[3].trim();
        transformerClass = args[4].trim();
        targetQMgr = args[5].trim();
        iniFile = args[6].trim();

        // Access the local WMQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Create the queue...
        createListener(bridgeName, proxyName, connectionName, listenerName, transformerClass, targetQMgr, myQMgr 
        );

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the WMQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will create a bridge transmission queue listener, and sends it
     * to a named queue manager.
     * 
     * @param bridgeName The name of the bridge for which the listener will be created.
     * 
     * @param proxyName The name of the WMQ queue manager proxy for which the transmission queue listener will be created.
     * 
     * @param connectionName
     *                   The name of the client connection object for which the transmission queue listener will be created.
     * 
     * @param listenerName
     *                   The name of the listener to create.
     *                   This name should match the name of a transmission queue on the WMQ queue manager.
     * 
     * @param transformerClass
     *                   The class to use (including the .class extension) which will be used by the listener to
     *                   convert between WMQe and WMQ message format.
     *                   A special value of "null" means the default transformer for the bridge is used.
     * 
     * @param targetQMgr The queue manager on which the listener will be created.
     * 
     * @param myQMgr A reference to an active WMQe queue manager which will be used to send and receive admin messages and responses.
     * @return void
     * @exception Exception
     */
    protected void createListener(String bridgeName, String proxyName, String connectionName, String listenerName, String transformerClass, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        // Create a fields object in which we can pre-initialise the object...
        MQeFields params = new MQeFields();

        if (transformerClass.equals("null")) {// Don't set the field to use the transformer set on the bridge object.
        } else {
            params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_TRANSFORMER, transformerClass
            );
        }

        // Set the command in the admin msg to "create".
        msg.create(params);

        // Set the name of the object we wish to create into the admin msg...
        msg.setName(bridgeName, proxyName, connectionName, listenerName);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
    } 

} 
