/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */ 

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.console;


import examples.administration.console.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;
import examples.awt.*;
import java.awt.*;


/*****************************************************************************
 * 
 * A class providing a specialist form of the example admin application
 * which has extra MQbridge-related functions added.
 *
 ******************************************************************************/
public class AdminGateway extends Admin implements java.beans.PropertyChangeListener {

    /**
     * Version number of this MQe object...
     */
    public          static short version[] = { 2, 0, 0, 5 }; 

    /** 
     * Each type of resource that is managed has its own panel where
     * the list of admin actions supported for the resource are displayed
     */

    // References to the bridge-specific text fields which are put into the 
    // central panel of the admin app.
    TextField bridgeNameTextField = null;
    TextField clientConnectionNameTextField = null;
    TextField proxyNameTextField = null;
    TextField listenerNameTextField = null;
    Checkbox  affectChildrenCheckbox = null;

    AdminModel  adminModel = null;

    // Override the title of the application.
    // This is used in the "about" dialog, and the title of the application's
    // window frame.
    public static String appTitle = "Example MQe Gateway Admin Console";

    // Indices of the bridge-specific buttons on the west panel of the 
    // admin frame.
    public int westButtonIndexManageBridges = 0;
    public int westButtonIndexManageBridge = 0;
    public int westButtonIndexManageProxy = 0;
    public int westButtonIndexManageClientConnection = 0;
    public int westButtonIndexManageListener = 0;
    public int westButtonIndexManageMQBridgeQ = 0;

    public int centralBridgeNameTextFieldIndex = 0;
    public int centralProxyNameTextFieldIndex = 0;
    public int centralClientConnectionNameTextFieldIndex = 0;
    public int centralListenerNameTextFieldIndex = 0;
    public int centralAffectChildrenCheckboxFieldIndex = 0;

    /**
     * Constructor
     */
    public AdminGateway() throws Exception {
        super();
    }

    public AdminGateway(String args[]) throws Exception {
        super(args);
    }

    /**
     * Main method - standalone startup of Example Gateway administration GUI
     * ...Must over-ride the superclass main() so that the AdminGateway is
     * created rather than the base Admin object.
     */
    public static void main(String[] args) {
        try {

            // Instantiate the application
            new AdminGateway(args).activate();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Returns the current admin model object, or a new one if one
     * didn't exist.
     * @return AdminModel
     */
    public AdminModel getAdminModel() {
        // If there is no admin model instantiate one
        if (adminModel == null) {
            try {
                adminModel = (AdminModel) new AdminBridgeModel();
            } catch (Exception ex) {
                message("getAdminModel(): " + ex.toString());
            }
        };
        return adminModel;
    }

    /**
     * Creates a list of the labels which appear at the center of the admin 
     * panel.
     */
    public String[][][] getCenterNLSLabels() {

        String[][][] parentLabels = super.getCenterNLSLabels();

        int base = parentLabels.length;
        String[][][] results = new String[base + 5][][];

        for (int i = 0; i < base; i++) {
            results[i] = parentLabels[i];
        }

        centralBridgeNameTextFieldIndex = base * 2 + 1;
        centralProxyNameTextFieldIndex = base * 2 + 3;
        centralClientConnectionNameTextFieldIndex = base * 2 + 5;
        centralListenerNameTextFieldIndex = base * 2 + 7;
        centralAffectChildrenCheckboxFieldIndex = base * 2 + 9;

        results[base] = new String[][] {
                    { "L", "Default Bridge"},         
                    { "T", "MQBridgeV100"} 
                };

        results[base + 1] = new String[][] {
                    { "L", "Default MQ QMgr Proxy"},
                    { "T", ""}
                };

        results[base + 2] = new String[][] { 
                    { "L", "Default MQ Client Connection"},    
                    { "T", ""}
                };

        results[base + 3] = new String[][] { 
                    { "L", "Default MQ XmitQ Listener"},
                    { "T", ""}
                };

        results[base + 4] = new String[][] { 
                    { "L", "Default 'Affects Children' Flag"},
                    { "C!", ""}
                };

        return results;
    }

    /**
     * Initialises references to the GUI widgets.
     */
    public void setupWidgetReferences() {

        // Allow the superclass to set up its own widgets...
        super.setupWidgetReferences();

        // Get references to the text fields holding bridge-specific things
        // so we can access their values quickly.
        bridgeNameTextField = (java.awt.TextField) getObject('C' /* center panel */, centralBridgeNameTextFieldIndex 
                );

        proxyNameTextField = (java.awt.TextField) getObject('C' /* center panel */, centralProxyNameTextFieldIndex
                );

        clientConnectionNameTextField = (java.awt.TextField) getObject('C' /* center panel */, centralClientConnectionNameTextFieldIndex 
                );

        listenerNameTextField = (java.awt.TextField) getObject('C' /* center panel */, centralListenerNameTextFieldIndex
                );

        affectChildrenCheckbox = (java.awt.Checkbox) getObject('C' /* center panel */, centralAffectChildrenCheckboxFieldIndex
                );

    }

    /**
     * Process an MQe awt event
     */
    public void action(Object e, int Where, int Index, String  Choice, boolean State) {

        // Get the current values out of the central fields...
        if (bridgeNameTextField != null) {
            ((AdminBridgeModel) getAdminModel()).bridgeName = bridgeNameTextField.getText();
        }
        if (proxyNameTextField != null) {
            ((AdminBridgeModel) getAdminModel()).proxyName = proxyNameTextField.getText();
        }
        if (clientConnectionNameTextField != null) {
            ((AdminBridgeModel) getAdminModel()).clientConnectionName = clientConnectionNameTextField.getText();
        }
        if (listenerNameTextField != null) {
            ((AdminBridgeModel) getAdminModel()).listenerName = listenerNameTextField.getText();
        }

        if (affectChildrenCheckbox != null) {
            ((AdminBridgeModel) getAdminModel()).affectChildren = affectChildrenCheckbox.getState();
        }

        // Let the superclass handle its actions if it can...
        super.action(e, Where, Index, Choice, State);

    }

    /**
     * Method to handle events for the PropertyChangeListener interface.
     * @param evt java.beans.PropertyChangeEvent
     */
    public void propertyChange(java.beans.PropertyChangeEvent evt) {

        // Let the superclass handle the event as normal...
        super.propertyChange(evt);

        if ((evt.getSource() == getAdminModel()) && (evt.getPropertyName().equals("manageQM"))) {
            // Queue manager name changed so display it
            String managedQM = getAdminModel().getManageQM();

            bridgeNameTextField.setText(managedQM + ".MQBridgeV100");
        }
    }

    /**
     * Returns the set of AdminFrames for resources to be managed
     * @return java.lang.String[]
     */                                                                      
    public String[] getResourceClasses() {
        String[] original = super.getResourceClasses();

        String[] results = new String[ original.length + 6 ];

        for (int i = 0; i < original.length; i += 1) {
            results[i] = original[i];
        }
        results[original.length] = "examples.mqbridge.administration.console.AdminMQBridges";
        results[original.length + 1] = "examples.mqbridge.administration.console.AdminMQBridge";
        results[original.length + 2] = "examples.mqbridge.administration.console.AdminMQQMgrProxy";
        results[original.length + 3] = "examples.mqbridge.administration.console.AdminMQClientConnection";
        results[original.length + 4] = "examples.mqbridge.administration.console.AdminMQListener";
        results[original.length + 5] = "examples.mqbridge.administration.console.AdminMQBridgeQueue";
        return results;
    }
}
