/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */ 

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/ 
package examples.mqbridge.administration.console;


import com.ibm.mqe.*;
import examples.administration.console.*;
import com.ibm.mqe.mqbridge.*;
import examples.awt.*;
import java.awt.*; 


/********************************************************************************
 *
 * Provides an example of how you might administer an MQ-bridge object
 * This class provides an extra button on the west panel.
 * A set of action-buttons are provided in the east panel also.
 *
 ********************************************************************************/
public class AdminMQBridge extends AdminFrame {

    public        static short    version[] = { 2, 0, 0, 5 }; 

    /**
     * Constructor.
     *
     * Creates a default AdminMQBridge object.
     */
    public AdminMQBridge() {
        super("MQ Bridge Administration");
    }

    /**
     * Constructor.
     *
     * @param msg An admin message to contain the parameters for the action. If it already 
     * contains any parameters then these are taken as defaults
     * @param action The action to be performed
     * @param adminModel The admin model which will send the message.
     * 
     */
    public AdminMQBridge(MQeAdminMsg msg, int action, AdminModel adminModel) throws Exception {
        super(msg, action, adminModel);
    }

    /**
     * Constructor.
     *
     * @param String The title of the frame.
     */
    public AdminMQBridge(String title) throws Exception {
        super(title);
    }

    /**
     * Add any aliases related to this resource
     */
    public void addAlias() throws Exception {
        MQe.alias("MQeMQBridgeAdminMsgGUI",
            "examples.mqbridge.administration.console.AdminMQBridge");
    }

    /**
     * @return Panel - an actions panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        String[][][] actions = new String[][][]
            { { { "B", "Inquire"}
                },     // 0
                { { "B", "Update"}
                },     // 1
                { { "B", "Start"}
                },       // 2
                { { "B", "Stop"}
                },        // 3
                { { "B", "Create"}
                },      // 4 
                { { "B", "Delete"}
                }       // 5
            };

        // ..Queue actions
        AwtFormat actionF = new AwtFormat(eventHandler, East);

        return actionF.panel(new Panel(), actions);
    }

    /**     
     * Return the name of resource being managed
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return "MQ Bridge";
    }

    /**
     * Process a queue manager action request
     */
    public void resourceAction(Object e, int Where, int Index, String  Choice, boolean State 
    ) throws Exception {
        int act = -1;       // Unknown action

        // Decide on which action the user selected ?
        switch (Index) {
        case 0:
            act = MQeMQBridgeAdminMsg.Action_InquireAll;
            break;              

        case 1:
            act = MQeMQBridgeAdminMsg.Action_Update;        
            break;

        case 2:
            act = MQeMQBridgeAdminMsg.Action_Start;
            break;

        case 3:
            act = MQeMQBridgeAdminMsg.Action_Stop;
            break;

        case 4:
            act = MQeMQBridgeAdminMsg.Action_Create;
            break;

        case 5:
            act = MQeMQBridgeAdminMsg.Action_Delete;
            break;
        }

        // Display the message in a window	
        activate(new MQeMQBridgeAdminMsg(((AdminBridgeModel) adminModel).bridgeName, ((AdminBridgeModel) adminModel).affectChildren
            ), act, adminModel 
        );    
    }

    /**
     * Setup the admin message with the action to be performed and the
     * parameters required by the action
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {
        switch (new Integer(action).intValue()) {
        case MQeMQBridgeAdminMsg.Action_InquireAll:
            // Inquire 
            ((MQeMQBridgeAdminMsg) msg).inquireAll(parms);
            break;

        case MQeMQBridgeAdminMsg.Action_Update:
            // Update
            ((MQeMQBridgeAdminMsg) msg).update(parms);
            break;

        case MQeMQBridgeAdminMsg.Action_Start:
            // Start
            ((MQeMQBridgeAdminMsg) msg).start(parms);
            break;

        case MQeMQBridgeAdminMsg.Action_Stop:
            // Stop
            ((MQeMQBridgeAdminMsg) msg).stop(parms);
            break;

        case MQeMQBridgeAdminMsg.Action_Create:
            // Create
            ((MQeMQBridgeAdminMsg) msg).create(parms);
            break;

        case MQeMQBridgeAdminMsg.Action_Delete:
            // Delete
            ((MQeMQBridgeAdminMsg) msg).delete(parms);
            break;
        }
        return (MQeAdminMsg) msg;
    }

    /**
     * Configure the list of actions that can be performed.
     */
    protected void setupActions() {
        super.setupActions();

        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_Update),
            new String[][] {    
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, "O"}, // Optional
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_ADMINISTERED_OBJECT_CLASS, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, "O"},
            } 
        );

        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_InquireAll), 
            new String[][] {    
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
            } 
        );
        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_Start),  
            new String[][] {    
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_AFFECT_CHILDREN, "M"},
            } 
        );
        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_Stop),   
            new String[][] {    
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
            } 
        );
        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_Create),
            new String[][] {  
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, "O"}, // Optional
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_ADMINISTERED_OBJECT_CLASS, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS, "O"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, "O"},
            } 
        );
        addAction(new String().valueOf(MQeMQBridgeAdminMsg.Action_Delete),
            new String[][] {    
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "M"},
                { MQeCharacteristicLabels.MQE_FIELD_LABEL_AFFECT_CHILDREN, "M"},
            } 
        );
    }

    /**
     * Set up a valid set of characteristics
     * - the sequence is important as this is the sequence that the fields
     *   will be displayed
     */
    protected void setupFields() {
        super.setupFields();

        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_ADMINISTERED_OBJECT_CLASS, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE, "TF", null);
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN, "C", null);  // A list field
        addCharacteristic(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, "TF", null);
    }

    /**
     * Set up a list of labels for the panel.
     */
    protected void setupNLS() {
        super.setupNLS();

        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "MQ Bridge Name *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, "Default Transformer Class *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, "Heartbeat Interval (minutes) *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_ADMINISTERED_OBJECT_CLASS, "Bridge Class *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS, "Startup Rule class *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE, "Run State");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN, "MQ QMgr Proxy Children");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_AFFECT_CHILDREN, "Affect children *");
        addNLSName(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, "Description *");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_InquireAll), "Inquire");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_Update), "Update");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_Start), "Start");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_Stop), "Stop");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_Create), "Create");
        addNLSName((new String()).valueOf(MQeMQBridgeAdminMsg.Action_Delete), "Delete");
        addNLSName("cancel", "Cancel");
    }

    /**
     * Returns a string that provides information about the current frame.
     * This is displayed in the south panel of the main admin gui when a frame is selected.
     * @return String documenting the current frame.
     */
    public String getMessageHelp() {
        return "The MQ Bridge allows MQSeries Everyplace to communicate with MQSeries Queue Managers.";
    }
}
