/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */ 

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.console;


import com.ibm.mqe.*;
import examples.administration.console.*;
import com.ibm.mqe.administration.*;
import java.util.*;
import com.ibm.mqe.mqbridge.*;
import examples.awt.*;
import java.awt.*; 


/********************************************************************************
 *
 * Provides an example of how you might administer an MQ-bridge-queue object
 * This class provides an extra button on the west panel.
 * A set of action-buttons are provided in the east panel also.
 *
 ********************************************************************************/
public class AdminMQBridgeQueue extends AdminRemoteQueue {

    public        static short    version[] = { 2, 0, 0, 5 }; 

    /**
     * The action which denotes "put an MQ Style message to a queue".
     */
    public static final int Action_PutTestMQMessage = 102;

    /**
     * Default constructor
     */
    public AdminMQBridgeQueue() {
        super();
    }

    /**
     * Main constructor.
     *
     * @param msg An admin message to contain the parameters for the action. If it already
     * contains any parameters then these are taken as defaults
     * @param action The action to be performed
     * @param adminModel The admin model which will send the message.
     *
     */
    public AdminMQBridgeQueue(MQeAdminMsg msg, int action, AdminModel adminModel
    ) throws Exception {
        super(msg, action, adminModel);
    }

    /**
     * AdminQueue constructor
     */
    public AdminMQBridgeQueue(String title) throws Exception {
        super(title);
    }

    /**
     * Deal with the action to put an MQ style message to a test queue.
     */
    public void action(Object e, int where, int index, String  choice, boolean state) {
        boolean handled = false;

        try {
            switch (where) {                          // based on N, S, E, W, C, M
            case South:
    
                // If OK or apply button then process action
                if (e == actionB || e == applyB) {
                    
                    // Check input is valid
                    if (!validateInput())
                        break;
    
                    // Get the queue/qmgr name from the user entered data...
                    MQeFields parms = getChanges();
                    String qN = parms.getAscii(MQeQueueAdminMsg.Admin_Name);
                    String qmN = null;
    
                    if (parms.contains(MQeQueueAdminMsg.Queue_QMgrName)) {
                        qmN = parms.getAscii(MQeQueueAdminMsg.Queue_QMgrName);
    
                    }
    
                    int thisAction = new Integer(action).intValue();

                    if (thisAction == Action_PutTestMQMessage) {
    
                        // Deal with the put/get test message.
                        // Enclose in a try/catch block- be prepared for the test msg failing.
                        try {
    
                            sendTestMessage(qN, qmN);
    
                            // Create an "it worked" dialog for user feedback.
                            new AwtDialog("Test MQ Message sent ok", AwtDialog.Show_OK, new String[][][] {
                                    { { "L", "Good"}
                                    },
                                }
                            );
    
                        } catch (Exception problem) {
    
                            // Work out the failure string we will report.
                            String problemString = problem.toString();

                            if (problem instanceof MQeException) {
                                problemString = "(Failure Code="
                                        + ((MQeException) problem).code()
                                        + ") "
                                        + problemString;
                            }
    
                            new AwtDialog("Failed!", AwtDialog.Show_OK, new String[][][] {
                                    { { "AP", problemString}
                                    },
                                }
                            );
    
                            problem.printStackTrace();
                        } finally {
                            close();
                            handled = true;
                        }
                    } // end if put test MQ message
                } // end if action == ok or apply
    
            } // end switch
    
            if (!handled)
                super.action(e, where, index, choice, state);
    
        } catch (Exception ex) {
            message(e.toString());
            ex.printStackTrace();
        }
    }

    /**
     * Add any aliases related to this resource
     */
    public void addAlias() throws Exception {
        MQe.alias("MQeMQBridgeQueueAdminMsgGUI",
            "examples.mqbridge.administration.console.AdminMQBridgeQueue");
    }

    /**
     * @return An actions panel
     */
    public Panel getActionsPanel(AwtEvent eventHandler) {
        String[][][] actions = new String[][][]


            { { { "B", "Inquire"}
                }, // 0
                { { "B", "Update"}
                },  // 1
                { { "B", "Create"}
                },  // 2
                { { "B", "Delete"}
                },  // 3
                { { "B", "Put Test MQ Message"}
                }, // 4
                { { "B", "Get Message"}
                }, // 5

            };

        // ..Queue actions
        AwtFormat actionF = new AwtFormat(eventHandler, East);
        Panel results = actionF.panel(new Panel(), actions);

        return results;
    }

    /**
     * Return the name of resource being managed
     * @return java.lang.String
     */
    public String getResourceTitle() {
        return "MQ Bridge Queue";
    }

    /**
     * Process a queue manager action request
     */
    public void resourceAction(Object e, int Where, int Index, String  Choice, boolean State
    ) throws Exception {
        int act = -1;       // Unknown action

        // Decide on which action user selected ?
        switch (Index) {
        case 0:
            act = MQeListenerAdminMsg.Action_InquireAll;
            break;

        case 1:
            act = MQeListenerAdminMsg.Action_Update;
            break;

        case 2:
            act = MQeListenerAdminMsg.Action_Create;
            break;

        case 3:
            act = MQeListenerAdminMsg.Action_Delete;
            break;

        case 4:
            act = AdminMQBridgeQueue.Action_PutTestMQMessage;
            break;

        case 5:
            act = AdminQueue.Action_GetMessage;
            break;
        }

        MQeMQBridgeQueueAdminMsg msg = new MQeMQBridgeQueueAdminMsg(((AdminBridgeModel) adminModel).bridgeName, ((AdminBridgeModel) adminModel).proxyName, ((AdminBridgeModel) adminModel).clientConnectionName, 5 
            );

        try {
            // Display the message in a window
            activate(msg, act, adminModel
            );
        } catch (Exception ex) {
            System.out.println("Exception when activating the dialog.:" + ex + "\n");
            ex.printStackTrace();
        }
    }

    /**
     * Configure the valid set of actions
     */
    protected String[][] getAddCharacteristics() {
        return new String[][] {

                { MQeQueueAdminMsg.Admin_Name, "M"},
                { MQeQueueAdminMsg.Queue_QMgrName, "M"},
                { MQeQueueAdminMsg.Queue_Description, "O"},
                { MQeQueueAdminMsg.Queue_Mode, "O"},
                { MQeQueueAdminMsg.Queue_Cryptor, "O"},
                { MQeQueueAdminMsg.Queue_Authenticator, "O"},
                { MQeQueueAdminMsg.Queue_Compressor, "O"},
                { MQeQueueAdminMsg.Queue_TargetRegistry, "O"},
                { MQeQueueAdminMsg.Queue_AttrRule, "O"},
                { MQeQueueAdminMsg.Queue_Rule, "O"},
                { MQeQueueAdminMsg.Queue_Expiry, "O"},
                { MQeQueueAdminMsg.Queue_MaxMsgSize, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_BridgeName, "M"},
                { MQeMQBridgeQueueAdminMsg.Queue_MQQMgr, "M"},
                { MQeMQBridgeQueueAdminMsg.Queue_ClientConnection, "M"},
                { MQeMQBridgeQueueAdminMsg.Queue_RemoteQName, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_Transformer, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_MaxIdleTime, "O"},
            };
    }

    /**
     * Updates a set of actions
     */
    protected String[][] getUpdateCharacteristics() {
        return  new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M"},
                { MQeQueueAdminMsg.Queue_QMgrName, "M"},
                { MQeQueueAdminMsg.Queue_Mode, "O"},
                { MQeQueueAdminMsg.Queue_Description, "O"},
                { MQeQueueAdminMsg.Queue_Cryptor, "O"},
                { MQeQueueAdminMsg.Queue_Authenticator, "O"},
                { MQeQueueAdminMsg.Queue_Compressor, "O"},
                { MQeQueueAdminMsg.Queue_TargetRegistry, "O"},
                { MQeQueueAdminMsg.Queue_AttrRule, "O"},
                { MQeQueueAdminMsg.Queue_Rule, "O"},
                { MQeQueueAdminMsg.Queue_Expiry, "O"},
                { MQeQueueAdminMsg.Queue_MaxMsgSize, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_BridgeName, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_MQQMgr, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_ClientConnection, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_RemoteQName, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_Transformer, "O"},
                { MQeMQBridgeQueueAdminMsg.Queue_MaxIdleTime, "O"},
            };
    }

    /**
     * Set up the admin message with the action to be performed and the
     * parameters required by the action
     */
    public MQeAdminMsg setupAction(MQeFields parms) throws Exception {

        return (MQeAdminMsg) super.setupAction(parms);
    }

    /**
     * Configure the valid set of actions
     */
    protected void setupActions() {

        addAction(new String().valueOf(Action_PutTestMQMessage),
            new String[][] {
                { MQeQueueAdminMsg.Admin_Name, "M" },
                { MQeQueueAdminMsg.Queue_QMgrName, "O" }
            }
        );

        super.setupActions();
    }

    /**
     * Set up the valid set of characteristics
     * - the sequence is important as this is the sequence that the fields
     *   will displayed
     */
    protected void setupFields() {

        super.setupFields();

        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_BridgeName, "TF", null);
        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_MQQMgr, "TF", null);
        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_ClientConnection, "TF", null);
        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_RemoteQName, "TF", null);
        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_Transformer, "TF", null);
        addCharacteristic(MQeMQBridgeQueueAdminMsg.Queue_MaxIdleTime, "TF", null);
    }

    /**
     * Setup labels for the various items.
     */
    protected void setupNLS() {
        super.setupNLS();

        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_BridgeName, "MQ Bridge name *");
        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_MQQMgr, "MQ Queue Manager proxy name");
        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_ClientConnection, "Client Connection name");
        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_RemoteQName, "Remote MQ Queue name *");
        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_Transformer, "Transformer Class *");
        addNLSName(MQeMQBridgeQueueAdminMsg.Queue_MaxIdleTime, "Idle Time (minutes) *");

        addNLSName((new String()).valueOf(Action_PutTestMQMessage), "Put Test MQ Message");
    }

    /**
     * Send a test message to the specified q on the specified q manager.
     */
    protected void sendTestMessage(String qName, String qMgrName
    ) throws Exception {
        com.ibm.mqe.mqemqmessage.MQeMQMsgObject msg = new com.ibm.mqe.mqemqmessage.MQeMQMsgObject();

        msg.putArrayOfByte("MQ_Data", new byte[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 }
        );

        // Put the message to the queue...
        adminModel.getQM().putMessage(qMgrName                     // Queue manager we're sending to.
            , qName                       // Bridge queue we're sending to.
            , msg                         // Message we're sending.
            , null                        // Attributes of the message.
            , (long) 0                     // We're not sending a confirm.
        );
    }

    /**
     * Returns a string that provides information about the current frame.
     * This is displayed in the south panel of the main admin gui when a frame is selected.
     * @return String documenting the current frame.
     */
    public String getMessageHelp() {
        return "MQ Bridge Queues are references to queues on MQSeries Queue Managers.";
    }

}
