// Version 1.2 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperBridges.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.administration.*;


public class AdminHelperBridges {
    public static short[] version = { 2, 0, 0, 5 }; 	
    private Reporter reporter; 
    private AdminMessageDispatcher dispatcher;
	
    public AdminHelperBridges(Reporter reporter, AdminMessageDispatcher dispatcher) {
        this.reporter = reporter;
        this.dispatcher = dispatcher;
    }
	
    /**
     * Finds out if the bridges list exists of not.
     * This is equivalent to askin the queue manager whether the "bridge-enabled" flag is set.
     * @return boolean true if the bridges list exists, and the queue manager is bridge-enabled,
     * false otherwise.
     */
    public boolean doesExist(String targetMQeQueueManagerName) throws Exception {
		
        boolean doesExist = false;
		
        MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg();
        MQeFields params = new MQeFields();

        // When we do the inquire, the bridge capable flag is the only 
        // thing we're interrested in.
        params.putBoolean(MQeQueueManagerAdminMsg.QMgr_BridgeCapable, true);
        msg.inquire(params);
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);

        if (!results.contains(MQeQueueManagerAdminMsg.QMgr_BridgeCapable)) {
            throw new Exception("Error: cannot find out if the target queue manager is bridge capable.\n"
                    + "       Could indicate that the target queue manager is using a version\n"
                    + "       of MQe prior to v1.2.7"
                );
        } else {								
            // The result has been sent back to us.
            doesExist = results.getBoolean(MQeQueueManagerAdminMsg.QMgr_BridgeCapable);
        } 
			
        return doesExist;
    }
	
    /**
     * Starts the bridges resource running, and optionally starts all children resources too.
     */
    public void start(String targetMQeQueueManagerName, boolean affectsChildren
    ) throws Exception {
        MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();

        msg.start(affectsChildren);
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }	
	
    /**
     * Stops the bridges resource, and all resources in the hierarchy below from running.
     */
    public void stop(String targetMQeQueueManagerName
    ) throws Exception {
        MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();

        msg.stop();
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }
		
    /**
     * Lists all the children resources of the bridges object.
     * <p>Each such resource is the name of a bridge objcet.
     */		
    public Hashtable listChildren(String targetMQeQueueManagerName) throws Exception {
        Hashtable results = new Hashtable();
        MQeFields queryResults = queryAll(targetMQeQueueManagerName);		
        MQeFields[] children = queryResults.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);

        for (int childIndex = 0; childIndex < children.length; childIndex += 1) {	
            String resourceName = children[childIndex].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);

            results.put(resourceName, resourceName);	
        }
        return results;											     	
    }
		
    /**
     * Queries all facets of the bridges resource.
     */		
    public MQeFields queryAll(String targetMQeQueueManagerName) throws Exception {
        MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();

        msg.inquireAll(new MQeFields());
        MQeFields queryResults = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);

        return queryResults;												  													  	
    }
	
    /**
     * Checks to see if the bridges resource is running or not.
     */
    public boolean isRunning(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        boolean isRunning = false;
        MQeFields queryResults = queryAll(targetMQeQueueManagerName);
        int runState = queryResults.getInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE);

        if (runState == MQeRunState.RUN_STATE_RUNNING) {
            isRunning = true;
        } else {
            isRunning = false;
        }		   												
        return isRunning;												  	
    }
}
