// Version 1.3 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperConnection.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.administration.*;


/**
 * Provides methods to manipulate client connections.
 */
public class AdminHelperConnection {
    public static short[] version = { 2, 0, 0, 5 }; 	
    Reporter reporter;
    AdminMessageDispatcher dispatcher;
	
    /**
     * Builds an object able to administer bridge resources.
     * 
     * @param reporter The object to which debug and error information is sent.
     * @param dispatcher An object used to dispatch administration messages.
     */
    public AdminHelperConnection(Reporter reporter, AdminMessageDispatcher dispatcher
    ) {
        this.reporter = reporter;
        this.dispatcher = dispatcher;
    }	
	
    /**
     * Indicates whether the connection exists on a specified queue manager or not.
     */
    public boolean doesExist(String mqeQueueManager, String connectionName) throws Exception {

        AdminHelperQueueManager helper = new AdminHelperQueueManager(reporter, dispatcher);
        MQeFields connectionData = helper.queryAll(mqeQueueManager);
        String[] connectionList = connectionData.getAsciiArray(MQeQueueManagerAdminMsg.QMgr_Connections);
        boolean found = false;

        for (int i = 0; (i < connectionList.length) & (!found); i += 1) {
            if (connectionList[i].equals(connectionName)) {
                found = true;
            }
        }		
        return found;
    }

    public boolean createConnectionToMQQMgr(String mqeQueueManagerName, String connectionName 
    ) throws Exception {
        boolean isOk = true;
												 	
        reporter.comment("Creating a connection to " + connectionName + " on queue manager " + mqeQueueManagerName);
		
        MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg(connectionName);
        MQeFields params = new MQeFields();

        params.putAscii(msg.Con_Channel, null);

        // Say this admin message should create the connection...
        msg.create(params);

        // Set the name of the connection.
        msg.setName(connectionName);
        reporter.comment("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, mqeQueueManagerName, msg);

        if (results == null) {
            isOk = false;
        }
        return isOk;
    }										 										 	
										 
    /**
     * Creates a connection to another queue manager using a TCP-style comms adapter.
     * @param mqeQueueManagerName
     * @param connectionName
     * @param channelClassName
     * @param protocolAndAddress
     * @throws Exception
     */
    public void createDirectTcpConnection(String mqeQueueManagerName, String connectionName, String channelClassName, String protocol, String ipAddress, int    port
    ) throws Exception {
										   	
        boolean isOk = true;										   	
		
        reporter.comment("Creating a connection to " + connectionName + " on queue manager " + mqeQueueManagerName);
		
        MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg(connectionName);

        MQeFields params = new MQeFields();

        if (channelClassName == null) {
            params.putAscii(msg.Con_Channel, null);
        } else {
            if (channelClassName.equals("default")) {// Do nothing.
            } else {
                params.putAscii(msg.Con_Channel, channelClassName);
            }
        }	              
       	
        String destination = protocol + ":" + ipAddress + ":" + port;

        params.putAscii(msg.Con_Adapter, destination);

        // Say this admin message should create the connection...
        msg.create(params);

        // Set the name of the connection.
        msg.setName(connectionName);
        reporter.comment("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, mqeQueueManagerName, msg);

        if (null == results) {
            isOk = false;
        }
    }        	
}
