// Version 1.1 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperLocalQueue.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.administration.*;


/**
 * Actions which manipulate local queues.
 */
public class AdminHelperLocalQueue {
    public static short[] version = { 2, 0, 0, 5 }; 	
    Reporter reporter;
    AdminMessageDispatcher dispatcher;
	
    /**
     * Builds an object able to administer a local queue resource.
     * 
     * @param reporter The object to which debug and error information is sent.
     * @param dispatcher An object used to dispatch administration messages.
     */
    public AdminHelperLocalQueue(Reporter reporter, AdminMessageDispatcher dispatcher
    ) {
        this.reporter = reporter;
        this.dispatcher = dispatcher;
    }
		
    /**
     * Finds out if the queue exists already
     */
    public boolean doesExist(String mqeQueueManagerName, String queueQMgrName, String queueQueueName 
    ) throws Exception {
			
        boolean doesExist = true;					  	
        AdminHelperQueueManager	helper = new AdminHelperQueueManager(reporter, dispatcher);

        doesExist = helper.doesQueueExist(mqeQueueManagerName, queueQMgrName, queueQueueName);				
        return doesExist;							  	
    }
	
    /**
     * Creates the local queue.
     */
    public void create(String mqeQueueManagerName, String queueName
    ) throws Exception {
					   	
        try {
            MQeQueueAdminMsg msg = new MQeQueueAdminMsg(mqeQueueManagerName, queueName);
            MQeFields params = null;

            msg.create(params);	

            // Send the message.
            MQeFields results = dispatcher.sendWaitMessage(reporter, mqeQueueManagerName, msg);
	        
        } catch (Exception e) {
            reporter.error(e);
            throw e;
        }
    }
	
    /**
     * Creates a local queue with an explicit storage location for its' messages.
     */
    public void create(String mqeQueueManagerName, String queueName, String storageLocation
    ) throws Exception {
        try {
            MQeQueueAdminMsg msg = new MQeQueueAdminMsg(mqeQueueManagerName, queueName);
            MQeFields params = new MQeFields();

            params.putAscii(msg.Queue_FileDesc, storageLocation);
            msg.create(params);	

            // Send the message.
            MQeFields results = dispatcher.sendWaitMessage(reporter, mqeQueueManagerName, msg);
	        
        } catch (Exception e) {
            reporter.error(e);
            throw e;
        }								
    }			
	
    /**
     * Remove the local queue.
     */
    public void delete(String mqeQueueManagerName, String queueName
    ) throws Exception {
        try {
            MQeQueueAdminMsg msg = new MQeQueueAdminMsg(mqeQueueManagerName, queueName);
            MQeFields params = new MQeFields();

            msg.delete(params);	

            // Send the message.
            MQeFields results = dispatcher.sendWaitMessage(reporter, mqeQueueManagerName, msg);	        
        } catch (Exception e) {
            reporter.error(e);
            throw e;
        }										
    }					
}
