
// Version 1.2 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperMQTransmitQueueListener.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
 
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;


/**
 * Utility methods used to manipulate a transmit queue listener resource.
 */
public class AdminHelperMQTransmitQueueListener {
    public static short[] version = { 2, 0, 0, 5 }; 	
    private Reporter reporter; 
    private AdminMessageDispatcher dispatcher;
	
    public AdminHelperMQTransmitQueueListener(Reporter reporter, AdminMessageDispatcher dispatcher) {
        this.reporter = reporter;
        this.dispatcher = dispatcher;
    }	
	
    /**
     * Queries all facets of the MQ transmission queue listener object
     */
    public MQeFields queryAll(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName
    ) throws Exception {
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);
        msg.inquireAll(new MQeFields());
        MQeFields queryResults = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);

        return queryResults;												  													  	
    }	
	
    /**
     * Starts the resource running.
     */
    public void start(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName, boolean affectsChildren
    ) throws Exception {
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);
        msg.start(affectsChildren);
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }	
	
    /**
     * Stops the resource from running.
     */
    public void stop(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName
    ) throws Exception {
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);
        msg.stop();
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }			
	
    /**
     * Checks to see if the resource is running or not.
     */
    public boolean isRunning(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName
    ) throws Exception {
        boolean isRunning = false;
        MQeFields queryResults = queryAll(targetMQeQueueManagerName, bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName
            );
        int runState = queryResults.getInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE);

        if (runState == MQeRunState.RUN_STATE_RUNNING) {
            isRunning = true;
        } else {
            isRunning = false;
        }		   												
        return isRunning;												  	
    }
		
    /**
     * Checks to see if the resource exists of not.
     */
    public boolean doesExist(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String transmitQueueListenerName
    ) throws Exception {
        boolean doesExist = false;
		
        reporter.comment("looking to see if transmit queue listener exists or not."
            + "\n\tmqe queue manager name=" + targetMQeQueueManagerName
            + "\n\tbridge name=" + bridgeName
            + "\n\tproxy name=" + mqQMgrProxyName
            + "\n\tclient connection name=" + clientConnectionName
            + "\n"
        );

        AdminHelperMQClientConnection helper = new AdminHelperMQClientConnection(reporter, dispatcher);						  	
        Hashtable childList = helper.listChildren(targetMQeQueueManagerName, bridgeName, mqQMgrProxyName, clientConnectionName 
            );

        if (childList.get(transmitQueueListenerName) == null) {
            doesExist = false;
        } else {
            doesExist = true;
        }
	
        return doesExist;																
    }	
	
    /**
     * Creates a transmit queue listener resource
     */
    public void create(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName, String resourceDescription, String deadLetterQueueName, String listenerStateStoreAdapterClassName, String undeliveredMessageRuleClassName
    ) throws Exception {
        reporter.debug("Building the simple admin message to create an client connection resource "
            + " queue manager effected=" + targetMQeQueueManagerName
            + " bridge name=" + bridgeName
            + " queue manager proxy name=" + mqQMgrProxyName
            + " client connection name=" + clientConnectionName
            + " resource description=" + resourceDescription
        );
            	        
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, resourceDescription);
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEAD_LETTER_Q_NAME, deadLetterQueueName);
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_LISTENER_STATE_STORE_ADAPTER, listenerStateStoreAdapterClassName);
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_UNDELIVERED_MESSAGE_RULE_CLASS, undeliveredMessageRuleClassName);
       
        // Set the command in the admin msg to "create".
        msg.create(params);
        
        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);

        reporter.debug("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  							  			  	 
    }
						  			  	
    /**
     * Creates a transmit queue listener resource, defaulting some of the values.
     */
    public void create(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName, String resourceDescription, String deadLetterQueueName
    ) throws Exception {
        reporter.debug("Building the simple admin message to create an client connection resource "
            + " queue manager effected=" + targetMQeQueueManagerName
            + " bridge name=" + bridgeName
            + " queue manager proxy name=" + mqQMgrProxyName
            + " client connection name=" + clientConnectionName
            + " resource description=" + resourceDescription
        );
            	        
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, resourceDescription);
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEAD_LETTER_Q_NAME, deadLetterQueueName);
	
        // Set the command in the admin msg to "create".
        msg.create(params);
        
        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);

        reporter.debug("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  							  			  	 
    }
						  			  						  			  	
    /**
     * Deletes a resource from a specified queue manager.
     */
    public void delete(String targetMQeQueueManagerName, String bridgeName, String mqQMgrProxyName, String clientConnectionName, String mqTransmitQueueListenerName
    ) throws Exception {

        reporter.debug("Building the admin message to delete a client connection resource "
            + "queue manager effected=" + targetMQeQueueManagerName
            + "bridge name=" + bridgeName
            + " queue manager proxy name=" + mqQMgrProxyName
            + " client connection name=" + clientConnectionName
        );
        MQeListenerAdminMsg msg = new MQeListenerAdminMsg();
        MQeFields params = new MQeFields();

        msg.delete(params);
        msg.setName(bridgeName, mqQMgrProxyName, clientConnectionName, mqTransmitQueueListenerName);
        reporter.debug("built the admin message.");
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  									 																		
    }	
}
