// Version 1.3 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperReporter.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.io.*;
import java.util.*;
import com.ibm.mqe.*;


/**
 * Provides a error reporting mechanism.
 * 
 * <p>Debug, error, warning, comment input is collected together, and 
 * can be obtained using the getCollectedOutput method.  
 */
public class AdminHelperReporter implements Reporter {
    public static short[] version = { 2, 0, 0, 5 }; 	
    private ByteArrayOutputStream collectedOutput;
    private PrintStream collectedPrintStream;
	
    public AdminHelperReporter() {
        collectedOutput = new ByteArrayOutputStream();
        collectedPrintStream = new PrintStream(collectedOutput);
        indentLevel = 3;
    }
	
    public static final boolean DEBUG = false;
	
    public void warning(String text) {
        output("Warning:" + text);
    }
	
    public void error(String text) {
        output("Error: " + text);		
    }
	
    public void enter(String text) {
        comment(text);
        indent();
    } 
	
    public void exit(String text) {
        comment(text);
        outdent();		
    }
	
    public void exit(String text, Throwable t) {
        error(text);
        error(t);
        outdent();
    }
	
    /**
     * Converts a throwable to a string, indluding the stack trace.
     * @param t
     * @return String
     */
    private static String throwableToString(Throwable t) {
        ByteArrayOutputStream tempData = new ByteArrayOutputStream();
        PrintStream tempPrintStream = new PrintStream(tempData);

        t.printStackTrace(tempPrintStream);
        return tempData.toString();
    }
	
    public void error(Throwable t) {
        output("Error: " + t.toString() + "\n" + throwableToString(t));						
    }
	
    public void debug(String text) {
        trace(text);
    }
	
    private void trace(String text) {
        MQeTrace.trace(null, (short) 0, MQeTrace.GROUP_USER_DEFINED_1, text);
    }
	
    public void comment(String text) {
        output(text);		
    }
	
    public void exception(Throwable t) {
        output("Exception:" + throwableToString(t));
    }
	
    /**
     * Put the text string we're passed to the output print stream, indenting wherever we find a \n character.
     * @param text
     */
    private void output(String unIndentedText) {
        String indentString = getIndent();
        StringBuffer indentedText = new StringBuffer();
        int along = 0;
        int lineEnd = 0;
        int lineStart = 0;
        int linesInInput = 0;

        while ((lineEnd = unIndentedText.indexOf("\n", lineStart)) > 0) {
            if (linesInInput > 0) {
                indentedText.append("\n");
            }
            indentedText.append(indentString);
            indentedText.append(unIndentedText.substring(lineStart, lineEnd));
            lineStart = lineEnd + 1;
            linesInInput += 1;
        }
        if (linesInInput > 0) {
            indentedText.append("\n");
        } 
        indentedText.append(getIndent());
        indentedText.append(unIndentedText.substring(lineStart, unIndentedText.length()));		
		
        collectedPrintStream.println(indentedText.toString());
        trace(indentedText.toString());
    }
	
    /**
     * Returns all the output we collected.
     * 
     * @return String
     */
    public String getCollectedOutput() {
        return collectedOutput.toString();				
    }
	
    public void indent() {
        indentLevel += 1;
    }
	
    public void outdent() {
        indentLevel -= 1;
        if (indentLevel < 0) {
            indentLevel = 0;
        }
    }
	
    private static final String SINGLE_INDENT_AS_SPACES = "    ";
    private int indentLevel;
	
    private String getIndent() {
        StringBuffer indentString = new StringBuffer();

        for (int i = 0; i < indentLevel; i += 1) {
            indentString.append(SINGLE_INDENT_AS_SPACES);
        }
        return indentString.toString();	
    }
}
