// Version mqe_java/source/examples/nativecode/JavaNT.java, MQeExamples, la200, la200-L030324.1 1.5

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.nativecode;


/*
 * JavaNT, Native method interface.                               
 **/
public class JavaNT {   /* start NT Native class                                                  */
    public static short  version[] = { 2, 0, 0, 5 }; 

    /*----------------------------------------------------------------------------*/
    
    /* Logon user types                                                           */
    
    /*----------------------------------------------------------------------------*/
    public final static int   INTERACTIVE = 2;
    public final static int   NETWORK = 3;
    public final static int   BATCH = 4;
    public final static int   SERVICE = 5;

    /*----------------------------------------------------------------------------*/
    
    /* Event log types                                                            */
    
    /*----------------------------------------------------------------------------*/
    public final static byte  EVENTLOG_SUCCESS = 0x00;
    public final static byte  EVENTLOG_ERROR_TYPE = 0x01;
    public final static byte  EVENTLOG_WARNING_TYPE = 0x02;
    public final static byte  EVENTLOG_INFORMATION_TYPE = 0x04;
    public final static byte  EVENTLOG_AUDIT_SUCCESS = 0x08;
    public final static byte  EVENTLOG_AUDIT_FAILURE = 0x10;

    protected int    handle = 0;

    /**
     * Logon a user and return Logon Handle.  
     * @param domain Domain, Unix/NT
     * @param userid User Id to logon with
     * @param password The password
     * @param logonType  Type of logon, user/adminstrator/superuser.
     * @return Logon handle, an integer value.
     **/
    public native int     logonUser(String domain,
        String userid,
        String password,
        int    logonType);

    /*
     * Re-set any impersonation and free Logon Handle.
     * @param logonHandle  Logon Handle to be reset.
     * @return void            
     **/
    public native void    logoffUser(int    logonHandle);

    /*
     * Impersonate user by Logon Handle. 
     * @param logonHandle  Logon Handle 
     * @return a boolean
     **/
    public native boolean impersonateLoggedOnUser(int    logonHandle);

    /*
     * Register and NT event source.
     * @param sourceName Event Source name.
     * @return int
     **/
    public native int     registerEventSource(String sourceName);

    /*
     * Report an NT event. 
     * @param eventHandle An integer value.
     * @param eventType The type of the event.
     * @param eventID 
     * @param stringData 
     * @param bytes[] 
     * @return boolean value
     **/
    public native boolean reportEvent(int     eventHandle,
        int     eventType,
        int     eventID,
        Object  stringData,
        byte    bytes[]);

    /*
     * De-register an NT event source.
     * @param eventHandle The event source to be de-registered.
     * @return a boolean                                            
     */
    public native boolean deregisterEventSource(int eventHandle);

    /*
     * Loads the JavaNT system library.
     * @exception java.lang.Exception                                                                         
     **/
    public JavaNT() throws Exception {                                           /*                              */
        System.loadLibrary("JavaNT");
    }                                           /*                              */

    /*
     * Register an NT event source.
     * @param sourceName Name of the source to be registered.
     * @exception java.lang.Exception                               
     **/
    public JavaNT(String sourceName) throws Exception {                                           /*                              */
        System.loadLibrary("JavaNT");
        if (handle == 0)                          /* registered already ?         */
            handle = registerEventSource(sourceName);       /* register             */
    }                                           /*                              */

    /*
     * Report an NT event.
     * @param  eventType a Byte indicating the type of the event to be reported.
     * @param  eventID An integer value, the eventID
     * @param stringData 
     * @param bytes[] 
     * @return boolean value           
     **/
    public boolean record(byte    eventType,
        int     eventID,
        Object  stringData,
        byte    bytes[]) {                                           /*                              */
        if (handle != 0)                          /* register already ?           */
            return (reportEvent(handle, eventType, eventID, stringData, bytes));
        else                                        /* ... no                       */
            return (false);                          /* didn't work                  */
    }                                           /*                              */

    /*
     * De-register an NT event source.
     * @return void                         
     */
    public void close() {                                           /*                              */
        if (handle != 0)                          /* valid handle ?               */
            deregisterEventSource(handle);          /* ... yes, release it          */
        handle = 0;                                 /* clear handle                 */
    }                                           /*                              */

    /**
     * De-register an NT event source.
     * @return void
     **/
    protected void finalize() {                                           /*                              */
        close();                                    /* finish with event log        */
    }                                           /*                              */

}   /* end NT Native class                                                    */

