package examples.osgi.admin;


// Version 1.5 mqe_java/source/examples/osgi/admin/MQeAdmin.java, MQeOSGi, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/

import com.ibm.mqe.trace.*;

import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.communications.*;
import com.ibm.mqe.adapters.MQeDiskFieldsAdapter;
import com.ibm.mqe.adapters.MQeMemoryFieldsAdapter;
import com.ibm.mqe.administration.*;
import java.io.File;
import java.util.*;


/**
 * MQeAdmin class provides helper methods to create and configure 
 * an MQeQueueManager.
 * 
 * The QM name and Reg location to be used are passed in via
 * the constructor.
 * 
 * This class configures QueueManagers to use the following adapters:
 * MQeTcpipHttpAdapter is used for comms.
 * MQeMemoryFieldsAdapter is used for registry storage. 
 * 
 */

public class MQeAdmin {
	
    public static short[] version = { 2, 0, 0, 5 }; 
    private String adapterClass = "com.ibm.mqe.adapters.MQeTcpipHttpAdapter";
 
    private long channelTimeout = 3600000;
    private int maxNumberChannels = 10;
    
    private MQeFields parms;
    private String myQmName = "myQM";
    private String myReg = "c:\\myQM\\";
    private MQeQueueManager qm = null;
    private MQeQueueManagerConfigure qmConfig = null;
    private boolean stopping = false;
    private String queueStore;
    private String MEM_ADAPTER = "com.ibm.mqe.adapters.MQeMemoryFieldsAdapter";
    private String remoteQueueManagerName = "MQeServer";
    private String queueName = "SYSTEM.DEFAULT.LOCAL.QUEUE";
    private String receiverQMIPAddress = "127.0.0.1";
    private String receiverQMPort;

    /**
     * Private reference to a memory adapter, to make sure
     * that configuration information set up in one method is not
     * garbage collected before it is used in other methods.
     */
    private MQeMemoryFieldsAdapter myMemory = null;
    
    /**
     * Constructor to create a new MQeAdmin.
     * This sets the QueueManger name and Registry location
     * to be used by the MQeQueuemanager. 
     */	
	
    public MQeAdmin(String qmName, String regLocation) {
		
        myQmName = qmName;
        myReg = regLocation;
	
    }
	
    /**
     * Sets the address and port of the server.
     * 
     * When this MQeAdmin class is used by a MQeServerApp the port set
     * here is used for the listener.
     * 
     * When this MQeAdmin class is used by a MQeClientApp the port and 
     * address set here is used for the connection defintion pointing to the 
     * queue manager used by the MQeServerApp
     * By default the Address is set to 127.0.0.1
     * By default the Port is set to 8085
     */	
	
    public void setServerAddressAndPort(String targetQMIPAddress, String targetQMPort) {
        if (targetQMIPAddress != null) {
            this.receiverQMIPAddress = targetQMIPAddress;
        }
        if (targetQMPort != null) {
            this.receiverQMPort = targetQMPort;
        } else {
            this.receiverQMPort = "8085";
        }
	
    }
	
    /**
     * Defines the Queue Manager
     */	
	
    public void defineQM() {
		
        try {
            myMemory = new com.ibm.mqe.adapters.MQeMemoryFieldsAdapter();
            queueStore = MEM_ADAPTER + ":" + myReg + File.separator + "queues";	
			
            parms = new MQeFields();
            MQeFields queueManagerParameters = new MQeFields();
            MQeFields registryParameters = new MQeFields();

            // add qm name here
            queueManagerParameters.putAscii(MQeQueueManager.Name, myQmName);
            // add reg location  here
            registryParameters.putAscii(MQeRegistry.DirName, myReg);
            registryParameters.putAscii(MQeRegistry.Adapter, MEM_ADAPTER);
            parms.putFields(MQeQueueManager.QueueManager, queueManagerParameters);
            parms.putFields(MQeQueueManager.Registry, registryParameters);
		
            // use MQeConfig
            qmConfig = new MQeQueueManagerConfigure(parms, queueStore);
				
            // define default queues
            qmConfig.defineQueueManager();
            qmConfig.defineDefaultSystemQueue();
            qmConfig.defineDefaultDeadLetterQueue();
            qmConfig.defineDefaultAdminReplyQueue();
            qmConfig.defineDefaultAdminQueue();
            qmConfig.close();
			
        } catch (Exception e) {
            System.out.println("Error defining QM " + e);
            System.out.println("Reg probably already exists so ignore");
        }

    }
	
    /**
     * Stops the Queue Manager
     */	
    public void stopQM() {
        try {
            System.out.println("Stopping QM");			
            qm.closeQuiesce(3000);
        } catch (Exception e) {
				
            System.out.println("Error closing QM " + e);
        }
		
    }

    /**
     * Deletes the Queue Manager definition.
     */
    public void deleteQM() {
        try {
            // use MQeConfig
            qmConfig = new MQeQueueManagerConfigure(parms, MEM_ADAPTER + ":" + "qmName" + File.separator +
                        "Queues" + File.separator);				
            System.out.println("Deleting QM");
            qmConfig.deleteStandardQMDefinitions();
            qmConfig.close();		
            myMemory = null;
					
        } catch (Exception e) {
            System.out.println("Error Deleting QM Definition" + e);
        } 
		
    }

    /**
     * Starts the Queue Manager
     */	

    public void startQM() {
        try {
			
            qm = new MQeQueueManager();
            //qm.activate(myQmName, myReg);
            qm.activate(parms);
        } catch (Exception e) {
				
            System.out.println("Error starting QM " + e);
        }
		
    }
	
    /**
     * Creates a listener on the Queue Manager on port 8085 by default.
     */	

    public void createListener() {
        try {
            MQeCommunicationsListenerAdminMsg createMessage =
                new MQeCommunicationsListenerAdminMsg();

            // first of all we shall create a queue manager definition and start it
            // set the name of the resource we wish to create
            createMessage.setName("Listener 1");
            // provide the parameters to the method that will set the action of the
            // administration message as well as the correct parameter names using our
            // input
            createMessage.create(adapterClass, Integer.parseInt(receiverQMPort), channelTimeout, maxNumberChannels);
            // set the target queue manager
            createMessage.setTargetQMgr(myQmName);

            // indicate that we require a reply message
            createMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            createMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            createMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            createMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("connection administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(myQmName, MQe.Admin_Queue_Name, createMessage, null, 0);

            System.out.println("create administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("listener created");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "listener not created, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("listener connection failed with: " + response.getReason());
                break;

            }
            if (response.getRC() != MQeAdminMsg.RC_Success) {
                throw new MQeException(response.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Starts the Listener that should have already been created on the QM.
     */	

    public void startListener() {
        try {
            MQeCommunicationsListenerAdminMsg startMessage =
                new MQeCommunicationsListenerAdminMsg();

            // set the name of the resource we wish to create
            startMessage.setName("Listener 1");
            // provide the parameters to the method that will set the action of the
            // administration message as well as the correct parameter names using our
            // input
            startMessage.start();

            // set the target queue manager
            startMessage.setTargetQMgr(myQmName);

            // indicate that we require a reply message
            startMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            startMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            startMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            startMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("connection administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(myQmName, MQe.Admin_Queue_Name, startMessage, null, 0);

            System.out.println("create administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("listener started");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "listener not started, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("listener start failed with: " + response.getReason());
                break;

            }
            
        } catch (Exception e) {
            System.out.println("Error creating starting Listener. " + e);
            System.out.println("Check that another server is not running and that nothing else is using this port");
        }
    }

    /**
     * Stops the Queue Managers Listener
     */	

    public void stopListener() {
        try {
            MQeCommunicationsListenerAdminMsg startMessage =
                new MQeCommunicationsListenerAdminMsg();

            // set the name of the resource we wish to create
            startMessage.setName("Listener 1");
            // provide the parameters to the method that will set the action of the
            // administration message as well as the correct parameter names using our
            // input
            startMessage.stop();

            // set the target queue manager
            startMessage.setTargetQMgr(myQmName);

            // indicate that we require a reply message
            startMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            startMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            startMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            startMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("stop listener administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(myQmName, MQe.Admin_Queue_Name, startMessage, null, 0);

            System.out.println(
                "stop listener administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("listener stopped");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "listener not stopped, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("listener stop failed with: " + response.getReason());
                break;

            }
            if (response.getRC() != MQeAdminMsg.RC_Success) {
                throw new MQeException(response.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** 
     * method to delete the listener using an administration message
     */
    public void deleteListener() {
        try {
            MQeCommunicationsListenerAdminMsg deleteMessage =
                new MQeCommunicationsListenerAdminMsg();

            // set the name of the resource we wish to create
            deleteMessage.setName("Listener 1");
            // provide the parameters to the method that will set the action of the
            // administration message as well as the correct parameter names using our
            // input
            deleteMessage.setAction(MQeAdminMsg.Action_Delete);

            // set the target queue manager
            deleteMessage.setTargetQMgr(myQmName);

            // indicate that we require a reply message
            deleteMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            deleteMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            deleteMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            deleteMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("delete listener administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(myQmName, MQe.Admin_Queue_Name, deleteMessage, null, 0);

            System.out.println(
                "delete listener administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("listener deleted");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "listener not deleted, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("listener delete failed with: " + response.getReason());
                break;

            }
            if (response.getRC() != MQeAdminMsg.RC_Success) {
                throw new MQeException(response.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
	
    /**
     * Adds a remote queue definition. This is used by the client to setup
     * a remote queue definition to the servers local queue.
     */	
	
    public void addRemoteQueueDefinition() {
        try {
			
            MQeRemoteQueueAdminMsg remQueueMsg = new MQeRemoteQueueAdminMsg(remoteQueueManagerName, queueName);
            MQeFields parms = new MQeFields();

            parms.putUnicode(MQeQueueAdminMsg.Queue_Description, "This is a def for MQeServers local queue");
            // set the queue access mode
						
            parms.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Asynchronous);
            parms.putAscii(MQeQueueAdminMsg.Queue_FileDesc, queueStore);
            remQueueMsg.create(parms);
            qm.putMessage(myQmName, MQe.Admin_Queue_Name, remQueueMsg, null, 0);
        } catch (Exception e) {}
    }
	
    /**
     * method to show how a connection definition may be created using an administration
     * message. This is used by the client to setup a connection definition to the
     * server so messages can be sent.
     */
    public void createConnection() {
        try {

            MQeConnectionAdminMsg connectionMessage = new MQeConnectionAdminMsg();

            // Set the name of the resource we wish to create. The connection definition name
            // is always the same as the name of the remote queue manager to which its 
            // information will allow you to go
            connectionMessage.setName(remoteQueueManagerName);

            // Call the method that will automatically set the correct MQeFields for our connection.
            connectionMessage.create(
                "com.ibm.mqe.adapters.MQeTcpipHttpAdapter:" + receiverQMIPAddress + ":" + receiverQMPort,
                null,
                null,
                "DefaultChannel",
                "Example connection");

            // set the target queue manager
            connectionMessage.setTargetQMgr(myQmName);

            // indicate that we require a reply message
            connectionMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            connectionMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            connectionMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            connectionMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("connection administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(
                myQmName,
                MQe.Admin_Queue_Name,
                connectionMessage,
                null,
                0);

            System.out.println("create administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("connection created");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "connection not created, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("create connection failed with: " + response.getReason());
                break;

            }

        } catch (Exception e) {
            System.out.println("Error creating connection " + e);
           
        }

    }

    /**
     * method to show how a connection definition may be deleted using an administration
     * message
     */
    private void deleteConnection() {
        try {
            MQeConnectionAdminMsg deleteMessage = new MQeConnectionAdminMsg();

            // set the target queue manager
            deleteMessage.setTargetQMgr(myQmName);
        
            deleteMessage.setName(remoteQueueManagerName);

            deleteMessage.setAction(MQeAdminMsg.Action_Delete);
           
            // indicate that we require a reply message
            deleteMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

            // use default reply-to queue on the target queue manager.
            deleteMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
            deleteMessage.putAscii(MQe.Msg_ReplyToQMgr, myQmName);

            // create a unique tag that we can identify the reply with
            String match = "Msg" + System.currentTimeMillis();

            deleteMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            System.out.println("delete connection administration message created");

            // now put the administration message to the default administration queue
            qm.putMessage(
                myQmName,
                MQe.Admin_Queue_Name,
                deleteMessage,
                null,
                0);

            System.out.println("delete administration message put to " + MQe.Admin_Queue_Name);

            // create the filter so we get the match
            MQeFields filter = new MQeFields();

            filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

            // now wait for a reply
            MQeAdminMsg response =
                (MQeAdminMsg) qm.waitForMessage(
                    myQmName,
                    MQe.Admin_Reply_Queue_Name,
                    filter,
                    null,
                    0,
                    3000);

            // the administration message has a method that will get out the return code 
            switch (response.getRC()) {
            case MQeAdminMsg.RC_Success:
                System.out.println("connection deleted");
                break;

            case MQeAdminMsg.RC_Fail:
                System.out.println(
                    "connection not deleted, failed with: " + response.getReason());
                break;

            case MQeAdminMsg.RC_Mixed:
                System.out.println("Partial failure with: " + response.getReason());
                MQeFields errors = response.getErrorFields();
                Enumeration fields = errors.fields();

                while (fields.hasMoreElements()) {
                    String value[];
                    String name = (String) fields.nextElement();

                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else
                        value = new String[] { errors.getAscii(name)};
                    for (int j = 0; j < value.length; j++)
                        System.out.println("Field " + name + " failed with " + value[j]);
                }
                System.out.println("delete connection failed with: " + response.getReason());
                break;

            }
            if (response.getRC() != MQeAdminMsg.RC_Success) {
                throw new MQeException(response.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
	
}
