package examples.osgi.server;


// Version 1.3 mqe_java/source/examples/osgi/server/MQeServerBundleActivator.java, MQeOSGi, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
 
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import java.lang.InterruptedException;


/***
 *  Example Bundle Activator. Shows how to start an MQe 
 *  Application within the OSGI framework
 *
 *  Example shows how to:
 * <pre>
 *  . start and configure the MQeServerApp
 *  . stop the MQeServerApp
 * </pre>
 *
 **/


public class MQeServerBundleActivator implements BundleActivator {

    public static short[] version = { 2, 0, 0, 5 }; 
    MQeServerApp dem;
    private boolean cleanStart = true;

    /**
     * Called by the framework when this bundle is started.
     * 
     * It takes the value stored in the java property examples.osgi.server.port
     * and passes this to the application. Under the covers this is the port 
     * number that the MQe listener will listen on. If this property is not set
     * the default port is 8085.
     * 
     *  
     * @param context org.osgi.framework.BundleContext, BundleContext.
     */
    public void start(BundleContext context) throws Exception {

        //we only need port for the MQeListener		
        String targetQMPort = System.getProperty("examples.osgi.server.port");
				
        System.out.println("MQeServerApp starting in 2 seconds...");
        Thread.sleep(2000);	
        dem = new MQeServerApp("MQeServer", "c:\\MQeServer");
        dem.setServerAddressAndPort(null, targetQMPort);
		
        try {
            dem.setupMQeServer();
            dem.runTest();
			
            System.out.println("MQeServerApp has been started OK");	
        } catch (Exception e) {
            cleanStart = false;	
            System.out.println(e.toString());
            System.out.println("MQeServerApp not started cleanly.");	
        }
		
    }

    /**
     * Called by the framework when this bundle is stopped.
     * 
     * It shutsdown the underlying MQeQueueManager.
     * 
     * @param context org.osgi.framework.BundleContext, BundleContext.
     */
    public void stop(BundleContext context) throws Exception {
        if (cleanStart) {
            dem.shutdownMQeServer();		
        }
        System.out.println("MQeServerApp stopped.");				
		
    }

}
