package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeChannelTimer.java, MQeExamples, la200, la200-L030324.1 1.11
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeChannelManager;
import com.ibm.mqe.MQeTrace;


/**
 * <p>A simple timer used in the process of timing out idle channels.
 * <p>This class extends {@link com.ibm.mqe.MQe} class.
 */
public class MQeChannelTimer extends MQe implements Runnable {
    public static short version[] = { 2, 0, 0, 5 }; 

    // Slave thread
    protected Thread running = null;

    // Flag which is set when shutdown of the background thread is required.
    private volatile boolean toldToStop = false;

    // Channel timeout (millseconds)
    protected long timeout = 10000;

    // Channel manager
    protected MQeChannelManager channelManager;

    /**
     * <p>This is a Constructor.
     * 
     * @param channelManager  A reference to the Channel Manager object.
     * @param timeout         Interval between checks for idle channels (in milliseconds).
     * @exception java.lang.Exception
     */
    public MQeChannelTimer(MQeChannelManager thisChannelManager, long thisTimeout) throws Exception {
        
        super();
        
        channelManager = thisChannelManager;
        timeout = thisTimeout;
        running = new Thread(this);
        toldToStop = false;

        // start the thread.
        running.start();
    }

    /**
     * Run procedure.
     * 
     * @return void
     */
    public void run() {        
        try {
            running.setName("ExampleMQeChannelTimerThread");
            // When the timer pops inform the channel manager so that it can
            // tidy up idle channels.
            while ((running != null) && (timeout > 0)) {
                running.sleep(timeout); // wait time interval
                
                MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, null);

                // Inform the channel manager
                channelManager.timeOut(timeout);
            }
        } catch (Exception e) {}

        // End the thread
        stop();
    }

    /**
     * Stop procedure.
     * 
     * @return void
     */
    public void stop() {
        toldToStop = true;
        if (running != null) {
            Thread tmp = running;

            try { 
                tmp.join(timeout);
                running = null;
            } catch (InterruptedException e) {}
        } 
    }
}
