// Version mqe_java/source/examples/rules/AttributeRule.java, MQeExamples, la200, la200-L030324.1 1.10

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.rules;


import com.ibm.mqe.*;
import java.util.*;


/*
 *                      
 * <p>Example Attribute Rule class.
 * Extends MQeAttributeRule, the base MQe Attribute Rule class.                                           
 *                                                                              
 **/
public class AttributeRule extends MQeAttributeRule { /* start class                               */
    public static short version[] = { 2, 0, 0, 5 }; 

    protected String cryptors[] =
        new String[] {
            "com.ibm.mqe.attributes.MQeXorCryptor",
            "com.ibm.mqe.attributes.MQeDESCryptor",
            "com.ibm.mqe.attributes.MQe3DESCryptor",
            "com.ibm.mqe.attributes.MQeRC4Cryptor",
            "com.ibm.mqe.attributes.MQeRC6Cryptor",
            "com.ibm.mqe.attributes.MQeMARSCryptor" };

    /*
     * Constructor. 
     **/
    public AttributeRule() { /*                              */
        super(); /* ancestor                     */
    } /*                              */

    /*
     * Validate new rules.
     * @param owner Object
     * @param thisRule An MQeAttributeRule
     * @return a new rule object.
     **/
    public MQeAttributeRule newRule(Object owner, MQeAttributeRule thisRule)
        throws Exception { /*                              */
        if (thisRule == null) /* Do not allow "no rule"       */
            return (this);
        return (super.newRule(owner, thisRule)); /* check with ancestor          */
    } /*                              */

    /*
     * compare objects for equal or better.
     * @param parameter An MQeAttribute object
     * @return a boolean
     **/
    public boolean isAcceptable(MQeAttribute parameter) { /*                              */
        int x = mask(parameter); /* get the mask value           */

        /* compare authenticators                                                   */
        switch (x & 0x44) /* just authenticators          */{

        /*                              */
        case 0x00:
            break; /* equal  - none                */

        case 0x04:
            break; /* better - Parameter           */

        case 0x40:
            return (false); /* worse  - Owner               */

        case 0x44: /* equal? - both                */{ /*                              */

                /* compare authenticators for equality                                  */
                MQeAttribute Q = (MQeAttribute) owner; /* retype owner             */
                MQeAttribute C = (MQeAttribute) parameter; /* retype Parameter         */

                if (!Q.getAuthenticator().type().equals(C.getAuthenticator().type()))
                    return (false); /* mis-match                    */
                break; /* done                         */
            } /*                              */
        } /*                              */

        /* compare cryptors                                                         */
        switch (x & 0x22) /* just cryptors                */{

        /*                              */
        case 0x00:
            break; /* equal  - none                */

        case 0x02:
            break; /* better - Parameter           */

        case 0x20:
            return (false); /* worse  - Owner               */

        case 0x22: /* equal? - both                */{ /*                              */
                String OwnerSecLevel = ((MQeAttribute) owner).getCryptor().type();

                String NewSecLevel = ((MQeAttribute) parameter).getCryptor().type();

                /* find owner seclevel                                                  */
                int i; /* work variable                */

                for (i = 0; i < cryptors.length; i = i + 1) /* find owner           */
                    if (OwnerSecLevel.equals(abbreviate(cryptors[i], 1))) /* is it this one ?     */
                        break; /* ... yes                      */
                if (i > 1)
                    i = 2; /* set correct scan start point */

                /* is new seclevel equal to or beter than owner ?                       */
                for (i = i; i < cryptors.length; i = i + 1) /* find new             */
                    if (NewSecLevel.equals(abbreviate(cryptors[i], 1))) /* is it this one       */
                        break; /* ... yes                      */
                if (i >= cryptors.length) /* past end of table ?   >=     */
                    return (false); /* ... yes, not equal or better */
                break; /* done                         */
            } /*                              */
        } /*                              */

        /* compare compresors                                                       */
        switch (x & 0x11) /* just compressors             */{

        /*                              */
        case 0x00:
            break; /* equal  - none                */

        case 0x01:
            break; /* better - Parameter           */

        case 0x10:
            break; /* worse  - Owner               */

        case 0x11:
            break; /* equal  - both                */
        } /*                              */

        return (true); /*  return state                */
    } /*                              */

    /*
     * Create the mask.
     * @param two An Object
     * @return the mask.                                                           
     **/
    public int mask(Object two) { /*                              */
        int Result = 0; /* state mask                   */

        /* Owner - setup of mask                                                    */
        if (owner instanceof MQeAttribute) /* is owner an attribute ?      */{

            /* ... yes                      */
            MQeAttribute A = (MQeAttribute) owner; /* retype object                */

            if (!(A.getAuthenticator() == null))
                Result = Result + 0x40;
            if (!(A.getCryptor() == null))
                Result = Result + 0x20;
            if (!(A.getCompressor() == null))
                Result = Result + 0x10;
        } /*                              */

        /* parameter - setup of mask                                                */
        if (two instanceof MQeAttribute) /* is paramter an attribute ?   */{

            /* ... yes, not attribute       */
            MQeAttribute B = (MQeAttribute) two; /* retype object                */

            if (!(B.getAuthenticator() == null))
                Result = Result + 0x04;
            if (!(B.getCryptor() == null))
                Result = Result + 0x02;
            if (!(B.getCompressor() == null))
                Result = Result + 0x01;
        } /*                              */

        return (Result); /* return mask value            */
    } /*                              */

    /**
     * process a Permit request.
     * @param owner 
     * @param from
     * @param to
     * @return Object
     * @exception java.lang.Exception         
     **/
    public Object permit(Object owner, Object from, Object to)
        throws Exception { /*                              */
        if ((owner instanceof MQeAttribute)
            && (to instanceof MQeAttribute)) { /* ... no, not attribute        */
            int x = mask(to); /* state mask                   */

            /* check Authenticator, Cryptor and compressor states                     */
            if ((x & 0x44) == 0x44) /* replace authenticator ?      */
                replaceAuthenticator(to); /* ... yes, is it Ok ?          */
            if ((x & 0x22) == 0x22) /* replace cryptor ?            */
                replaceCryptor(to); /* ... yes, is it Ok ?          */
            if ((x & 0x11) == 0x11) /* replace compressor ?         */
                replaceCompressor(to); /* ... yes, is it Ok ?          */

            /* don't allow removal of Authenticator or cryptor                        */
            if (!(((x & 0x44) == 0x40) || ((x & 0x22) == 0x20))) /* is it ?      */
                return (to); /* return new object            */
        } /*                              */
        throw new MQeException(Except_Rule, "Disallowed by rule");
    } /*                              */

    /*
     * Verify Authenticator replacement.  
     * @param to
     * @return void
     * @exception java.lang.Exception 
     **/
    protected void replaceAuthenticator(Object to)
        throws Exception { /*                              */
        if (((MQeAttribute) owner).getAuthenticator().authenticatedID() != null)
            throw new MQeException(Except_Rule, "Disallowed by rule");
    } /*                              */

    /*
     * Verify Cryptor replacement.
     * @param to 
     * @return void
     * @exception java.lang.Exception 
     **/
    protected void replaceCryptor(Object to)
        throws Exception { /*                              */
        String FromSecLevel = ((MQeAttribute) owner).getCryptor().type();
        String ToSecLevel = ((MQeAttribute) to).getCryptor().type();

        /* moving to XOR                                                            */
        if (ToSecLevel.equals(abbreviate(cryptors[0], 1)))
            for (int i = 1; i < cryptors.length; i = i + 1)
                if (FromSecLevel.equals(abbreviate(cryptors[i], 1)))
                    throw new MQeException(Except_Rule, "Disallowed by rule");

        /* moving to DES                                                            */
        if (ToSecLevel.equals(abbreviate(cryptors[1], 1)))
            for (int i = 2; i < cryptors.length; i = i + 1)
                if (FromSecLevel.equals(abbreviate(cryptors[i], 1)))
                    throw new MQeException(Except_Rule, "Disallowed by rule");
    } /*                              */

    /*
     * Verify Compressor replacement.
     * @param to 
     * @return void
     * @exception java.lang.Exception     
     **/
    protected void replaceCompressor(Object To)
        throws Exception { /*                              */} /*                              */

} /* end class                                                             */
