// Version mqe_java/source/examples/rules/QueueLoggerRule.java, MQeExamples, la200, la200-L030324.1 1.4

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/* -----------------------------------------------------------------------------*/
package examples.rules;


import examples.eventlog.*;
import com.ibm.mqe.*;


/*
 *                       
 * <p>Example queue rules class.                                               
 * <p>Logs activation and closure of queues.                          
 * See the Application Programming Guide for more information. 
 * <p>Extends MQeQueueRule, the base MQe Queue Rule class. 
 *                                          
 *                                                                            
 **/

public class QueueLoggerRule extends com.ibm.mqe.MQeQueueRule {
    public static short version[] = { 2, 0, 0, 5 }; 

    /**
     * The file name that the rule will use
     */
    private static final String FILENAME = "log.txt";

    /**
     * The log instance that will record information
     */
    private LogToDiskFile logFile = null;
    
    /**
     * This method is called whenever a queue is activated.
     * It puts the name of the queue to a predetermined file.
     * @return void
     */
    public void queueActivate() {
        try {

            // Open log file
            logFile = new LogToDiskFile(FILENAME);

            // Log the activation
            log(MQe_Log_Information, Event_Activate, "Queue " +
                ((MQeQueue) owner).getQueueManagerName() + "+" +
                ((MQeQueue) owner).getQueueName() + " active");

        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    /**
     * This method is called whenever a queue is closed.
     * It puts the name of the queue to a predetermined file.
     * @return void
     */
    public void queueClose() {
        try {

            // Log the closure
            log(MQe_Log_Information, Event_Close, "Queue " +
                ((MQeQueue) owner).getQueueManagerName() + "+" +
                ((MQeQueue) owner).getQueueName() + " closed");
            
            // Close log file
            logFile.close();

        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }
    
}
