// Version mqe_java/source/examples/security/localSecure.java, MQeSecurity, la200, la200-L030324.1 1.2

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

package examples.security;


import com.ibm.mqe.*;
import com.ibm.mqe.attributes.*;


/** localSecurity_1.java
 * <p>Use the MQeLocalSecure class to write some data securely to a file,
 * then read it back
 **/

public class localSecure {
    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // the name of the file to write the data to
    String fileName = "TestSecureData.txt";
    // the cryptor to use
    MQeCryptor cryptor = new MQeDESCryptor();
    // the key seed for the cryptor
    String     key = "my secret key";

    /**
     * encrypt some data and write it to a file using the
     * MQeLocalSecure class
     *
     * @param      testString           the String to be encrypted
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    void encrypt(String testString) throws Exception {
        // create an attribute using the cryptor
        MQeAttribute attr = new MQeAttribute(null, cryptor, null);

        // create the MQeLocalSecure class
        MQeLocalSecure ls = new MQeLocalSecure();

        // open LocalSecure obj identifying the target directory and file
        ls.open(".", fileName);

        // convert the String data to a byte array
        byte[] data = MQe.asciiToByte(testString);

        // use LocalSecure to protect the data and write it to the file
        ls.write(data, attr, key);

    }

    /**
     * read the data back from the file and decrypt it using the
     * MQeLocalSecure class
     *
     * @return     the descrypted String
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    String decrypt() throws Exception {
        // create an attribute using the cryptor
        MQeAttribute attr = new MQeAttribute(null, cryptor, null);

        // create the MQeLocalSecure class
        MQeLocalSecure ls = new MQeLocalSecure();

        // open LocalSecure obj identifying the target directory and file
        ls.open(".", fileName);

        // use LocalSecure to read and decrypt the data from the file
        byte[] data = ls.read(attr, key);

        // convert the data back to a String
        String result = MQe.byteToAscii(data);

        // return the String
        return result;
    }

    /**
     * main method
     **/
    public static void main(String[] args) {
        // the test data to protect
        String testString = "0123456789abcdef....";

        try {
            // create an instance of this object
            localSecure localSec1 = new localSecure();

            // encrypt and write the data
            localSec1.encrypt(testString);

            // read and decrypt the data
            String result = localSec1.decrypt();

            System.out.println("read data: " + result);

            // check that the data was read back correctly
            if (testString.equals(result)) {
                System.out.println("good, same as data written to file");
            } else {
                System.out.println("data read from file differs from data written to it");
            }
        } catch (Exception e) {
            System.err.println("Exception caught: " + e);
            e.printStackTrace();
        }
    }
}

