// Version mqe_java/source/examples/security/localSecurity.java, MQeSecurity, la200, la200-L030324.1 1.2

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

package examples.security;


import com.ibm.mqe.*;
import com.ibm.mqe.attributes.*;


/** localSecurity_2.java
 * <p>Use MQe local security to encrypt some data, then decrypt it and compare
 * it with the original
 **/

public class localSecurity {
    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // the cryptor to use
    MQeCryptor cryptor = new MQeDESCryptor();
    // the key seed for the cryptor
    String     key = "my secret key";
    // the field name within the MQeFields object
    String     fieldName = "stringData";

    /**
     * encrpyt some data using MQe local security
     *
     * @param      testString           the String to be encrypted
     * @return     a byte array holding the encrypted data
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    byte[] encrypt(String testString) throws Exception {
        // create an attribute using the cryptor
        MQeAttribute attr = new MQeAttribute(null, cryptor, null);

        // create a Key object
        MQeKey localkey = new MQeKey();

        // set the passphrase in the Key object
        localkey.setLocalKey(key);

        // attach the key to the attribute
        attr.setKey(localkey);

        // instantiate an MQeFields object to hold the data
        MQeFields fields = new MQeFields();

        // add the data to the fields object
        fields.putAscii(fieldName, testString);

        // attach the attribute to the fields object
        fields.setAttribute(attr);

        // encrypt the data
        byte[] protectedData = fields.dump();

        // return the encrypted data
        return protectedData;
    }

    /**
     * decrypt the data using MQe local security
     *
     * @param      protectedData  the encrypted data to be decrypted
     * @return     a String containing the decrypted data
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    String decrypt(byte[] protectedData) throws Exception {
        // create an attribute using the cryptor
        MQeAttribute attr = new MQeAttribute(null, cryptor, null);

        // create a Key object
        MQeKey localkey = new MQeKey();

        // set the passphrase in the Key object
        localkey.setLocalKey(key);

        // attach the key to the attribute
        attr.setKey(localkey);

        // instantiate a MQeFields object to hold the decrypted data
        MQeFields fields = new MQeFields();

        // attach the attribute to the fields object
        fields.setAttribute(attr);

        // decrypt the data
        fields.restore(protectedData);

        // return the decrypted String
        return fields.getAscii(fieldName);
    }

    /**
     * main method
     **/
    public static void main(String[] args) {
        // the test data to protect
        String testString = "0123456789abcdef....";

        try {
            // create an instance of this object
            localSecurity localSec2 = new localSecurity();

            // encrypt the data
            byte[] protectedData = localSec2.encrypt(testString);

            // decrypt the data
            String result = localSec2.decrypt(protectedData);

            System.out.println("decrypted data: " + result);

            // check that the data was decrypted correctly
            if (testString.equals(result)) {
                System.out.println("good, same as data encrypted");
            } else {
                System.out.println("decrypted data differs from data encrypted");
            }
        } catch (Exception e) {
            System.err.println("Exception caught: " + e);
            e.printStackTrace();
        }
    }

}

