// 1.1 mqe_java/source/examples/stockplugin/gui/MainFrame.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.stockplugin.gui;


import java.awt.*;
import java.awt.event.*;

import java.util.*;

import examples.stockplugin.model.*;


/**
 * This class provides the main GUI frame for StockServer example.  This is the backend server to the Native
 * StockBroker Example.
 * 
 * This class provides the GUI interface, and starts the program logic.  The logic itself does not
 * provide much logic.  Just enough to respond to messages and send responces.  It doesn't do
 * any simulation - as it lets you by and sell any shares whatsoever.
 * 
 * This class can be run either as an Explorer Plugin (ES01) Support Pack, or it can be run from 
 * {@link examples.stockplugin.server.StocServerRuntime}.
 * 
 * Please the Explorer documentation for exact details on how to run programs.  However this is the class
 * that is required. 
 ** 
 * <p>
 * 
 * The GUI is split into two.  On the left are messages about the BrokerAdmin messages,
 * and on the right are messages about the trades.
 * 
 * Two threads handling both types of messages - these can be started and stopped using the buttons
 * at the top.  Both these threads will performed "waitForMessages" for a long time.
 */
public class MainFrame extends Frame {

    public static short[] version = { 2, 0, 0, 5 }; 

    private Label testLabel;
    private Button processAdminQ, processTradeQ;

    private Button stopAdminProcessing, stopTradeProcessing;

    private Panel buttonPanel, topPanel;
    ScrollPane statusScrollPane;
    private ScrollPane statusAdminPane;
    private TextArea statusAdminArea;

    private ScrollPane statusTradePane;
    private TextArea statusTradeArea;

    private Model model = null;

    InfoLog adminLog, tradeLog;

    /** InfoLog
     */
    class InfoLogImpl implements InfoLog {

        private TextArea areaRef;

        public InfoLogImpl(TextArea area) {
            areaRef = area;
        }

        public void addStatus(String info) {
            areaRef.append(info);
        }

    }

    /**
     * Constructor for MainFrame
     */
    public MainFrame() {
        super();

        // Set the layout manager
        setLayout(new BorderLayout(10, 10));

        // Create the panels
        buttonPanel = new Panel();
        topPanel = new Panel();

        statusScrollPane = new ScrollPane();

        statusAdminPane = new ScrollPane();
        statusTradePane = new ScrollPane();

        // Create the Layout
        testLabel = new Label("StockMarket Server", Label.CENTER);

        // Create the buttons
        processAdminQ = new Button("Process Admin");
        processTradeQ = new Button("Process Trades");
        stopAdminProcessing = new Button("Stop Admin");
        stopTradeProcessing = new Button("Stop Trade");

        buttonPanel.add(processAdminQ);
        buttonPanel.add(stopAdminProcessing);
        buttonPanel.add(stopTradeProcessing);
        buttonPanel.add(processTradeQ);

        statusAdminArea = new TextArea();
        statusTradeArea = new TextArea();
        //statusAdminPane.add(statusAdminArea);
        //statusTradePane.add(statusTradeArea);

        adminLog = new InfoLogImpl(statusAdminArea);
        tradeLog = new InfoLogImpl(statusTradeArea);

        Panel tempPanel = new Panel();

        tempPanel.add(statusAdminArea);
        tempPanel.add(statusTradeArea);

        statusScrollPane.add(tempPanel);
        tempPanel.setLayout(new GridLayout(0, 2));

        //statusPanel.add(statusAdminPane);
        //statusPanel.add(statusTradePane);
        adminLog.addStatus(">>> Information about Admin will go here\n ");
        tradeLog.addStatus(">>> Information about Trading will go here\n ");

        // Add the layouts
        topPanel.setLayout(new BorderLayout(10, 10));
        topPanel.add(testLabel, BorderLayout.NORTH);
        topPanel.add(buttonPanel, BorderLayout.SOUTH);

        add(topPanel, BorderLayout.NORTH);
        add(statusScrollPane, BorderLayout.CENTER);

        _addActionListeners();

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    dispose();
                };

            }
        );

        pack();

        setSize(800, 600);

        // create the model
        model = new Model(adminLog, tradeLog);

    }

    private void _addActionListeners() {
        processAdminQ.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    //model.processAdminRequests();
                    model.goAdminThread();
                }

            }
        );

        processTradeQ.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    //model.processTradeRequests();
                    model.goTradeThread();
                }

            }
        );

        stopTradeProcessing.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    //model.processTradeRequests();
                    model.stopTradeThread();
                }

            }
        );
        stopAdminProcessing.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    //model.processTradeRequests();
                    model.stopAdminThread();
                }

            }
        );
    }

    /**
     * Constructor for MainFrame
     */
    public MainFrame(GraphicsConfiguration arg0) {
        super(arg0);
    }

    /**
     * Constructor for MainFrame
     */
    public MainFrame(String arg0) {
        super(arg0);
    }

    /**
     * Constructor for MainFrame
     */
    public MainFrame(String arg0, GraphicsConfiguration arg1) {
        super(arg0, arg1);
    }

    public static void main(String[] args) {
        MainFrame mainFrame = new MainFrame();

        mainFrame.show();
    }

}
