// 1.1 mqe_java/source/examples/stockplugin/server/StockServerCreate.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.stockplugin.server;


import java.util.Enumeration;

import com.ibm.mqe.*;
import com.ibm.mqe.administration.MQeCommunicationsListenerAdminMsg;
import com.ibm.mqe.administration.MQeQueueAdminMsg;
import com.ibm.mqe.administration.MQeStoreAndForwardQueueAdminMsg;
import com.ibm.mqe.registry.*;


/**
 *
 * Running this program will create the Registry for the StockServer.  This is a very
 * simple server. It has the following features
 * 
 *  - Uses the TcpipHttp Adapter to listen on port 8082
 *  - Creates 2 local queues, plus one StoreAndForward Queue
 *  - Location of the registry and queue stores are rooted at e:\TestServer
 *  - QueueManager name is ServerQM
 * 
 * All the code for the various methods are taken from other examples.
 * 
 * This program does the following:
 * 
 *  - Creates a new QueueManager Regsitry
 *  - Creates and starts a QueueManager based on this registry
 *  - Creates 2 local queues
 *  - Creates a Store and Forward Queue
 *  - Creates a new Listener (HTTP on port 8082)
 * 
 */
public class StockServerCreate {

    public static short[] version = { 2, 0, 0, 5 }; 

    static MQeQueueManager myQueueManager = null;
    static String baseDirectoryName = "c:\\_WMQe\\ServerQM";
    static String queueManagerName = "ServerQM";

    /** This method creates a new QueueManager Registry using QueueManagerConfigure
     *  
     * This is taken from examples.config.CreateQueueManager
     */
    private static void _createQueueManager() throws Exception {

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(
            MQeRegistry.Adapter,
            "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(
            MQeRegistry.DirName,
            baseDirectoryName + "/Registry");
        config.putFields("Registry", registrySection);

        // Construct a queue manager configuration utility object.
        MQeQueueManagerConfigure configurator =
            new MQeQueueManagerConfigure(
                config,
                "com.ibm.mqe.adapters.MQeDiskFieldsAdapter"
                + ":"
                + baseDirectoryName
                + "/Queues");

        // Define a queue manager.
        configurator.defineQueueManager();

        // Define some queues on the queue manager.
        configurator.defineDefaultAdminQueue();
        configurator.defineDefaultAdminReplyQueue();
        configurator.defineDefaultDeadLetterQueue();
        configurator.defineDefaultSystemQueue();

        // Close the queue manager configuration utility class.
        configurator.close();

    }

    /**
     * This creates and starts the QueueManager from a pre-existing registry.
     * 
     */
    private static void _startQueueManager() throws Exception {
        System.out.println("Starting the queue manager.");

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(
            MQeRegistry.Adapter,
            "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(
            MQeRegistry.DirName,
            baseDirectoryName + "/Registry");

        config.putFields("Registry", registrySection);

        System.out.println("Starting the queue manager");
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config);
        System.out.println("Queue manager started.");

    }

    /** This will create a listener - this turns this QueueManager into a server role.
     * 
     * examples.config.ConfigListener
     */
    private static void _createListener() throws Exception {

        MQeCommunicationsListenerAdminMsg createMessage =
            new MQeCommunicationsListenerAdminMsg();

        // set the name of the resource we wish to create
        createMessage.setName("Listener 1");
        // provide the parameters to the method that will set the action of the
        // administration message as well as the correct parameter names using our
        // input
        createMessage.create(
            "com.ibm.mqe.adapters.MQeTcpipHttpAdapter",
            8082,
            3600000,
            10);
        // set the target queue manager
        createMessage.setTargetQMgr(queueManagerName);

        // indicate that we require a reply message
        createMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

        // use default reply-to queue on the target queue manager.
        createMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        createMessage.putAscii(MQe.Msg_ReplyToQMgr, queueManagerName);

        // create a unique tag that we can identify the reply with
        String match = "Msg" + System.currentTimeMillis();

        createMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        System.out.println("connection administration message created");

        // now put the administration message to the default administration queue
        myQueueManager.putMessage(
            queueManagerName,
            MQe.Admin_Queue_Name,
            createMessage,
            null,
            0);

        System.out.println(
            "create administration message put to " + MQe.Admin_Queue_Name);

        // create the filter so we get the match
        MQeFields filter = new MQeFields();

        filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        // now wait for a reply
        MQeAdminMsg response =
            (MQeAdminMsg) myQueueManager.waitForMessage(
                queueManagerName,
                MQe.Admin_Reply_Queue_Name,
                filter,
                null,
                0,
                3000);

        // the administration message has a method that will get out the return code 
        switch (response.getRC()) {
        case MQeAdminMsg.RC_Success:
            System.out.println("listener created");
            break;

        case MQeAdminMsg.RC_Fail:
            System.out.println(
                "listener not created, failed with: "
                + response.getReason());
            break;

        case MQeAdminMsg.RC_Mixed:
            System.out.println(
                "Partial failure with: " + response.getReason());
            MQeFields errors = response.getErrorFields();
            Enumeration fields = errors.fields();

            while (fields.hasMoreElements()) {
                String value[];
                String name = (String) fields.nextElement();

                if (errors.dataType(name) == MQeField.TypeArrayElements)
                    value = errors.getAsciiArray(name);
                else
                    value = new String[] { errors.getAscii(name)};
                for (int j = 0; j < value.length; j++)
                    System.out.println(
                        "Field " + name + " failed with " + value[j]);
            }
            System.out.println(
                "listener connection failed with: " + response.getReason());
            break;

        }
        if (response.getRC() != MQeAdminMsg.RC_Success) {
            throw new MQeException(response.getReason());
        }

    }

    /** This will create LocalQueues, the name is passed as a paramter.
     * 
     * Two local queues need to be created BrokerTradeQ and BrokerAdminQ
     * 
     */
    public static void _createLocalQueues(String queueName) throws Exception {

        MQeQueueAdminMsg createMessage = new MQeQueueAdminMsg();
        MQeFields params = new MQeFields();

        // set the name of the resource we wish to create
        createMessage.setName(queueManagerName, queueName);

        // provide the parameters to the method that will set the action of the
        // administration message as well as the correct parameter names using our
        // input
        createMessage.create(params);

        // set the target queue manager
        createMessage.setTargetQMgr(queueManagerName);

        // indicate that we require a reply message
        createMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

        // use default reply-to queue on the target queue manager.
        createMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        createMessage.putAscii(MQe.Msg_ReplyToQMgr, queueManagerName);

        // create a unique tag that we can identify the reply with
        String match = "Msg" + System.currentTimeMillis();

        createMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        System.out.println("connection administration message created");

        // now put the administration message to the default administration queue
        myQueueManager.putMessage(
            queueManagerName,
            MQe.Admin_Queue_Name,
            createMessage,
            null,
            0);

        System.out.println(
            "create administration message put to " + MQe.Admin_Queue_Name);

        // create the filter so we get the match
        MQeFields filter = new MQeFields();

        filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        // now wait for a reply
        MQeAdminMsg response =
            (MQeAdminMsg) myQueueManager.waitForMessage(
                queueManagerName,
                MQe.Admin_Reply_Queue_Name,
                filter,
                null,
                0,
                3000);

        // the administration message has a method that will get out the return code 
        switch (response.getRC()) {
        case MQeAdminMsg.RC_Success:
            System.out.println("queue created");
            break;

        case MQeAdminMsg.RC_Fail:
            System.out.println(
                "queue not created, failed with: " + response.getReason());
            break;

        case MQeAdminMsg.RC_Mixed:
            System.out.println(
                "Partial failure with: " + response.getReason());
            MQeFields errors = response.getErrorFields();
            Enumeration fields = errors.fields();

            while (fields.hasMoreElements()) {
                String value[];
                String name = (String) fields.nextElement();

                if (errors.dataType(name) == MQeField.TypeArrayElements)
                    value = errors.getAsciiArray(name);
                else
                    value = new String[] { errors.getAscii(name)};
                for (int j = 0; j < value.length; j++)
                    System.out.println(
                        "Field " + name + " failed with " + value[j]);
            }
            System.out.println(
                "queue connection failed with: " + response.getReason());
            break;

        }
        if (response.getRC() != MQeAdminMsg.RC_Success) {
            throw new MQeException(response.getReason());
        }

    }

    /** This creates a Store and Forward Queue
     * 
     * 
     */
    public static void _createSAFQueue() throws Exception {

        MQeStoreAndForwardQueueAdminMsg createMessage =
            new MQeStoreAndForwardQueueAdminMsg();
        MQeFields params = new MQeFields();

        // set the name of the resource we wish to create
        createMessage.setName(queueManagerName, "InformationQ");

        // provide the parameters to the method that will set the action of the
        // administration message as well as the correct parameter names using our
        // input
        createMessage.create(params);
        // set the target queue manager
        createMessage.setTargetQMgr(queueManagerName);

        // indicate that we require a reply message
        createMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

        // use default reply-to queue on the target queue manager.
        createMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        createMessage.putAscii(MQe.Msg_ReplyToQMgr, queueManagerName);

        // create a unique tag that we can identify the reply with
        String match = "Msg" + System.currentTimeMillis();

        createMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        System.out.println("connection administration message created");

        // now put the administration message to the default administration queue
        myQueueManager.putMessage(
            queueManagerName,
            MQe.Admin_Queue_Name,
            createMessage,
            null,
            0);

        System.out.println(
            "create administration message put to " + MQe.Admin_Queue_Name);

        // create the filter so we get the match
        MQeFields filter = new MQeFields();

        filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        // now wait for a reply
        MQeAdminMsg response =
            (MQeAdminMsg) myQueueManager.waitForMessage(
                queueManagerName,
                MQe.Admin_Reply_Queue_Name,
                filter,
                null,
                0,
                3000);

        // the administration message has a method that will get out the return code 
        switch (response.getRC()) {
        case MQeAdminMsg.RC_Success:
            System.out.println("SAFQ created");
            break;

        case MQeAdminMsg.RC_Fail:
            System.out.println(
                "SAFQ not created, failed with: " + response.getReason());
            break;

        case MQeAdminMsg.RC_Mixed:
            System.out.println(
                "Partial failure with: " + response.getReason());
            MQeFields errors = response.getErrorFields();
            Enumeration fields = errors.fields();

            while (fields.hasMoreElements()) {
                String value[];
                String name = (String) fields.nextElement();

                if (errors.dataType(name) == MQeField.TypeArrayElements)
                    value = errors.getAsciiArray(name);
                else
                    value = new String[] { errors.getAscii(name)};
                for (int j = 0; j < value.length; j++)
                    System.out.println(
                        "Field " + name + " failed with " + value[j]);
            }
            System.out.println(
                "SAFQ connection failed with: " + response.getReason());
            break;

        }
        if (response.getRC() != MQeAdminMsg.RC_Success) {
            throw new MQeException(response.getReason());
        }

    }

    /** The main function - creates all the componnets that will be required
     *  
     * 
     */
    public static void main(String[] args) {

        try {
            System.out.println("Creating QM...");
            _createQueueManager();

            System.out.println("Starting QM...");
            _startQueueManager();

            System.out.println("Creating LocalQ...");
            _createLocalQueues("BrokerAdminQ");

            System.out.println("Creating LocalQ...");
            _createLocalQueues("BrokerTradeQ");

            System.out.println("Creating Listener...");
            _createListener();

            System.out.println("Creating SAFQ...");
            _createSAFQueue();

            System.out.println("Stopping QM...");
            myQueueManager.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
