// 1.1 mqe_java/source/examples/stockplugin/server/StockServerRuntime.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.stockplugin.server;


import java.util.Enumeration;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeField;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.MQeTrace;
import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.administration.MQeCommunicationsListenerAdminMsg;
import com.ibm.mqe.registry.MQeRegistry;
import examples.stockplugin.gui.MainFrame;


/**
 * This class provides the runtime code for a Server functionality QueueManager
 * to act as the backend of the Native StockBroker Example.
 * 
 * This class provides the mechanism to start the QueueManager as a server from a regsitry
 * create by the {@link examples.stockplugin.server.StockServerCreate} class.
 * 
 * The program logic and gui are implemented by the examples.stockplugin.model and 
 * examples.stockplugin.gui respectively.
 * 
 * For information on how to use the actual server, please see the
 * {@examples.stockplugin.gui.MainFrame} class.
 * 
 * To run this, ensure that the WMQe Classes on your classpath, and run this class
 * 
 * java examples.stockplugin.StockServerRuntime.
 * 
 * Note that the location of the registry and queuestore are hard coded to be e:\TestServer
 * 
 */
public class StockServerRuntime {

    public static short[] version = { 2, 0, 0, 5 }; 

    static MQeQueueManager myQueueManager = null;
    static String baseDirectoryName = "c:\\_WMQe\\ServerQM";;
    static String queueManagerName = "ServerQM";

    private static void _startQueueManager() throws Exception {
        System.out.println("Creating the queue manager.");

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(
            MQeRegistry.Adapter,
            "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(
            MQeRegistry.DirName,
            baseDirectoryName + "/Registry");

        config.putFields("Registry", registrySection);

        System.out.println("Starting the queue manager");
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config);
        System.out.println("Queue manager started.");

    }

    /** Note that is not acy
     public static void _startListeners() throws Exception {
     MQeCommunicationsListenerAdminMsg startMessage =
     new MQeCommunicationsListenerAdminMsg();

     // set the name of the resource we wish to create
     startMessage.setName("Listener 1");
     // provide the parameters to the method that will set the action of the
     // administration message as well as the correct parameter names using our
     // input
     startMessage.start();

     // set the target queue manager
     startMessage.setTargetQMgr(queueManagerName);

     // indicate that we require a reply message
     startMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

     // use default reply-to queue on the target queue manager.
     startMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
     startMessage.putAscii(MQe.Msg_ReplyToQMgr, queueManagerName);

     // create a unique tag that we can identify the reply with
     String match = "Msg" + System.currentTimeMillis();

     startMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

     System.out.println("connection administration message created");

     // now put the administration message to the default administration queue
     myQueueManager.putMessage(
     queueManagerName,
     MQe.Admin_Queue_Name,
     startMessage,
     null,
     0);

     System.out.println(
     "create administration message put to " + MQe.Admin_Queue_Name);

     // create the filter so we get the match
     MQeFields filter = new MQeFields();

     filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

     // now wait for a reply
     MQeAdminMsg response =
     (MQeAdminMsg) myQueueManager.waitForMessage(
     queueManagerName,
     MQe.Admin_Reply_Queue_Name,
     filter,
     null,
     0,
     3000);

     // the administration message has a method that will get out the return code 
     switch (response.getRC()) {
     case MQeAdminMsg.RC_Success :
     System.out.println("listener started");
     break;

     case MQeAdminMsg.RC_Fail :
     System.out.println(
     "listener not started, failed with: "
     + response.getReason());
     break;

     case MQeAdminMsg.RC_Mixed :
     System.out.println(
     "Partial failure with: " + response.getReason());
     MQeFields errors = response.getErrorFields();
     Enumeration fields = errors.fields();

     while (fields.hasMoreElements()) {
     String value[];
     String name = (String) fields.nextElement();

     if (errors.dataType(name) == MQeField.TypeArrayElements)
     value = errors.getAsciiArray(name);
     else
     value = new String[] { errors.getAscii(name)};
     for (int j = 0; j < value.length; j++)
     System.out.println(
     "Field " + name + " failed with " + value[j]);
     }
     System.out.println(
     "listener start failed with: " + response.getReason());
     break;

     }
     if (response.getRC() != MQeAdminMsg.RC_Success) {
     throw new MQeException(response.getReason());
     }
     }*/

    public static void main(String[] args) {

        try {
            // Step 1. Create & Start the QueueManager
            _startQueueManager();
			
            // Note that the listeners will be started automatically

            // Create the GUI, 
            MainFrame mainFrame = new MainFrame();

            mainFrame.show();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }
	
    // -----------------------------------------------------------------------------------------
    // Trace helpers
    static MQeTrace myMQeTrace;
	
    /**
     * Turns MQe trace on.
     */
    public static void traceOn() throws Exception {

        // Allocate a trace instance, so our handler isn't garbage collected when its' on.
        myMQeTrace = new MQeTrace();

        // Allocate a trace handler
        // This one puts trace output to stdout by default.
        MQeTraceHandler handler = (MQeTraceHandler) new com.ibm.mqe.trace.MQeTraceToBinaryFile();

        // Set this handler as the one MQe uses.
        MQeTrace.setHandler(handler);

        // Set the filter so we collect those pieces of trace we are interrested in.
        // In this case, it't the MQe API calls, errors, warnings, and exceptions.
        /*MQeTrace.setFilter(MQeTrace.GROUP_API | MQeTrace.GROUP_ERROR
         | MQeTrace.GROUP_WARNING | MQeTrace.GROUP_EXCEPTION
         );*/
    }
    
    /**
     * Turns MQe trace off.
     */
    public static void traceOff() throws Exception {
        // Stop collecting trace information.
        MQeTrace.setFilter(MQeTrace.GROUP_MASK_NONE);

        // Remove the trace handler.
        MQeTrace.setHandler(null);
    }
	
}
