package examples.trace;


// Version mqe_java/source/examples/trace/MQeTrace.java, MQeExamples, la200, la200-L030324.1 1.19
/* -----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

import java.text.DateFormat;
import java.util.Date;
import java.util.TimeZone;

import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.trace.MQeTraceRenderer;
import com.ibm.mqe.trace.MQeTraceToReadable;


/**
 * A dummy trace handler example. 
 * 
 * <p>All messages are logged to the basic trace handler provided by the basic
 * trace handler supplied with the WMQe toolkit. Namely the {@link com.ibm.mqe.trace.MQeTraceToReadable} class.
 * <p>This example doesn't really do any logic itself except straight pass-through to the product trace
 * handler. It is here mainly because it was here in previous releases of WMQe. If solutions rely on this
 * code being present, and this class being included in their .ini files or solutions, then it still exists.
 * <p>This trace handler could be used as a basis to provide a trace handler which does more complex filtering 
 * on the trace messages, perhaps excluding or including trace messages in certain ranges, or trace groups.
 * <p>Users of this class must activate the class before the trace handler is plugged-in to handle trace input.
 * If the object is not activated, then it will not pass any trace output onwards.
 * @see com.ibm.mqe.trace
 */
public class MQeTrace implements MQeTraceHandler {
    public static short version[] = { 2, 0, 0, 5 }; 

    protected String title = ""; // trace title
    protected long filter = 0xFFFFFFFFFFFFFFFFL;
    protected MQeTraceHandler baseHandler = null;
    
    private boolean activated = false;

    public MQeTrace() {
        super();
    }

    public MQeTrace(String title) {
        super();
        activate(title);
    }

    public void setFilter(long newFilter) {
        filter = newFilter;
    }

    /**
     * Common activation code
     */
    public void activate(String title) {
        try {
            this.title = title;

            DateFormat dtf = DateFormat.getDateTimeInstance();

            dtf.setTimeZone(TimeZone.getDefault());
            System.out.println(this.title + " " + dtf.format(new Date()) + " - System.out.println");
            System.err.println(this.title + " " + dtf.format(new Date()) + " - System.err.println");

            com.ibm.mqe.MQeTrace.setHandler(this); // Assign the trace handler
            MQeTraceRenderer renderer = new MQeTraceRenderer();

            baseHandler = new MQeTraceToReadable(System.out, renderer);
            baseHandler.setFilter(filter);
            activated = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Close and un-hook the Trace from MQe
     */
    public void close() {
        activated = false;
        if (com.ibm.mqe.MQeTrace.getHandler() == this) {
            com.ibm.mqe.MQeTrace.setHandler(null);
        }
    }

    /**
     * decide if the template is viewable or not
     */
    public boolean display(long groups) {
        boolean send = false;

        if ((filter & groups) > 0) {
            send = true;
        }

        return (send);
    }

    /**
     * Trace a message
     */
    public void traceMessage(Object selfReference, short messageNumber, long groups) {
        if (activated) {
            // check to see if it is displayable
            if (display(groups)) {
                baseHandler.traceMessage(selfReference, messageNumber, groups);
            }
        }
    }

    /**
     * Trace a message
     */
    public void traceMessage(Object selfReference, short messageNumber, long groups, Object insert1) {
        if (activated) {
            // check to see if it is displayable
            if (display(groups)) {
                baseHandler.traceMessage(selfReference, messageNumber, groups, insert1);
            }
        }
    }

    /**
     * Trace a message
     */
    public void traceMessage(
        Object selfReference,
        short messageNumber,
        long groups,
        Object insert1,
        Object insert2) {
        if (activated) {        	
            // check to see if it is displayable
            if (display(groups)) {
                baseHandler.traceMessage(selfReference, messageNumber, groups, insert1, insert2);
            }
        }
    }

    /**
     * Trace a message
     */
    public void traceMessage(
        Object selfReference,
        short messageNumber,
        long groups,
        Object insert1,
        Object insert2,
        Object insert3) {
        if (activated) {        	
            // check to see if it is displayable
            if (display(groups)) {
                baseHandler.traceMessage(selfReference, messageNumber, groups, insert1, insert2, insert3);
            }
        }
    }

    /**
     * Trace a message.
     */
    public void traceMessage(
        Object selfReference,
        short messageNumber,
        long groups,
        Object insert1,
        Object insert2,
        Object insert3,
        Object insert4) {
        if (activated) {        	
            // check to see if it is displayable
            if (display(groups)) {
                baseHandler.traceMessage(selfReference, messageNumber, groups, insert1, insert2, insert3, insert4);
            }
        }
    }
}

