// Version mqe_java/source/examples/trace/MQeTraceToFile.java, MQe, la200, la200-L030324.1 1.4

/* -----------------------------------------------------------------------------*/ 

/*                    MQSeries Everyplace for Multiplatforms                    */ 

/*                                                                              */ 

/* Licensed Materials - Property of IBM                                         */ 

/*                                                                              */ 

/* 5765-E63                                                                     */ 

/*                                                                              */ 

/* (C) Copyright IBM Corp. 1999                                                 */ 

/* -----------------------------------------------------------------------------*/ 
package examples.trace;


import  com.ibm.mqe.*;
import  com.ibm.mqe.trace.*;
import  java.text.*;
import  java.util.*;
import  java.io.*;


/**
 *
 * To trace a collection of trace messages to a file on disk.
 *
 * <p>Both the stdout and stderr streams are captured in the same file.</p>
 * 
 * <p>The file in which trace information is placed is dynamically named,
 * starting with the prefix <TT>mqe_trace_</TT> followed by the current time
 * expressed in milliseconds, followed by the <TT>.txt</TT> file extension.</p>
 *
 */
public class MQeTraceToFile extends examples.trace.MQeTrace {

    public static short version[] = { 2, 0, 0, 5 }; 
    public static final String outputFileNameStart = "mqe_trace_";
    public static final String outputFileNameEnd = ".txt";

    public synchronized void activate(String title) {

        super.activate(title);

        String outputFileName = outputFileNameStart + System.currentTimeMillis() + outputFileNameEnd;

        // Open the file we intend to write to...
        try {
            System.out.println("About to write trace output to " + outputFileName);
            logFile = new DataOutputStream(new FileOutputStream(new File(outputFileName)));
            ((MQeTraceToReadable) baseHandler).setOutputStream(new PrintStream(logFile));
        } catch (Exception e) {
            System.err.println("Failed to open log file " + outputFileName);
        }
    } 

    static DataOutputStream logFile = null;

    public synchronized void finalize() {
        if (logFile != null) {
            try {
                logFile.close(); 
                logFile = null; 
            } catch (Exception e) {
                System.err.println("Failed to close the trace file!" + e.toString());
            }
        }
    }

    public boolean display(long groups) {
        boolean display = super.display(groups);

        return display;
    }

}
